Package: pathview
Type: Package
Title: a tool set for pathway based data integration and visualization
Version: 1.52.0
Date: 2025-03-28
Author: Weijun Luo
Maintainer: Weijun Luo <luo_weijun@yahoo.com>
Description: Pathview is a tool set for pathway based data integration
        and visualization. It maps and renders a wide variety of
        biological data on relevant pathway graphs. All users need is
        to supply their data and specify the target pathway. Pathview
        automatically downloads the pathway graph data, parses the data
        file, maps user data to the pathway, and render pathway graph
        with the mapped data. In addition, Pathview also seamlessly
        integrates with pathway and gene set (enrichment) analysis
        tools for large-scale and fully automated analysis.
biocViews: Pathways, GraphAndNetwork, Visualization, GeneSetEnrichment,
        DifferentialExpression, GeneExpression, Microarray, RNASeq,
        Genetics, Metabolomics, Proteomics, SystemsBiology, Sequencing
Depends: R (>= 3.5.0)
Imports: KEGGgraph, XML, Rgraphviz, graph, png, AnnotationDbi,
        org.Hs.eg.db, KEGGREST, methods, utils
Suggests: gage, org.Mm.eg.db, RUnit, BiocGenerics
License: GPL (>=3.0)
URL: https://github.com/datapplab/pathview, https://pathview.uncc.edu/
LazyLoad: yes
LazyData: yes
Config/pak/sysreqs: make libpng-dev libxml2-dev libssl-dev zlib1g-dev
Repository: https://bioc-release.r-universe.dev
Date/Publication: 2026-04-28 12:37:26 UTC
RemoteUrl: https://github.com/bioc/pathview
RemoteRef: RELEASE_3_23
RemoteSha: 18f70b18cbfe57bec7ef629d6e21f63f155db044
NeedsCompilation: no
Packaged: 2026-04-29 13:51:38 UTC; root
Built: R 4.6.0; ; 2026-04-29 14:00:15 UTC; unix

import(
       KEGGgraph,
       Rgraphviz,
       KEGGREST,
       methods,
       graph
       )

importFrom(AnnotationDbi, mget, select, columns, keys)
importClassesFrom(AnnotationDbi,AnnotationDb)
importFrom(org.Hs.eg.db, org.Hs.eg.db)

importFrom(png, readPNG)
importFrom(utils, data, installed.packages, read.delim, globalVariables)
importFrom(XML, xmlAttrs, xmlChildren, xmlName, xmlRoot, xmlTreeParse)
importFrom(grDevices, col2rgb, dev.off, pdf, png, rgb)
importFrom(graphics, image, layout, par, polygon,
           rasterImage, rect, segments, text)
importFrom(stats, IQR, rnorm)

export(
       download.kegg,
       node.info,
       combineKEGGnodes, reaction2edge,

       node.map,
       eg2id, id2eg, geneannot.map,
       cpdkegg2name, cpdname2kegg, cpd2kegg, cpdidmap,
       kegg.species.code,
       mol.sum,
       sim.mol.data,

       node.color, col.key,
       
       wordwrap, strfit,

       pathview,
       keggview.native,
       keggview.graph
       )

# pathview R package

[![](https://img.shields.io/badge/release%20version-1.46.0-blue.svg)](https://www.bioconductor.org/packages/pathview)
[![](https://img.shields.io/badge/devel%20version-1.47.1-green.svg)](https://github.com/datapplab/pathview)
[![](https://img.shields.io/badge/BioC%20since-2013-blue.svg)](https://www.bioconductor.org/packages/pathview)
[![](https://img.shields.io/badge/GitHub%20since-2020-green.svg)](https://github.com/datapplab/pathview)

## Overview

Pathview is a leading tool for pathway based data integration and visualization. It maps, integrates and renders a wide variety of biological data on relevant pathway graphs. Pathview has 3 important features: 
* Interpretable graphs with publication quality: KEGG view for easy interpretation and Graphviz view for better graphical control. 
* Strong data integration capacity. It works with: 1) all data mappable to pathways, 2) 30 of molecular ID types (genes/protein, compound/metabolite), 3) 10000+ species, 4) various data attributes and formats. 
* Simple and powerful: fully automated and error-resistant, seamlessly integrates with a wide range of pathway and gene set (enrichment) analysis tools.


## Citation

Please cite the Pathview paper when using this open-source  package. This will help the project and our team:

Luo W, Brouwer C. Pathview: an R/Biocondutor package for pathway-based data integration and visualization. Bioinformatics, 2013, 29(14):1830-1831, <a href=https://doi.org/10.1093/bioinformatics/btt285>doi: 10.1093/bioinformatics/btt285</a>

## Installation (within R)

``` r
# install from BioConductor
if(!requireNamespace("BiocManager", quietly = TRUE))
    install.packages("BiocManager")
BiocManager::install("pathview")

# Or the development version from GitHub:
# install.packages("devtools")
devtools::install_github("datapplab/pathview")
```

## Quick start with demo data (R code)

Note Pathview focuses on KEGG pathways, which is good for most regular analyses. If you are interested in working with other major pathway databases, including Reactome, MetaCyc, SMPDB, PANTHER, METACROP etc, you can use [SBGNview](https://github.com/datapplab/SBGNview). Please check [the quick start  page](https://github.com/datapplab/SBGNview) and [the main tutorial](https://bioconductor.org/packages/devel/bioc/vignettes/SBGNview/inst/doc/SBGNview.Vignette.html) for details.

``` r
library(pathview)
data(gse16873.d)
pv.out <- pathview(gene.data = gse16873.d[, 1], pathway.id = "04110",
species = "hsa", out.suffix = "gse16873")
```

## More information

Please check the <a href=https://bioconductor.org/packages/pathview/>BioC page</a> for tutorials and extra documentations. 

Also see the <a href=https://pathview.uncc.edu/>Pathview Web server</a> for interactive GUI with example graphics.

Thank you for your interest.

### R code from vignette source 'pathview.Rnw'

###################################################
### code chunk number 1: synopsis1 (eval = FALSE)
###################################################
## library(pathview)
## data(gse16873.d)
## pv.out <- pathview(gene.data = gse16873.d[, 1], pathway.id = "04110",
##                    species = "hsa", out.suffix = "gse16873")


###################################################
### code chunk number 2: install (eval = FALSE)
###################################################
## source("http://bioconductor.org/biocLite.R")
## biocLite("pathview")


###################################################
### code chunk number 3: <install (eval = FALSE)
###################################################
## source("http://bioconductor.org/biocLite.R")
## biocLite(c("Rgraphviz", "png", "KEGGgraph", "org.Hs.eg.db"))


###################################################
### code chunk number 4: install (eval = FALSE)
###################################################
## install.packages("pathview",repos="http://R-Forge.R-project.org")


###################################################
### code chunk number 5: install (eval = FALSE)
###################################################
## install.packages("/your/local/directory/pathview_1.0.0.tar.gz", 
##     repos = NULL, type = "source")


###################################################
### code chunk number 6: <install (eval = FALSE)
###################################################
## install.packages("/your/local/directory/XML_3.95-0.2.zip", repos = NULL)


###################################################
### code chunk number 7: start
###################################################
options(width=80)


###################################################
### code chunk number 8: start
###################################################
library(pathview)


###################################################
### code chunk number 9: start (eval = FALSE)
###################################################
## library(help=pathview)


###################################################
### code chunk number 10: start (eval = FALSE)
###################################################
## help(pathview)
## ?pathview


###################################################
### code chunk number 11: dataPrep.gse16873
###################################################
data(gse16873.d)


###################################################
### code chunk number 12: readin (eval = FALSE)
###################################################
## filename=system.file("extdata/gse16873.demo", package = "pathview")
## gse16873=read.delim(filename, row.names=1)
## gse16873.d=gse16873[,2*(1:6)]-gse16873[,2*(1:6)-1]


###################################################
### code chunk number 13: dataPrep.demo.paths
###################################################
data(demo.paths)


###################################################
### code chunk number 14: dataPreppaths.hsa
###################################################
data(paths.hsa)
head(paths.hsa,3)


###################################################
### code chunk number 15: kegg.native
###################################################
i <- 1
pv.out <- pathview(gene.data = gse16873.d[, 1], pathway.id = demo.paths$sel.paths[i],
                   species = "hsa", out.suffix = "gse16873", kegg.native = T)
list.files(pattern="hsa04110", full.names=T)
str(pv.out)
head(pv.out$plot.data.gene)


###################################################
### code chunk number 16: kegg.native_2layer
###################################################
pv.out <- pathview(gene.data = gse16873.d[, 1], pathway.id = demo.paths$sel.paths[i],
                   species = "hsa", out.suffix = "gse16873.2layer", kegg.native = T,
                   same.layer = F)


###################################################
### code chunk number 17: graphviz
###################################################
pv.out <- pathview(gene.data = gse16873.d[, 1], pathway.id = demo.paths$sel.paths[i],
                   species = "hsa", out.suffix = "gse16873", kegg.native = F,
                   sign.pos = demo.paths$spos[i])
#pv.out remains the same
dim(pv.out$plot.data.gene)
head(pv.out$plot.data.gene)


###################################################
### code chunk number 18: graphviz.2layer
###################################################
pv.out <- pathview(gene.data = gse16873.d[, 1], pathway.id = demo.paths$sel.paths[i], 
    species = "hsa", out.suffix = "gse16873.2layer", kegg.native = F, 
    sign.pos = demo.paths$spos[i], same.layer = F)


###################################################
### code chunk number 19: split
###################################################
pv.out <- pathview(gene.data = gse16873.d[, 1], pathway.id = demo.paths$sel.paths[i], 
    species = "hsa", out.suffix = "gse16873.split", kegg.native = F, 
    sign.pos = demo.paths$spos[i], split.group = T)
dim(pv.out$plot.data.gene)
head(pv.out$plot.data.gene)


###################################################
### code chunk number 20: expanded
###################################################
pv.out <- pathview(gene.data = gse16873.d[, 1], pathway.id = demo.paths$sel.paths[i], 
    species = "hsa", out.suffix = "gse16873.split.expanded", kegg.native = F,
    sign.pos = demo.paths$spos[i], split.group = T, expand.node = T)
dim(pv.out$plot.data.gene)
head(pv.out$plot.data.gene)


###################################################
### code chunk number 21: dataPrep.sim.cpd
###################################################
sim.cpd.data=sim.mol.data(mol.type="cpd", nmol=3000)
data(cpd.simtypes)


###################################################
### code chunk number 22: gene_cpd.data
###################################################
i <- 3
print(demo.paths$sel.paths[i])
pv.out <- pathview(gene.data = gse16873.d[, 1], cpd.data = sim.cpd.data, 
    pathway.id = demo.paths$sel.paths[i], species = "hsa", out.suffix = "gse16873.cpd", 
    keys.align = "y", kegg.native = T, key.pos = demo.paths$kpos1[i])
str(pv.out)
head(pv.out$plot.data.cpd)


###################################################
### code chunk number 23: graphviz.gene_cpd.data
###################################################
pv.out <- pathview(gene.data = gse16873.d[, 1], cpd.data = sim.cpd.data, 
    pathway.id = demo.paths$sel.paths[i], species = "hsa", out.suffix = "gse16873.cpd", 
    keys.align = "y", kegg.native = F, key.pos = demo.paths$kpos2[i], 
    sign.pos = demo.paths$spos[i], cpd.lab.offset = demo.paths$offs[i])


###################################################
### code chunk number 24: sim.cpd.data2
###################################################
set.seed(10)
sim.cpd.data2 = matrix(sample(sim.cpd.data, 18000, 
    replace = T), ncol = 6)
rownames(sim.cpd.data2) = names(sim.cpd.data)
colnames(sim.cpd.data2) = paste("exp", 1:6, sep = "")
head(sim.cpd.data2, 3)


###################################################
### code chunk number 25: multisample.gene_cpd.data
###################################################
#KEGG view
pv.out <- pathview(gene.data = gse16873.d[, 1:3], 
    cpd.data = sim.cpd.data2[, 1:2], pathway.id = demo.paths$sel.paths[i], 
    species = "hsa", out.suffix = "gse16873.cpd.3-2s", keys.align = "y", 
    kegg.native = T, match.data = F, multi.state = T, same.layer = T)
head(pv.out$plot.data.cpd)
#KEGG view with data match
pv.out <- pathview(gene.data = gse16873.d[, 1:3], 
    cpd.data = sim.cpd.data2[, 1:2], pathway.id = demo.paths$sel.paths[i], 
    species = "hsa", out.suffix = "gse16873.cpd.3-2s.match", 
    keys.align = "y", kegg.native = T, match.data = T, multi.state = T, 
    same.layer = T)
#graphviz view
pv.out <- pathview(gene.data = gse16873.d[, 1:3], 
    cpd.data = sim.cpd.data2[, 1:2], pathway.id = demo.paths$sel.paths[i], 
    species = "hsa", out.suffix = "gse16873.cpd.3-2s", keys.align = "y", 
    kegg.native = F, match.data = F, multi.state = T, same.layer = T,
    key.pos = demo.paths$kpos2[i], sign.pos = demo.paths$spos[i])


###################################################
### code chunk number 26: multisample.gene_cpd.data.seperate (eval = FALSE)
###################################################
## #plot samples/states separately
## pv.out <- pathview(gene.data = gse16873.d[, 1:3], 
##     cpd.data = sim.cpd.data2[, 1:2], pathway.id = demo.paths$sel.paths[i], 
##     species = "hsa", out.suffix = "gse16873.cpd.3-2s", keys.align = "y", 
##     kegg.native = T, match.data = F, multi.state = F, same.layer = T)


###################################################
### code chunk number 27: multisample.gene_cpd.data.2layer (eval = FALSE)
###################################################
## pv.out <- pathview(gene.data = gse16873.d[, 1:3], 
##     cpd.data = sim.cpd.data2[, 1:2], pathway.id = demo.paths$sel.paths[i], 
##     species = "hsa", out.suffix = "gse16873.cpd.3-2s.2layer", 
##     keys.align = "y", kegg.native = T, match.data = F, multi.state = T, 
##     same.layer = F)


###################################################
### code chunk number 28: discrete.gene_cpd.data
###################################################
require(org.Hs.eg.db)
gse16873.t <- apply(gse16873.d, 1, function(x) t.test(x, 
    alternative = "two.sided")$p.value)
sel.genes <- names(gse16873.t)[gse16873.t < 0.1]
sel.cpds <- names(sim.cpd.data)[abs(sim.cpd.data) > 0.5]
pv.out <- pathview(gene.data = sel.genes, cpd.data = sel.cpds, 
    pathway.id = demo.paths$sel.paths[i], species = "hsa", out.suffix = "sel.genes.sel.cpd", 
    keys.align = "y", kegg.native = T, key.pos = demo.paths$kpos1[i], 
    limit = list(gene = 5, cpd = 2), bins = list(gene = 5, cpd = 2), 
    na.col = "gray", discrete = list(gene = T, cpd = T))
pv.out <- pathview(gene.data = sel.genes, cpd.data = sim.cpd.data, 
    pathway.id = demo.paths$sel.paths[i], species = "hsa", out.suffix = "sel.genes.cpd",
    keys.align = "y", kegg.native = T, key.pos = demo.paths$kpos1[i], 
    limit = list(gene = 5, cpd = 1), bins = list(gene = 5, cpd = 10), 
    na.col = "gray", discrete = list(gene = T, cpd = F))


###################################################
### code chunk number 29: gene.ensprot_cpd.cas
###################################################
cpd.cas <- sim.mol.data(mol.type = "cpd", id.type = cpd.simtypes[2], 
    nmol = 10000)
gene.ensprot <- sim.mol.data(mol.type = "gene", id.type = gene.idtype.list[4], 
    nmol = 50000)
pv.out <- pathview(gene.data = gene.ensprot, cpd.data = cpd.cas, 
    gene.idtype = gene.idtype.list[4], cpd.idtype = cpd.simtypes[2], 
    pathway.id = demo.paths$sel.paths[i], species = "hsa", same.layer = T, 
    out.suffix = "gene.ensprot.cpd.cas", keys.align = "y", kegg.native = T, 
    key.pos = demo.paths$kpos2[i], sign.pos = demo.paths$spos[i], 
    limit = list(gene = 3, cpd = 3), bins = list(gene = 6, cpd = 6))


###################################################
### code chunk number 30: gene.ensprot_cpd.cas.manual.map
###################################################
id.map.cas <- cpdidmap(in.ids = names(cpd.cas), in.type = cpd.simtypes[2], 
    out.type = "KEGG COMPOUND accession")
cpd.kc <- mol.sum(mol.data = cpd.cas, id.map = id.map.cas)
id.map.ensprot <- id2eg(ids = names(gene.ensprot), 
    category = gene.idtype.list[4], org = "Hs")
gene.entrez <- mol.sum(mol.data = gene.ensprot, id.map = id.map.ensprot)
pv.out <- pathview(gene.data = gene.entrez, cpd.data = cpd.kc, 
    pathway.id = demo.paths$sel.paths[i], species = "hsa", same.layer = T, 
    out.suffix = "gene.entrez.cpd.kc", keys.align = "y", kegg.native = T, 
    key.pos = demo.paths$kpos2[i], sign.pos = demo.paths$spos[i], 
    limit = list(gene = 3, cpd = 3), bins = list(gene = 6, cpd = 6))


###################################################
### code chunk number 31: korg
###################################################
data(korg)
head(korg)
#number of species which use Entrez Gene as the default ID
sum(korg[,"entrez.gnodes"]=="1",na.rm=T)
#number of species which use other ID types or none as the default ID
sum(korg[,"entrez.gnodes"]=="0",na.rm=T)
#new from 2017: most species which do not have Entrez Gene annotation any more
na.idx=is.na(korg[,"ncbi.geneid"])
sum(na.idx)


###################################################
### code chunk number 32: bods_gene.idtype.list
###################################################
data(bods)
bods
data(gene.idtype.list)
gene.idtype.list


###################################################
### code chunk number 33: eco.dat.kegg
###################################################
eco.dat.kegg <- sim.mol.data(mol.type="gene",id.type="kegg",species="eco",nmol=3000)
head(eco.dat.kegg)
pv.out <- pathview(gene.data = eco.dat.kegg, gene.idtype="kegg",
    pathway.id = "00640", species = "eco", out.suffix = "eco.kegg",
    kegg.native = T, same.layer=T)


###################################################
### code chunk number 34: eco.dat.entrez (eval = FALSE)
###################################################
## eco.dat.entrez <- sim.mol.data(mol.type="gene",id.type="entrez",species="eco",nmol=3000)
## head(eco.dat.entrez)
## pv.out <- pathview(gene.data = eco.dat.entrez, gene.idtype="entrez",
##     pathway.id = "00640", species = "eco", out.suffix = "eco.entrez",
##     kegg.native = T, same.layer=T)


###################################################
### code chunk number 35: eco.dat.symbol (eval = FALSE)
###################################################
## egid.eco=eg2id(names(eco.dat.entrez), category="symbol", pkg="org.EcK12.eg.db")
## eco.dat.symbol <- eco.dat.entrez
## names(eco.dat.symbol) <- egid.eco[,2]
## head(eco.dat.kegg)
## pv.out <- pathview(gene.data = eco.dat.symbol, gene.idtype="symbol",
##     pathway.id = "00640", species = "eco", out.suffix = "eco.symbol.2layer",
##     kegg.native = T, same.layer=F)


###################################################
### code chunk number 36: gene.ensprot_cpd.cas.manual.map
###################################################
ko.data=sim.mol.data(mol.type="gene.ko", nmol=5000)
pv.out <- pathview(gene.data = ko.data, pathway.id = "04112",
                   species = "ko", out.suffix = "ko.data", kegg.native = T)


###################################################
### code chunk number 37: GAGE.Pathview.pipeline (eval = FALSE)
###################################################
## library(gage)
## data(gse16873)
## cn <- colnames(gse16873)
## hn <- grep('HN',cn, ignore.case =TRUE)
## dcis <- grep('DCIS',cn, ignore.case =TRUE)
## data(kegg.gs)
## #pathway analysis using gage
## gse16873.kegg.p <- gage(gse16873, gsets = kegg.gs,
##     ref = hn, samp = dcis)
## #prepare the differential expression data
## gse16873.d <- gagePrep(gse16873, ref = hn, samp = dcis)
## #equivalently, you can do simple subtraction for paired samples
## gse16873.d <- gse16873[,dcis]-gse16873[,hn]
## #select significant pathways and extract their IDs
## sel <- gse16873.kegg.p$greater[, "q.val"] < 0.1 & !is.na(gse16873.kegg.p$greater[, 
##     "q.val"])
## path.ids <- rownames(gse16873.kegg.p$greater)[sel]
## path.ids2 <- substr(path.ids[c(1, 2, 7)], 1, 8)
## #pathview visualization
## pv.out.list <- pathview(gene.data = gse16873.d[, 1:3], pathway.id = path.ids2, 
##     species = "hsa")

parseReaction2 <-
function (reaction)
  {
    attrs <- xmlAttrs(reaction)
    name <- attrs[["name"]]
    type <- attrs[["type"]]
    children <- xmlChildren(reaction)
    childrenNames <- names(children)
    substrateIndices <- grep("^substrate$", childrenNames)
    productIndices <- grep("^product$", childrenNames)
    substrateName <- substrateAltName <- vector("character",
                                                length(substrateIndices))
    productName <- productAltName <- vector("character", length(productIndices))
    for (i in seq(along = substrateIndices)) {
      ind <- substrateIndices[i]
      substrate <- children[[ind]]
      substrateName[i] <- xmlAttrs(substrate)[["id"]]
      substrateChildren <- xmlChildren(substrate)
      if (length(substrateChildren) > 0) {
        substrateAlt <- substrateChildren$alt
        substrateAltName[i] <- xmlAttrs(substrateAlt)[["name"]]
      }
      else {
        substrateAlt <- as.character(NA)
        substrateAltName[i] <- as.character(NA)
      }
    }
    for (i in seq(along = productIndices)) {
      ind <- productIndices[i]
      product <- children[[ind]]
      productName[i] <- xmlAttrs(product)[["id"]]
      productChildren <- xmlChildren(product)
      if (length(productChildren) > 0) {
        productAlt <- productChildren$alt
        productAltName[i] <- xmlAttrs(productAlt)[["name"]]
      }
      else {
        productAlt <- as.character(NA)
        productAltName[i] <- as.character(NA)
      }
    }
    new("KEGGReaction", name = name, type = type, substrateName = substrateName,
        substrateAltName = substrateAltName, productName = productName,
        productAltName = productAltName)
  }

cpdkegg2name <-
function(in.ids, in.type=c("KEGG", "KEGG COMPOUND accession")[1]){
#kegg accession to cpd name
  cnames=c(in.type, "NAME")
  in.type=tolower(in.type)
#  out.type=tolower(out.type)
  data(rn.list)
  cpd.type=names(rn.list)=tolower(names(rn.list))
    cpd.type=c(cpd.type,"kegg")
    kg.type=cpd.type[grep("kegg", cpd.type)]
    if(!in.type %in% kg.type) stop("Incorrect type!")
    in.type=gsub(" accession", "", in.type)

  data(cpd.names)
    if(in.type=="kegg") sel.rn=1:nrow(cpd.names)
  else sel.rn=cpd.names$SOURCE==in.type
    sel.cn=c("ACCESSION_NUMBER", "NAME")
      cpd.names=as.matrix(cpd.names[sel.rn, sel.cn])
    rownames(cpd.names)=NULL
  data(kegg.met)
    cpd.names=as.data.frame(rbind(kegg.met[,1:2],cpd.names))
  colnames(cpd.names)=sel.cn    #@

#  in.ids=unique(in.ids)
  len.id=length(in.ids)
      out.names=in.ids
#  out.names=rep(NA, len.id)

  in.idx=in.ids  %in% cpd.names$ACCESSION_NUMBER
  if(sum(in.idx)<1) {
    message("Note: ", "None of the compound ids mapped to the specified type!")
  } else{
    out.names[in.idx]=as.character(cpd.names$NAME[match(in.ids[in.idx], cpd.names$ACCESSION_NUMBER)])
  }
  out.names=cbind(in.ids, out.names)
  colnames(out.names)=cnames
  return(out.names)
}

col.key <-
  function(discrete=FALSE, limit=1.0, bins=10, cols=NULL, both.dirs=TRUE, low="green", mid="gray", high="red", graph.size, node.size, size.by.graph=TRUE, key.pos="topright", off.sets=c(x=0,y=0), align="n",  cex=1, lwd=1){

  if(both.dirs & length(limit)==1){
    limit=c(-abs(limit), abs(limit))
  } else if(length(limit)==1){
    limit=c(0,limit)
  }
  disc.cond1=all(as.integer(limit)==limit)
  disc.cond2=(limit[2]-limit[1]) %% bins==0
  discrete=discrete & disc.cond1 & disc.cond2
  if(discrete){
    limit[2]=limit[2]+1
    bins=bins+1
  }

  width=graph.size[1]
  height=graph.size[2]
  if(size.by.graph==T){
    xs=width/80
    ys=height/40
  } else if(!missing(node.sizes)){
    xs=node.size[1]*3/bins
    ys=node.size[2]
  } else{
    message("Note: ", "color key not plotted, node.size is needed\n when size.by.graph=FALSE!")
    return(off.sets)
  }
  
  if(align=="x") {
    off.sets['x']=2*xs
    off.sets['y']=off.sets['y']+3*ys
  }
  if(align=="y")   off.sets=off.sets+c(x=3*xs, y=0)
  if(align=="n")   off.sets=off.sets+c(x=2*xs, y=2*ys)
  if(length(grep('right',key.pos))==1) {
    off.sets['x']=off.sets['x']+bins*xs
    x=width-off.sets['x']
  } else {
    x=off.sets['x']
    off.sets['x']=off.sets['x']+bins*xs
  }
  if(length(grep('top',key.pos))==1) y=height-off.sets['y'] else y=off.sets['y']

  ckx=seq(x,x+bins*xs,length=bins+1)
  cky=c(y,y+ys)

  if(is.null(cols)){
     if(both.dirs){
    cols = colorpanel2(bins, low=low, mid=mid, high=high)
  } else cols = colorpanel2(bins, low=mid, high=high)
   }

  data.cuts = seq(from = limit[1], to =limit[2], length=bins+1)
  image(x=ckx, y=cky, z=cbind(data.cuts[-1]),col = cols, axes = FALSE, add=T)
if(!discrete){
  label=format(data.cuts[c(1,bins/2+1,bins+1)], digits=2)
  text(x=seq(x,x+bins*xs,length=length(label)), y=rep(y-ys, length(label)),label=label, cex=cex)
  } else{
  label=paste(as.integer(data.cuts[c(1,bins)]))
  text(x=seq(x,x+bins*xs,length=length(label))+c(xs,-xs)/2, y=rep(y-ys, length(label)),label=label, cex=cex)
  }
  cky=c(y-0.25*ys,y+ys)
  for(i in 1:(bins+1)) lines(rep(ckx[i],2), cky, lwd=lwd)

  return(off.sets)
}

colorpanel2<-
  function (n, low, mid, high)
  {
    if (missing(mid) || missing(high)) {
      low <- col2rgb(low)
      if (missing(high))
        high <- col2rgb(mid)
      else high <- col2rgb(high)
      red <- seq(low[1, 1], high[1, 1], length = n)/255
      green <- seq(low[3, 1], high[3, 1], length = n)/255
      blue <- seq(low[2, 1], high[2, 1], length = n)/255
    }
    else {
      isodd <- n%%2 == 1
      if (isodd) {
        n <- n + 1
      }
      low <- col2rgb(low)
      mid <- col2rgb(mid)
      high <- col2rgb(high)
      lower <- floor(n/2)
      upper <- n - lower
      red <- c(seq(low[1, 1], mid[1, 1], length = lower),
               seq(mid[1, 1], high[1, 1], length = upper))/255
      green <- c(seq(low[3, 1], mid[3, 1], length = lower),
                 seq(mid[3, 1], high[3, 1], length = upper))/255
      blue <- c(seq(low[2, 1], mid[2, 1], length = lower),
                seq(mid[2, 1], high[2, 1], length = upper))/255
      if (isodd) {
        red <- red[-(lower + 1)]
        green <- green[-(lower + 1)]
        blue <- blue[-(lower + 1)]
      }
    }
    rgb(red, blue, green)
  }

node.map <-
function(mol.data=NULL, node.data, node.types=c("gene", "ortholog", "compound")[1], node.sum =c("sum","mean", "median", "max", "max.abs", "random")[1], entrez.gnodes=TRUE){
type.sel=node.data$type %in% node.types
if(sum(type.sel)<1){
  message("Note: ", "No specified node types in the pathway!")
  plot.data=NULL
  return(plot.data)
}
node.data=lapply(node.data, "[", type.sel)
n.nodes=length(node.data$kegg.names)
spacials=as.matrix(as.data.frame(node.data[c("type", "x", "y", "width", "height")]))
if(node.types[1]=="gene"){
kng=node.data$kegg.names[node.data$type=="gene"]
kng.char=gsub("[0-9]", "", unlist(kng))
if(any(kng.char>"")) entrez.gnodes=FALSE
}

na.plot.data=function(){
    sapply(1:n.nodes, function(i){
      kns=node.data$kegg.names[[i]]
    if(node.types[1]=="gene" & entrez.gnodes) items=as.numeric(kns)
    else items=kns
    ord=order(items)
    items=items[ord]
     kns=kns[ord]
      return(c(kns[1],"", spacials[i,], NA))
    })
  }

if(is.null(mol.data)){
  plot.data=na.plot.data()
} else{
  
#map gene data  
    if(is.character(mol.data)){
      gd.names=mol.data
      mol.data=rep(1, length(mol.data))
      names(mol.data)=gd.names
    }
    mol.data=cbind(mol.data)

    if(is.null(colnames(mol.data))) colnames(mol.data)=paste("ge", 1:ncol(mol.data),sep="")
    mapped.mols <- intersect(unlist(node.data$kegg.names), row.names(mol.data))
    if(length(mapped.mols)==0){
      message("Warning: ", paste("None of the genes or compounds mapped to the pathway!",
                    "Argument gene.idtype or cpd.idtype may be wrong.", sep="\n"))
      plot.data=na.plot.data()
    } else{
      if(node.types[1]=="gene" & entrez.gnodes) mapped.mols =as.numeric(mapped.mols)


      plot.data=sapply(1:n.nodes, function(i){
        kns=node.data$kegg.names[[i]]
        if(node.types[1]=="gene" & entrez.gnodes) items=as.numeric(kns)
        else items=kns
        ord=order(items)
        items=items[ord]
        kns=kns[ord]
        hit=items %in% mapped.mols 
        if(sum(hit)==0) {
          return(c(kns[1], "", spacials[i,], rep(NA, ncol(mol.data))))
        } else if(sum(hit)==1) {
          edata=mol.data[as.character(items[hit]),]
          return(c(kns[hit], kns[hit], spacials[i,], edata))
        } else {
          node.sum=eval(as.name(node.sum))
                                        #      edata=apply(cbind(mol.data[as.character(items[hit]),]), 2, node.sum, na.rm=T)
#          edata=apply(cbind(mol.data[as.character(items[hit]),]), 2, function(x){
          edata=apply(mol.data[as.character(items[hit]),,drop=F], 2, function(x){
            x=x[!is.na(x)]
            if(length(x)<1) return(NA)
            else return(node.sum(x, na.rm=F))
          })
          return(c(kns[hit][1], paste(kns[hit],collapse=","), spacials[i,], edata))
        }    
      })
    }
  }

colnames(plot.data)=names(node.data$kegg.names)
plot.data=as.data.frame(t(plot.data), stringsAsFactors = F)
  plot.data$labels=node.data$labels
  ncs=ncol(plot.data)
  plot.data=plot.data[,c(1,ncs,2:(ncs-1))]
if(is.null(mol.data)) cns="mol.data" else cns=colnames(mol.data)
colnames(plot.data)[c(1,3,9:ncs)]=c("kegg.names","all.mapped",cns)#c(1,8:ncs)
for(ic in (1:ncol(plot.data))[-c(1:4)]) plot.data[,ic]=as.numeric(plot.data[,ic])#-c(1:3)

return(plot.data)
}

.onLoad <- function(libname, pkgname) {
if(FALSE){
  pnames=rownames(installed.packages())
  if("pathview" %in% pnames){
    data(gene.idtype.list, package ="pathview")
    data(gene.idtype.bods, package ="pathview")
    data(cpd.simtypes, package ="pathview")
    data(korg, package ="pathview")
#    korg=try(read.delim(file="https://pathview.uncc.edu/data/korg.tsv", sep="\t"))
#    if(class(korg)[1]=="data.frame"){
#    korg$ncbi.geneid=as.character(korg$ncbi.geneid)
#    korg=as.matrix(korg)
#  } else data(korg, package ="pathview")
  }
}
disclaimer="##############################################################################\nPathview is an open source software package distributed under GNU General Public License version 3 (GPLv3). Details of GPLv3 is available at http://www.gnu.org/licenses/gpl-3.0.html. Particullary, users are required to formally cite the original Pathview paper (not just mention it) in publications or products. For details, do citation(\"pathview\") within R. \n\nThe pathview downloads and uses KEGG data. Non-academic uses may require a KEGG license agreement (details at http://www.kegg.jp/kegg/legal.html).\n##############################################################################\n\n"
packageStartupMessage(wordwrap(disclaimer, 80))
}

strfit <-
function(s, width=20){
  spaces=c(" ", "\t", "\n")
  s=gsub("([ \t\n])+"," ", s) #  s=gsub("([ \t\n])+","\\1", s)
  sv=strsplit(s, "")[[1]]
  len.s=length(sv)
  passed=NULL

  while(len.s>0){
    if (length(sv)<width+3) {
      passed=c(passed, s)
      break
    }
    
    start.pos=1
    if(sv[width] %in% spaces){
      stop.pos=width-1
      start.next=width+1
      postpend=""
    } else if(sv[width+1] %in% spaces){
      stop.pos=width
      start.next=width+2
      postpend=""
    } else if(sv[width+2] %in% spaces){
      stop.pos=width+1
      start.next=width+3
      postpend=""
    } else if(sv[width-1] %in% spaces){
      stop.pos=width-2
      start.next=width
      postpend=""
    } else if(sv[width-2] %in% spaces){
      stop.pos=width-3
      start.next=width-1
      postpend="\\"
    } else{
      stop.pos=width
      start.next=width+1
      postpend="\\"
    }

    passed=c(passed, paste(substr(s, start.pos, stop.pos), postpend, sep=""))
    start.pos=start.next
    s=substr(s, start.pos, length(sv))
    sv=sv[-c(1:(start.pos-1))]
    len.s=length(sv)
  }
  passed=paste(passed, collapse="\n")
  return(passed)    
}

eg2id <-function(eg, category=gene.idtype.list[1:2], org="Hs", pkg.name=NULL, ...){
  category=tolower(category)
  ei=category %in% c("entrez", "eg", "entrezid")
  if(all(ei)) stop("output ID or category cannot all be Entrez Gene ID!")
  if(any(ei)) category=category[!ei]
  geneannot.map(in.ids=eg, in.type="entrez", out.type=category, org=org, pkg.name=pkg.name, ...)#, unique.map=TRUE, na.rm=TRUE)
}

test_id2eg <- function() {
  symbs.good=c("A1BG","SERPINA3","PLIN2")
  egs.good=c("1","12","123")

  egs.mapped1=id2eg(symbs.good, category="SYMBOL", org = "Hs")[,2]
  checkEquals(egs.mapped1, egs.good)

  egs.mapped2=id2eg(c(egs.good,symbs.good), category="SYMBOL", org = "Hs")[,2]
  checkEquals(egs.mapped2, c(rep(NA,3), egs.good))

  checkException(id2eg(symbs.good, category="SYMBOL", org = "Hss"))
  checkException(id2eg(symbs.good, category="SYMBL", org = "Hs"))
}

test_eg2id <- function() {
  symbs.good=c("A1BG","SERPINA3","PLIN2")
  egs.good=c("1","12","123")

  symbs.mapped1=eg2id(egs.good, category="SYMBOL", org = "Hs")[,2]
  checkEquals(symbs.mapped1, symbs.good)

  symbs.mapped2=eg2id(c(egs.good,symbs.good), category="SYMBOL", org = "Hs")[,2]
  checkEquals(symbs.mapped2, c(symbs.good,rep(NA,3)))

  checkException(eg2id(egs.good, category="SYMBOL", org = "Hss"))
  checkException(eg2id(egs.good, category="SYMBL", org = "Hs"))
}

test_cpdidmap <- function() {
  cpd.cas=c("49620-06-6", "104206-65-7", "507-60-8", "7235-40-7",
    "17440-83-4")

#map and reverse map being consistent  
  id.map.cas <- cpdidmap(in.ids = cpd.cas, in.type = "CAS Registry Number",
                         out.type = "KEGG COMPOUND accession")
  id.map.cas2 <- cpdidmap(in.ids = id.map.cas[,2],
                          in.type = "KEGG COMPOUND accession",
                          out.type = "CAS Registry Number")
  na.idx=is.na(id.map.cas[,2])
  checkEquals(id.map.cas2[,2], cpd.cas[!na.idx])

#in.ids and in.type not matching  
  id.map.cas3 <- cpdidmap(in.ids = cpd.cas, in.type = "KEGG COMPOUND accession",
                          out.type = "CAS Registry Number")
  checkTrue(all(is.na(id.map.cas3[,2])))

#in.type and out.type are similar, no need to map  
  id.map.cas4<-cpdidmap(in.ids = cpd.cas, in.type = "KEGG COMPOUND accession",
                        out.type = "kegg")
  checkEquals(id.map.cas4, 0)

#wrong in.type, error  
  checkException(cpdidmap(in.ids = cpd.cas, in.type = "Unknown accession",
                          out.type = "kegg"))
}

keggview.graph <-function(
                           plot.data.gene=NULL,
                           plot.data.cpd=NULL,
                           cols.ts.gene=NULL,
                           cols.ts.cpd=NULL,
                           node.data,
                           path.graph,
                           pathway.name,
                           out.suffix="pathview",
                           pdf.size=c(7,7),

                          multi.state=TRUE,
                          same.layer=TRUE,
                          match.data=TRUE,
                           rankdir=c("LR","TB")[1],
                           is.signal=TRUE,
                           split.group=F,
                           afactor=1,

                           text.width=15, #k
                           cex=0.5,
                           map.cpdname=FALSE, #k
                           cpd.lab.offset=1.0,


                          discrete=list(gene=FALSE, cpd=FALSE),
                          limit=list(gene=1, cpd=1),
                         bins=list(gene=10, cpd=10),
                         both.dirs=list(gene=T, cpd=T),
                          low = list(gene = "green", cpd = "blue"),
                         mid = list(gene = "gray", cpd = "gray"),
                         high = list(gene = "red", cpd = "yellow"),
                           na.col="transparent",
         
                         new.signature=TRUE,
                         plot.col.key=TRUE,
                         key.align="x",
                         key.pos="topright",
                         sign.pos="bottomright",#g
                           ...){
  
  gR1=path.graph

                                        #group nodes mapping and merge
  grp.idx=node.data$size>1
  if(sum(grp.idx)>0 & !split.group){
    sub2grp=cbind(unlist(node.data$component[grp.idx], use.names=F), rep(names(grp.idx)[grp.idx], node.data$size[grp.idx]))
    du.idx=duplicated(sub2grp[,1])
    if(sum(du.idx)>0){
      du.rn=sub2grp[,1] %in% sub2grp[du.idx,1]
      message("Warning: reconcile groups sharing member nodes!")
      print(sub2grp[du.rn,])
      du.grps=sub2grp[du.idx,]
      rn=which(du.idx)
      for(r in rn){
        comps=node.data$component[[sub2grp[r,2]]]
        comps=comps[comps!=sub2grp[r,1]]
        node.data$component[[sub2grp[r,2]]]=comps
        node.data$size[sub2grp[r,2]]=node.data$size[sub2grp[r,2]]-1
      }
      sub2grp=sub2grp[!du.idx,]
    }
    rownames(sub2grp)=sub2grp[,1]
  } else sub2grp=NULL
  
  if(sum(grp.idx)>0 & !split.group){
    for(gn in names(grp.idx)[grp.idx]){
      gR1=combineKEGGnodes(node.data$component[[gn]], gR1, gn)
    }
  } else if(split.group){
    gR1=subGraph(nodes(gR1)[node.data$size==1], gR1)
  }
  nNames=nodes(gR1)
  nSizes=node.data$size[nNames]

                                        #unconnected nodes processing
  deg=degree(gR1)
  deg=deg$inDegree+deg$outDegree
  if(is.signal & sum(deg<1)>0){
    gR2=subKEGGgraph(nNames[deg>0], gR1)
    nNames=nNames[deg>0]
    nSizes=nSizes[deg>0]
    if(!is.null(sub2grp)){
                                        #  sub.idx=!sub2grp[,1] %in% names(deg[deg<1])
      sub.idx=sub2grp[,1] %in% nNames |sub2grp[,2] %in% nNames
    } else sub.idx=0
  } else {
    gR2=gR1
    if(!is.null(sub2grp)){
      sub.idx=rep(T, nrow(sub2grp))
    } else sub.idx=0
  }

  if(length(nNames)<2){
        msg=sprintf("%s not rendered, 0 or 1 connected nodes!\nTry \"kegg.native=T\" instead!", pathway.name)
            message("Note: ", msg)
            return(list())
      }
  
                                        #give up the KEGG positions, use graphviz layout


                                        #general attributes
  attrs=list()
  attrs$graph$rankdir="LR"
  attrs$node <- list(fixedsize=FALSE)

                                        #node attributes
  ntype=node.data$type[nNames]
  cpd.idx=ntype=="compound"
  map.idx=ntype=="map"
  rect.idx=!(cpd.idx|map.idx)
  nAttr=list()
  nAttr$label=rep('', length(nNames))
                                        #nAttr$shape=rep('rectangle', length(nNames))
  shapes=node.data$shape[nNames]
  if(any(cpd.idx)) shapes[cpd.idx]="ellipse"
  if(any(map.idx)) shapes[map.idx]="plaintext"
                                        #if(any(map.idx)) shapes[map.idx]="rectangle"
  nAttr$shape=shapes
  nAttr$height=.75*17/46*nSizes*afactor
  nAttr$width=rep(.75, length(nNames))*afactor
  if(any(cpd.idx)){
    nAttr$height[cpd.idx]=nAttr$height[cpd.idx]*1.5
    nAttr$width[cpd.idx]=nAttr$width[cpd.idx]*1.5
  }
  if(any(map.idx)){
    nAttr$height[map.idx]=nAttr$height[map.idx]*1.5
    nAttr$width[map.idx]=nAttr$width[map.idx]*2
  }
  nAttr<- lapply(nAttr, function(x) {names(x) <- nNames
                                     x})

  na.col=colorpanel2(1, low=na.col, high=na.col)
  fillcol=rep(na.col, length(nNames))
  names(fillcol)=nNames

                                        #edge attributes
  subdisplay <- subtypeDisplay(gR2)
  if(length(subdisplay)<1) eAttrs=list() else{
    na.rn=apply(subdisplay, 2, function(x) sum(is.na(x))==7)
  if(sum(na.rn)>0) subdisplay[,na.rn]=KEGGEdgeSubtype[KEGGEdgeSubtype[,1]=="others",rownames(subdisplay)]
  eLabel <- subdisplay["label", ]
  eCol <- subdisplay["color", ]
  eTextCol <- subdisplay["fontcolor", ]
  eLty <- subdisplay["style", ]
  eArrowhead <- subdisplay["arrowhead", ]
  if (ncol(subdisplay) == 1) {
    tmp <- colnames(subdisplay)[1]
    names(eLabel) <- names(eCol) <- names(eTextCol) <- tmp
    names(eLty) <- names(eArrowhead) <- tmp
  }
  eAttrs <- list(lty = eLty, col = eCol, textCol = eTextCol, 
                 label = eLabel, arrowhead = eArrowhead)
  }
  
  gR2.layout=gR2
  edgeRenderInfo(gR2.layout)=eAttrs
  layoutType=ifelse(is.signal, "dot", "neato")
  gR2.layout <- layoutGraph(gR2.layout, attrs = attrs, nodeAttrs=nAttr, layoutType=layoutType)
  edgeRenderInfo(gR2.layout)=eAttrs
#  edgeRenderInfo(gR2.layout)$col=eAttrs$col
  nri=nodeRenderInfo(gR2.layout)
  loc=list(x=nri$nodeX, y=nri$nodeY)
  if(sum(rect.idx)>0){
  w.unit=min(nri$lWidth[rect.idx])
  h.unit=min(nri$height[rect.idx])
}
  cni=nSizes>1
  if(sum(cni)>0){
    xloc=rep(loc[[1]][cni], nSizes[cni])
    sn.y=unlist(sapply(nSizes[cni], function(x) seq(-(x-1)/2, (x-1)/2,1)),use.names =F)
    yloc=rep(loc[[2]][cni], nSizes[cni])+h.unit*sn.y
  } else xloc=yloc=NULL
  xloc.nd=c(xloc,loc[[1]][nSizes==1 & rect.idx])
  yloc.nd=c(yloc,loc[[2]][nSizes==1 & rect.idx])
                                        #labs=gsub("-", "- ", node.data$labels)
                                        #labs=sapply(labs,wordwrap,len=text.width)
                                        #labs=gsub("- ", "-", labs)
  labs=node.data$labels
  labs[nNames[map.idx]]=sapply(labs[nNames[map.idx]],wordwrap,width=text.width, break.word=F)
  labs[nNames[cpd.idx]]=sapply(labs[nNames[cpd.idx]],wordwrap,width=text.width, break.word=T)


  cols.ts.gene=cbind(cols.ts.gene)
  cols.ts.cpd=cbind(cols.ts.cpd)
  nc.gene=max(ncol(cols.ts.gene),0)
  nc.cpd=max(ncol(cols.ts.cpd),0)#@
  nplots=max(nc.gene,nc.cpd)
  pn.suffix=colnames(cols.ts.gene)
  if(length(pn.suffix)<nc.cpd)  pn.suffix=colnames(cols.ts.cpd)
  if(length(pn.suffix)<nplots)  pn.suffix=1:nplots #no column names for both datasets
  if(length(pn.suffix)==1) {
    pn.suffix=out.suffix
  } else pn.suffix=paste(out.suffix, pn.suffix, sep=".")

  if((match.data | !multi.state) & nc.gene!=nc.cpd){
#  if(nc.gene>nc.cpd) cols.ts.cpd= cols.ts.cpd[, rep(1:nc.cpd, nplots)[1:nplots]]
#  if(nc.gene<nc.cpd) cols.ts.gene= cols.ts.gene[, rep(1:nc.gene, nplots)[1:nplots]]

    if(nc.gene>nc.cpd & !is.null(cols.ts.cpd)){
      na.mat=matrix(na.col, ncol=nplots-nc.cpd, nrow=nrow(cols.ts.cpd))
      cols.ts.cpd= cbind(cols.ts.cpd, na.mat)
    }
    if(nc.gene<nc.cpd & !is.null(cols.ts.gene)){
      na.mat=matrix(na.col, ncol=nplots-nc.gene, nrow=nrow(cols.ts.gene))
      cols.ts.gene= cbind(cols.ts.gene, na.mat)
    }
    nc.gene=nc.cpd=nplots
  }

  if(!is.null(cols.ts.gene)){
    nidx.gene=which(nNames %in% rownames(cols.ts.gene))
    cidx.gene=match(nNames[nidx.gene], rownames(cols.ts.gene))
    sci.gene=match(sub2grp[sub.idx,1], rownames(cols.ts.gene))
    sci.node=match(sub2grp[sub.idx,1], nNames)
  }
    if(!is.null(cols.ts.cpd)){
    nidx.cpd=which(nNames %in% rownames(cols.ts.cpd))
    cidx.cpd=match(nNames[nidx.cpd], rownames(cols.ts.cpd))
  }

  out.fmt="Working in directory %s"
  wdir=getwd()
  out.msg=sprintf(out.fmt, wdir)
  message("Info: ", out.msg)
  out.fmt="Writing image file %s"
  
#initialize node colors, independent of user data
  if(sum(rect.idx)>0){
    cn.col=rep(NA, sum(sub.idx))
    cn.col=fillcol[sci.node]
    names(cn.col)=sub2grp[sub.idx,1]
    rect.col=c(cn.col,fillcol[nSizes==1 & rect.idx])
    rect.col[rect.col==na.col]=NA
    rect.col=matrix(rep(rect.col,nplots), ncol=nplots)
  }
  if(sum(cpd.idx)>0){
    ell.col=fillcol[cpd.idx]
    ell.col[ell.col==na.col]=NA
    ell.col=matrix(rep(ell.col,nplots), ncol=nplots)
    w.e=min(nri$lWidth[cpd.idx])
    h.e=min(nri$height[cpd.idx])
    xloc.e=loc[[1]][cpd.idx]
    yloc.e=loc[[2]][cpd.idx]
  }


#  fillcol=matrix(rep(fillcol,nplots), ncol=nplots)
  fillcol=matrix(na.col, nrow=length(nNames), ncol=nplots)
  rownames(fillcol)=nNames

  if(!is.null(cols.ts.gene) & sum(rect.idx)>0){
    fillcol[nidx.gene,1:nc.gene]=cols.ts.gene[cidx.gene,] #need to be added in pathview.graph
    cn.col=matrix(NA, nrow=sum(sub.idx), ncol=nc.gene)
    cn.col[]=cols.ts.gene[sci.gene,]
    rownames(cn.col)=sub2grp[sub.idx,1]
    if(nc.gene>1) rect.col=rbind(cn.col,fillcol[nSizes==1 & rect.idx,1:nc.gene])
    else rect.col=c(cn.col,fillcol[nSizes==1 & rect.idx,1])
    rect.col[rect.col==na.col]=NA
  }

  if(!is.null(cols.ts.cpd) & sum(cpd.idx)>0){
    fillcol[nidx.cpd,1:nc.cpd]=cols.ts.cpd[cidx.cpd,]
  if(sum(cpd.idx)>0){
    ell.col=fillcol[cpd.idx,1:nc.cpd]
    ell.col[ell.col==na.col]=NA
  }
  }

  multi.state=multi.state & nplots>1
if(multi.state) {
  nplots=1
  pn.suffix=paste(out.suffix, "multi", sep=".")
  if(sum(rect.idx>0)) rect.col.plot=rect.col
  if(sum(cpd.idx)>0) ell.col.plot=ell.col
}
  
for(np in 1:nplots){
  gfile=paste(pathway.name, pn.suffix[np],"pdf", sep=".")
#  gfile=paste(pathway.name, pn.suffix[1],"pdf", sep=".")
  out.msg=sprintf(out.fmt, gfile)
  message("Info: ", out.msg)

#KEGG legend type
  ntypes=length(unique(node.data$type[nNames]))
  etypes=nrow(unique(t(subdisplay)))
  if(!same.layer) kl.type="both"
  
  else{
    if(ntypes<3 & sum(cpd.idx)<3) kl.type="edge"
    else if(etypes<3) kl.type="node"
    else {
      kl.type="both"
      same.layer=FALSE
    }
  }

  pdf.width=ifelse(same.layer,1.5,1)* pdf.size[1]
    pdf(gfile, width=pdf.width,height=pdf.size[2])
    op <- par(no.readonly = TRUE) # save default, for resetting...
    if(same.layer) nf <- layout(cbind(1,2), c(2,1))#, TRUE)
    rg=renderGraph(gR2.layout)
    gri=graphRenderInfo(rg)
    par(mai=gri$mai, usr=gri$usr)
  
                                        #text(loc, label = labs[nNames], cex = cex)
  if(sum(rect.idx)>0){
    if(!multi.state) rect.col.plot=cbind(rect.col)[,np]
#    rect(xloc.nd-w.unit,yloc.nd-h.unit/2, xloc.nd+w.unit,yloc.nd+h.unit/2, col=rect.col)
    rect.out=sliced.shapes(xloc.nd,yloc.nd, w.unit, h.unit/2, cols=rect.col.plot, shape="rectangle")
  }
  if(sum(cpd.idx)>0) {
    if(!multi.state) ell.col.plot=cbind(ell.col)[,np]
#    ellipses(xloc.e, yloc.e, w.e, h.e/2, cols=ell.col)
    ell.out=sliced.shapes(xloc.e, yloc.e, w.e, h.e/2, cols=ell.col.plot, shape="ellipse")
  }
  if(sum(cni)>0){
    if(sum(sub.idx)>0) text(xloc, yloc, label = labs[sub2grp[sub.idx,1]], cex = cex)
  }
                                        #text(loc, label = labs[nNames], cex = cex)
  if(sum(!cpd.idx)>0) text(loc[[1]][!cpd.idx], loc[[2]][!cpd.idx], label = labs[nNames[!cpd.idx]], cex = cex)
  if(sum(cpd.idx)>0) {
    if(map.cpdname & !is.null(cols.ts.cpd))  yloc.et=yloc.e+h.e*cpd.lab.offset else  yloc.et=yloc.e
    text(xloc.e, yloc.et, label = labs[nNames[cpd.idx]], cex = cex)
  }

  
  pv.pars=list()
  pv.pars$gsizes=c(gri$bbox[2,1],gri$bbox[2,2])
  if(sum(rect.idx)>0){
    pv.pars$nsizes=c(w.unit,h.unit)
  } else pv.pars$nsizes=c(w.e,h.e)
  pv.pars$op=op
  pv.pars$key.cex=cex*1.5
  pv.pars$key.lwd=1
  pv.pars$sign.cex=1.2*cex

  off.sets=c(x=0,y=0)
  align="n"

  ucol.gene=unique(as.vector(cols.ts.gene))
 na.col.gene=ucol.gene %in% c(na.col, NA)
  if(plot.col.key & !is.null(cols.ts.gene) & !all(na.col.gene))  {
    off.sets=col.key(limit=limit$gene, bins=bins$gene, both.dirs=both.dirs$gene, discrete=discrete$gene, graph.size=pv.pars$gsizes,
      node.size=pv.pars$nsizes, key.pos=key.pos, cex=pv.pars$key.cex, lwd=pv.pars$key.lwd, low=low$gene, mid=mid$gene, high=high$gene, align="n")
    align=key.align
    
  }
  
 ucol.cpd=unique(as.vector(cols.ts.cpd))
 na.col.cpd=ucol.cpd %in% c(na.col, NA)
  if(plot.col.key & !is.null(cols.ts.cpd) & !all(na.col.cpd)) {
    off.sets=col.key(limit=limit$cpd, bins=bins$cpd, both.dirs=both.dirs$cpd, discrete=discrete$cpd, graph.size=pv.pars$gsizes, node.size=pv.pars$nsizes, key.pos=key.pos, off.sets=off.sets, cex=pv.pars$key.cex, lwd=pv.pars$key.lwd, low=low$cpd, mid=mid$cpd, high=high$cpd, align=align)
  }
  
  if(new.signature) pathview.stamp(position=sign.pos, graph.sizes=pv.pars$gsizes, on.kegg=F, cex = pv.pars$sign.cex)
#  kegg.legend(edges.only=same.layer)
  kegg.legend(type=kl.type)
  par(pv.pars$op)
  dev.off()
}
  
  return(invisible(pv.pars))
}

parseKGML2<-function (file)
  {
    doc <- xmlTreeParse(file, getDTD = FALSE)
    r <- xmlRoot(doc)
    childnames <- sapply(xmlChildren(r), xmlName)
    isEntry <- childnames == "entry"
    isRelation <- childnames == "relation"
    isReaction <- childnames == "reaction"
    kegg.pathwayinfo <- parsePathwayInfo(r)
    kegg.nodes <- sapply(r[isEntry], parseEntry)
    kegg.edges <- sapply(r[isRelation], parseRelation)
    kegg.reactions <- sapply(r[isReaction], parseReaction2)
    names(kegg.nodes) <- sapply(kegg.nodes, getEntryID)
    pathway <- new("KEGGPathway", pathwayInfo = kegg.pathwayinfo,
                   nodes = kegg.nodes, edges = kegg.edges, reactions = kegg.reactions)
    return(pathway)
  }

cpdidmap <-
function(in.ids, in.type, out.type){
#map between cpd ids, both kegg and others
  cnames=c(in.type, out.type)
  in.type=tolower(in.type)
  out.type=tolower(out.type)
  data(rn.list)
  cpd.type=names(rn.list)=tolower(names(rn.list))
  
  if(!in.type %in% cpd.type) stop("Incorrect type!")
  if(out.type=="kegg")  kg.idx=grep("kegg", cpd.type)
  else if(out.type %in% cpd.type) kg.idx=which(cpd.type==out.type)
  else stop("Incorrect out.type!")
  if(in.type %in% cpd.type[kg.idx]) {
    message("Note: ", "A native KEGG compound ID type, no need to map!")
    return(invisible(0))
  }
  in.ids=in.ids[!is.na(in.ids)]
#  in.ids=unique(in.ids)
  len.id=length(in.ids)
  kg.accs=rep(NA, len.id)
  
  data(cpd.accs)
  accs.intype=cpd.accs[rn.list[[in.type]], ]
  in.idx=in.ids  %in% accs.intype$ACCESSION_NUMBER
  if(sum(in.idx)<1) {#stop("None of the compound ids mapped to the specified type!")
    message("Note: ", "None of the compound ids mapped to the specified type!")
  } else{
    rev.idx=accs.intype$ACCESSION_NUMBER %in% in.ids
    if(sum(rev.idx)>sum(in.idx)) message("Note: ", "multiple compounds may map to a input ID, only the first one kept!")
    in.cpd=accs.intype$COMPOUND_ID[match(in.ids[in.idx], accs.intype$ACCESSION_NUMBER)]


    accs.kg=cpd.accs[unlist(rn.list[kg.idx]), ]
    map.idx=in.cpd %in% accs.kg$COMPOUND_ID
    if(sum(map.idx)<1) message("Note: ", "None of the compound ids mapped to specified output ID type(s)!")
    in.cpd=in.cpd[map.idx]
    cpd.idx=match(in.cpd, accs.kg$COMPOUND_ID)
    kg.accs[in.idx][map.idx]=as.character(accs.kg$ACCESSION_NUMBER[cpd.idx])
  }
  kg.accs=cbind(in.ids, kg.accs)
  colnames(kg.accs)=cnames
  return(kg.accs)
}

utils::globalVariables("KEGGEdgeSubtype")

subtypeDisplay.kedge <-
function (object, ...) 
{
  .local <- function (object) 
    {
      subtypes <- getSubtype(object)
      if (length(subtypes) == 1) {
        display <- subtypeDisplay(subtypes[[1]])
      }
      else if(length(subtypes) > 1){
        display.matrix <- sapply(subtypes, subtypeDisplay)
        display <- display.matrix[,1]
        display[c("name","value")] <- apply(display.matrix[c("name","value"), ], 1, paste, collapse = ",")
        dlabel=display.matrix["label", ]
        display["label"] <- paste(dlabel[dlabel>""], collapse = ",")
      } else{
        etype=getType(object)
        if(etype=="PCrel"){
          st.uk=new("KEGGEdgeSubType", name = "compound", value = "unknown")
        } else{
          message("Info: ", paste("Edge type:", etype, "without subtype sepecified!"))
          st.uk=new("KEGGEdgeSubType", name = "unknown", value = "unknown")
        }
        display <- subtypeDisplay(st.uk)
      }
      return(display)
    }
  .local(object, ...)
}

setMethod("subtypeDisplay", signature(object = "KEGGEdge"), subtypeDisplay.kedge)

pathview <-
function(
                         gene.data=NULL,
                         cpd.data=NULL,
#                         xml.file=NULL,
                         pathway.id,
                         species = "hsa",
                         kegg.dir=".",
                   cpd.idtype="kegg",
                   gene.idtype="entrez",
                   gene.annotpkg=NULL,
                   min.nnodes=3,#
         
                   kegg.native=TRUE,
                   map.null=TRUE,
                   expand.node=FALSE, #g
                           split.group=FALSE, #g

                         map.symbol=TRUE,
                         map.cpdname=TRUE, #g
                         
                         node.sum="sum",
         discrete=list(gene=FALSE, cpd=FALSE),
         limit=list(gene=1, cpd=1),
                         bins=list(gene=10, cpd=10),
                         both.dirs=list(gene=T, cpd=T),
                         trans.fun = list(gene = NULL,  cpd = NULL),
         low = list(gene = "green", cpd = "blue"),
                         mid = list(gene = "gray", cpd = "gray"),
                         high = list(gene = "red", cpd = "yellow"),
         na.col="transparent",
         
#                         new.signature=TRUE,
#                         plot.col.key=TRUE,
#                         key.align="x",
#                         key.pos="topright",
#                         sign.pos="bottomright",#g
                         ...){

#length-2 arguments check

  dtypes=!is.null(gene.data)+!is.null(cpd.data)
  cond0=dtypes==1 & is.numeric(limit) & length(limit)>1
  if(cond0){
    if(limit[1]!=limit[2] & is.null(names(limit)))
      limit=list(gene=limit[1:2], cpd=limit[1:2])
  }
  if(is.null(trans.fun)) trans.fun=list(gene = NULL,  cpd = NULL)
  
  arg.len2=c("discrete", "limit","bins", "both.dirs", "trans.fun", "low", "mid", "high")
  for(arg in arg.len2){
    obj1=eval(as.name(arg))
    if(length(obj1)==1) obj1=rep(obj1,2)
    if(length(obj1)>2) obj1=obj1[1:2]
    obj1=as.list(obj1)
    ns=names(obj1)
    if(length(ns)==0 |!all(c("gene", "cpd") %in% ns)) names(obj1)=c("gene", "cpd")
    assign(arg, obj1)
  }

#data.checck
  if(is.character(gene.data)){
    gd.names=gene.data
    gene.data=rep(1, length(gene.data))
    names(gene.data)=gd.names
    both.dirs$gene=FALSE
    ng=length(gene.data)
    nsamp.g=1
  } else if(!is.null(gene.data)){
    if(length(dim(gene.data))==2){
      gd.names=rownames(gene.data)
      ng=nrow(gene.data)
      nsamp.g=2
    } else if(is.numeric(gene.data) & is.null(dim(gene.data))){
      gd.names=names(gene.data)
      ng=length(gene.data)
      nsamp.g=1
    } else stop("wrong gene.data format!")
  } else if(is.null(cpd.data)){
    stop("gene.data and cpd.data are both NULL!")
  }
  gene.idtype=toupper(gene.idtype)
  data(bods)
#  data(gene.idtype.bods)
  if(species!="ko"){
    species.data=kegg.species.code(species, na.rm=T, code.only=FALSE)
  } else {
    species.data=c(kegg.code="ko", entrez.gnodes="0", kegg.geneid="K01488", ncbi.geneid=NA, ncbi.proteinid=NA, uniprot=NA)
    gene.idtype="KEGG"
    msg.fmt="Only KEGG ortholog gene ID is supported, make sure it looks like \"%s\"!"
    msg=sprintf(msg.fmt, species.data["kegg.geneid"])
    message("Note: ", msg)
  }
  if(length(dim(species.data))==2) {
    message("Note: ", "More than two valide species!")
    species.data=species.data[1,]
  }
  species=species.data["kegg.code"]
  entrez.gnodes=species.data["entrez.gnodes"]==1
  if(is.na(species.data["ncbi.geneid"])){
    if(!is.na(species.data["kegg.geneid"])){
      msg.fmt="Mapping via KEGG gene ID (not Entrez) is supported for this species,\nit looks like \"%s\"!"
      msg=sprintf(msg.fmt, species.data["kegg.geneid"])
      message("Note: ", msg)
    } else{
      stop("This species is not annotated in KEGG!")
    }
  }
  if(is.null(gene.annotpkg)) gene.annotpkg=bods[match(species, bods[,3]),1]
  if(length(grep("ENTREZ|KEGG|NCBIPROT|UNIPROT", gene.idtype))<1 & !is.null(gene.data)){
    if(is.na(gene.annotpkg)) stop("No proper gene annotation package available!")
    if(!gene.idtype %in% gene.idtype.bods[[species]]) stop("Wrong input gene ID type!")
    gene.idmap=id2eg(gd.names, category=gene.idtype, pkg.name=gene.annotpkg, unique.map=F)
    gene.data=mol.sum(gene.data, gene.idmap)
    gene.idtype="ENTREZ"
  }

  if(gene.idtype!="KEGG" & !entrez.gnodes & !is.null(gene.data)){
    id.type=gene.idtype
    if(id.type=="ENTREZ") id.type="ENTREZID"
    kid.map=names(species.data)[-c(1:2)]
    kid.types=names(kid.map)=c("KEGG", "ENTREZID", "NCBIPROT", "UNIPROT")
    kid.map2=gsub("[.]", "-", kid.map)
    kid.map2["UNIPROT"]="up"
    if(is.na(kid.map[id.type])) stop("Wrong input gene ID type for the species!")
    message("Info: Getting gene ID data from KEGG...")
    gene.idmap=keggConv(kid.map2[id.type],species)
    message("Info: Done with data retrieval!")
    kegg.ids=gsub(paste(species, ":", sep=""), "", names(gene.idmap))
    in.ids=gsub(paste0(kid.map2[id.type],":"), "", gene.idmap)
    gene.idmap=cbind(in.ids, kegg.ids)
    gene.data=mol.sum(gene.data, gene.idmap)
    gene.idtype="KEGG"
  }

  
  if(is.character(cpd.data)){
    cpdd.names=cpd.data
    cpd.data=rep(1, length(cpd.data))
    names(cpd.data)=cpdd.names
    both.dirs$cpd=FALSE
    ncpd=length(cpd.data)
  } else if(!is.null(cpd.data)){
    if(length(dim(cpd.data))==2){
    cpdd.names=rownames(cpd.data)
    ncpd=nrow(cpd.data)
    } else if(is.numeric(cpd.data) & is.null(dim(cpd.data))){
        cpdd.names=names(cpd.data)
        ncpd=length(cpd.data)
    } else stop("wrong cpd.data format!")
  }
  if(length(grep("kegg", cpd.idtype))<1 & !is.null(cpd.data)){
    data(rn.list)
    cpd.types=c(names(rn.list),"name")
    cpd.types=tolower(cpd.types)
    cpd.types=cpd.types[-grep("kegg", cpd.types)]
    if(!tolower(cpd.idtype) %in% cpd.types) stop("Wrong input cpd ID type!")
    cpd.idmap=cpd2kegg(cpdd.names, in.type=cpd.idtype)
    cpd.data=mol.sum(cpd.data, cpd.idmap)
  }

  
#parse  
  warn.fmt="Parsing %s file failed, please check the file!"

  if(length(grep(species, pathway.id))>0) {
    pathway.name = pathway.id
    pathway.id = gsub(species, "", pathway.id)
  } else pathway.name = paste(species, pathway.id, sep = "")
  kfiles=list.files(path=kegg.dir, pattern="[.]xml|[.]png")
  npath=length(pathway.id)
  out.list=list()#vector(mode = "list", length = npath)
  
  #if(is.null(xml.file) | length(xml.file)!=npath)#custom xml and png file need to have the same names
  tfiles.xml=paste(pathway.name, "xml", sep=".")
  tfiles.png=paste(pathway.name, "png", sep=".")
  if(kegg.native) ttype=c("xml", "png") else ttype="xml"
  xml.file <- paste(kegg.dir, "/", tfiles.xml, sep = "")
  
  for(i in 1:npath){
    ##  out.list=lapply(1:npath, function(i){
    if(kegg.native) tfiles=c(tfiles.xml[i],tfiles.png[i])
    else tfiles=tfiles.xml[i]
    if(!all(tfiles %in% kfiles)){
      dstatus=download.kegg(pathway.id = pathway.id[i], species = species, kegg.dir=kegg.dir, file.type=ttype)
      if(dstatus=="failed") {
        warn.fmt="Failed to download KEGG xml/png files, %s skipped!"
        warn.msg=sprintf(warn.fmt, pathway.name[i])
        message("Warning: ", warn.msg)
        return(invisible(0))#out.list[[i]]=0
      }
    }

  if(kegg.native){
    node.data=try(node.info(xml.file[i]), silent=T)
    if(class(node.data)[1]=="try-error"){
      warn.msg=sprintf(warn.fmt, xml.file[i])
      message("Warning: ", warn.msg)
      return(invisible(0))
    }
    node.type=c("gene","enzyme", "compound", "ortholog")
    sel.idx=node.data$type %in% node.type
    nna.idx=!is.na(node.data$x+node.data$y+node.data$width+node.data$height)
    sel.idx=sel.idx & nna.idx
    if(sum(sel.idx)<min.nnodes){
      warn.fmt="Number of mappable nodes is below %d, %s skipped!"
      warn.msg=sprintf(warn.fmt, min.nnodes, pathway.name[i])
      message("Warning: ", warn.msg)
      return(invisible(0))
    }
    node.data=lapply(node.data, "[", sel.idx)
  } else {
    gR1=try(parseKGML2Graph2(xml.file[i], genes=F, expand=expand.node, split.group=split.group), silent=T)
    node.data=try(node.info(gR1), silent=T)
    if(class(node.data)[1]=="try-error"){
      warn.msg=sprintf(warn.fmt, xml.file[i])
      message("Warning: ", warn.msg)
      return(invisible(0))
    }
  }


  if(species=="ko") gene.node.type="ortholog" else gene.node.type="gene"
  if((!is.null(gene.data) |map.null) & sum(node.data$type==gene.node.type)>1){
    plot.data.gene=node.map(gene.data, node.data, node.types=gene.node.type, node.sum=node.sum, entrez.gnodes=entrez.gnodes)
    kng=plot.data.gene$kegg.names
    kng.char=gsub("[0-9]", "", unlist(kng))
    if(any(kng.char>"")) entrez.gnodes=FALSE
    if(map.symbol & species!="ko" & entrez.gnodes) {
              if(is.na(gene.annotpkg)) {
                warn.fmt="No annotation package for the species %s, gene symbols not mapped!"
                warn.msg=sprintf(warn.fmt, species)
                message("Warning: ", warn.msg)
              } else {
#                browser()
                plot.data.gene$labels=eg2id(as.character(plot.data.gene$kegg.names), category="SYMBOL", pkg.name=gene.annotpkg)[,2]
                mapped.gnodes=rownames(plot.data.gene)
                node.data$labels[mapped.gnodes]=plot.data.gene$labels
              }
            }
            cols.ts.gene=node.color(plot.data.gene, limit$gene, bins$gene, both.dirs=both.dirs$gene, trans.fun=trans.fun$gene, discrete=discrete$gene, low=low$gene, mid=mid$gene, high=high$gene,  na.col=na.col)
          } else plot.data.gene=cols.ts.gene=NULL
          if((!is.null(cpd.data) | map.null) & sum(node.data$type=="compound")>1){
#          if(sum(node.data$type=="compound")>1){
            plot.data.cpd=node.map(cpd.data, node.data, node.types="compound", node.sum=node.sum)
            if(map.cpdname & !kegg.native) { #@
              plot.data.cpd$labels=cpdkegg2name(plot.data.cpd$labels)[,2]
              mapped.cnodes=rownames(plot.data.cpd)
              node.data$labels[mapped.cnodes]=plot.data.cpd$labels
            }
            cols.ts.cpd=node.color(plot.data.cpd, limit$cpd, bins$cpd, both.dirs=both.dirs$cpd, trans.fun=trans.fun$cpd, discrete=discrete$cpd, low=low$cpd, mid=mid$cpd, high=high$cpd, na.col=na.col)
          } else plot.data.cpd=cols.ts.cpd=NULL

  if(kegg.native){
 pv.pars= keggview.native(plot.data.gene=plot.data.gene, cols.ts.gene=cols.ts.gene, plot.data.cpd=plot.data.cpd, cols.ts.cpd=cols.ts.cpd, node.data=node.data, pathway.name=pathway.name[i], kegg.dir=kegg.dir, limit=limit, bins=bins, both.dirs=both.dirs,discrete=discrete, low=low, mid=mid, high=high, na.col=na.col, ...)
} else{
  pv.pars= keggview.graph(plot.data.gene=plot.data.gene, cols.ts.gene=cols.ts.gene, plot.data.cpd=plot.data.cpd, cols.ts.cpd=cols.ts.cpd, node.data=node.data, path.graph=gR1, pathway.name=pathway.name[i],  map.cpdname=map.cpdname, split.group=split.group, limit=limit, bins=bins, both.dirs=both.dirs, discrete=discrete, low=low, mid=mid, high=high, na.col=na.col, ...)
}
  
  plot.data.gene=cbind(plot.data.gene, cols.ts.gene)
  if(!is.null(plot.data.gene)){
    cnames=colnames(plot.data.gene)[-(1:8)]
    nsamp=length(cnames)/2
    if(nsamp>1){
      cnames[(nsamp+1):(2*nsamp)]=paste(cnames[(nsamp+1):(2*nsamp)], "col", sep=".")
    } else cnames[2]="mol.col"
    colnames(plot.data.gene)[-(1:8)]=cnames
  }
  plot.data.cpd=cbind(plot.data.cpd, cols.ts.cpd)    
  if(!is.null(plot.data.cpd)){
    cnames=colnames(plot.data.cpd)[-(1:8)]
    nsamp=length(cnames)/2
    if(nsamp>1){
      cnames[(nsamp+1):(2*nsamp)]=paste(cnames[(nsamp+1):(2*nsamp)], "col", sep=".")
    } else cnames[2]="mol.col"
    colnames(plot.data.cpd)[-(1:8)]=cnames
  }
#  return(list(plot.data.gene=plot.data.gene, plot.data.cpd=plot.data.cpd))#out.list[[i]]=
  out.list[[i]]=list(plot.data.gene=plot.data.gene, plot.data.cpd=plot.data.cpd)
}

  if(npath==1) out.list=out.list[[1]] else names(out.list)=pathway.name
  
  return(invisible(out.list))
}

render.kegg.node <-
function(plot.data, cols.ts, img, same.layer=TRUE, type=c("gene","compound")[1], text.col="black", cex=0.25){
  width=ncol(img)
  height=nrow(img)
  nn=nrow(plot.data)
  pwids=plot.data$width
  if(!all(pwids==max(pwids))){
    message("Info: ", "some node width is different from others, and hence adjusted!")
    wc=table(pwids)
    pwids=plot.data$width=as.numeric(names(wc)[which.max(wc)])
  }

  if(type=="gene"){
  if(same.layer!=T){
    rect.out=sliced.shapes(plot.data$x+0.5, height-plot.data$y, plot.data$width/2-0.5, plot.data$height/2-0.25,  cols=cols.ts, draw.border=F, shape="rectangle")
    text(plot.data$x+0.5, height-plot.data$y, labels = as.character(plot.data$labels),
         cex = cex, col = text.col)
    return(invisible(1))
  } else{
    img2=img
    pidx=cbind(ceiling(plot.data$x-plot.data$width/2)+1,
      floor(plot.data$x+plot.data$width/2)+1,
      ceiling(plot.data$y-plot.data$height/2)+1,
      floor(plot.data$y+plot.data$height/2)+1)
    cols.ts=cbind(cols.ts)
    ns=ncol(cols.ts)
    brk.x= sapply(plot.data$width/2, function(wi) seq(-wi, wi, length = ns+1))
    for(k in 1:ns){
      col.rgb=col2rgb(cols.ts[,k])/255
      pxr=t(apply(pidx[,1:2], 1, function(x) x[1]:x[2]))-plot.data$x-1
      sel=pxr>=ceiling(brk.x[k,]) & pxr<=floor(brk.x[k+1,])
      for(i in 1:nn){
      sel.px=(pidx[i,1]:pidx[i,2])[sel[i,]]
      node.rgb=img[pidx[i,3]:pidx[i,4],sel.px, 1:3]
      node.rgb.sum=apply(node.rgb,c(1,2), sum)
      blk.ind=which(node.rgb.sum==0|node.rgb.sum==1,arr.ind=T)
      node.rgb=array(col.rgb[,i],dim(node.rgb)[3:1])
      node.rgb=aperm(node.rgb, 3:1)
      for(j in 1:3) node.rgb[cbind(blk.ind,j)]=0
      img2[pidx[i,3]:pidx[i,4],sel.px, 1:3]=node.rgb
    }
  }
    return(img2)
  }
} else if(type=="compound"){
  if(same.layer!=T){
    nc.cols=ncol(cbind(cols.ts))
    if(nc.cols>2){#block the background circle
      na.cols=rep("#FFFFFF", nrow(plot.data))
      cir.out=sliced.shapes(plot.data$x, height-plot.data$y, plot.data$width[1], plot.data$width[1], cols=na.cols, draw.border=F, shape="ellipse", lwd=0.2)
    }
    cir.out=sliced.shapes(plot.data$x, height-plot.data$y, plot.data$width[1], plot.data$width[1], cols=cols.ts, shape="ellipse", blwd=0.2)
    return(invisible(1))
  } else{
#    col.rgb=col2rgb(cols.ts)/255
    blk=c(0,0,0)
    img2=img
    w=ncol(img) #repeat
    h=nrow(img) #repeat
    cidx=rep(1:w, each=h)
    ridx=rep(1:h, w)
    pidx=lapply(1:nn, function(i){
      ii=which((cidx-plot.data$x[i])^2+(ridx-plot.data$y[i])^2<(plot.data$width[i])^2)
      imat=cbind(cbind(ridx, cidx)[rep(ii,each=3),],1:3)
      imat[,1:2]=imat[,1:2]+1
      ib=which(abs((cidx-plot.data$x[i])^2+(ridx-plot.data$y[i])^2-(plot.data$width[i])^2)<=8)
      ibmat=cbind(cbind(ridx, cidx)[rep(ib,each=3),],1:3)
      ibmat[,1:2]=ibmat[,1:2]+1
      return(list(fill=imat,border=ibmat))
    })

    cols.ts=cbind(cols.ts)
    ns=ncol(cols.ts)
    brk.x= sapply(plot.data$width, function(wi) seq(-wi, wi, length = ns+1))
    for(i in 1:nn){
      pxr=pidx[[i]]$fill[,2]-1-plot.data$x[i]
      col.rgb=col2rgb(cols.ts[i,])/255
      for(k in 1:ns){
        sel=pxr>=brk.x[k,i] & pxr<=brk.x[k+1,i]
        img2[pidx[[i]]$fill[sel,]]=col.rgb[,k]
      }
      img2[pidx[[i]]$border]=blk
    }
    return(img2)
  }
} else stop("unrecognized node type!")
}

id2eg <-function(ids, category=gene.idtype.list[1], org="Hs", pkg.name=NULL, ...){
  category=tolower(category)
  if(category %in% c("entrez", "eg", "entrezid")) stop("input ID or category is already Entrez Gene ID!")
  geneannot.map(in.ids=ids, in.type=category, out.type="entrez", org=org, pkg.name=pkg.name, ...)
}

node.info <-
function(object, short.name=TRUE){
  cobj=class(object)[1]
  if(cobj=="character"){
    object <-parseKGML2(object)
    ndata=nodes(object)    
  } else if(cobj=="KEGGPathway"){
    ndata=nodes(object)
  } else  if(cobj=="graphNEL"){
    ndata=getKEGGnodeData(object)
  }  else {
    stop("object should be either a filename, KEGGPathway, or graphNEL!")
  }

  nodeNames=sapply(ndata, getName)
  nodeType=sapply(ndata, getType)
  nodeComp=sapply(ndata, getComponent)
  node.size=sapply(nodeComp, length)

  grs1=sapply(ndata, function(x){
    grs=x@graphics
    c(labels=grs@name, shape=grs@type)
  })
  grs2=sapply(ndata, function(x){
    grs=x@graphics
    c(x=grs@x,y=grs@y,width=grs@width,height=grs@height)
  })
  grs1=t(grs1)
  grs2=t(grs2)

  graphic.data=as.list(cbind(data.frame(grs1, stringsAsFactors=F), data.frame(grs2)))
  nd.list=list(kegg.names=nodeNames, type=nodeType, component=nodeComp, size=node.size)
  nd.list=c(nd.list, graphic.data)
  if(short.name){
    gnames=sapply(strsplit(nd.list$labels, ", "), "[[", 1)
    map.idx=nd.list$type=="map"
    gnames[map.idx]=nd.list$labels[map.idx]
    gnames[is.na(gnames)]=""
    gnames=gsub("[.][.][.]", "", gnames)
    nd.list$labels=gnames
    nd.list$kegg.names=lapply(nd.list$kegg.names, function(x) gsub("^.*:", "", x))
  }
  
  nn=names(nodeNames)
  nd.list= lapply(nd.list, function(x) {
    names(x) <- nn
    return(x)
  })
  return(nd.list)
}

cpdname2kegg <-
function(in.ids){

  in.type="name"
#  in.ids=unique(in.ids)
  len.id=length(in.ids)
  kg.accs=rep(NA, len.id)

  data(cpd.names)
  in.idx=in.ids  %in% cpd.names$NAME
  if(sum(in.idx)<1){
    message("Note: ", "None of the compound ids mapped to the specified type!")
  } else{
    kg.accs[in.idx]=as.character(cpd.names$ACCESSION_NUMBER[match(in.ids[in.idx], cpd.names$NAME)])
  }
  kg.accs=cbind(in.ids, kg.accs)
  colnames(kg.accs)=c(in.type, "KEGG accession")
  return(kg.accs)
}

sliced.shapes=function (x0, y0, w, h, n = 100, cols, shape=c("ellipse", "rectangle")[1], draw.border=TRUE, blwd=1, ...)
  {
    cols=cbind(cols)
    dd=dim(cols)
    ns=dd[2]
    n=(round(n/ns)*ns+1)*2
    nw=length(w)
#    if(length(w)==1) x = seq(-w, w, length = n/2)
    x = sapply(w, function(wi) seq(-wi, wi, length = n/2))
    
    if(is.character(shape)){
      if(shape=="ellipse") {
        y = (1 - (x/w)^2)^0.5 * h
      } else if(shape=="rectangle"){
        n=(ns+1)*2
#        x = seq(-w, w, length = n/2)
#        y = rep(h, length(x))
        x = sapply(w, function(wi) seq(-wi, wi, length = n/2))
        y=x
        y[]=h
      } else stop("Wrong shape specified!")
    } else if(is.function(shape)) y=shape(x)
    else stop("Wrong shape class specified!")

#    x = c(x, rev(x))
#    y = c(y, -rev(y))
    x=apply(x, 2, function(xj) c(xj, rev(xj)))
    y=apply(y, 2, function(yj) c(yj, -rev(yj)))
    
    np = length(x0)
    x0 = matrix(rep(x0, each = n), nrow = n)
    y0 = matrix(rep(y0, each = n), nrow = n)

    x1 <- x0 + x[,1:nw]
    y1 <- y0 + y[,1:nw]

    if(ns==1) {
      cols=cols[,1]
      bidx=if(draw.border) NULL else NA
      for (i in 1:np) polygon(x1[, i], y1[, i], col = cols[i], border=bidx, lwd=blwd, ...)
    } else{
#      brk.x=seq(-w, w, length = ns+1)
      brk.x= sapply(w, function(wi) seq(-wi, wi, length = ns+1))
      for (i in 1:np) {
        for(j in 1:ns){
          if(nw==1) sel=x>=brk.x[j] & x<=brk.x[j+1]
          else  sel=x[,i]>=brk.x[j,i] & x[,i]<=brk.x[j+1,i]
          polygon(x1[sel, i], y1[sel, i], col = cols[i, j], border=NA, ...)
        }
        if(draw.border) polygon(x1[, i], y1[, i], col = NA, lwd=blwd, ...)
      }
    }
    return(list(x, y))
  }

KEGGpathway2Graph2 <-
function (pathway, genesOnly = TRUE, expandGenes = TRUE, split.group=FALSE, check.reaction=TRUE)
  {
    stopifnot(is(pathway, "KEGGPathway"))
    if(split.group) pathway <- splitKEGGgroup(pathway)
    rdata=(pathway@reactions)
    if (expandGenes){
      if(check.reaction & length(rdata)>0) message("Note: ", "Gene nodes not expanded when reactions are converted to edges!")
      else pathway <- expandKEGGPathway(pathway)
    }
    knodes <- nodes(pathway)
    kedges <- edges(pathway)
    node.entryIDs <- getEntryID(knodes)
    edge.entryIDs <- getEntryID(kedges)
    V <- node.entryIDs
    edL <- vector("list", length = length(V))
    names(edL) <- V
    if (is.null(nrow(edge.entryIDs))) {
      for (i in seq(along = edL)) {
        edL[[i]] <- list()
      }
    }
    else {
      for (i in 1:length(V)) {
        id <- node.entryIDs[i]
        hasRelation <- id == edge.entryIDs[, "Entry1ID"]
        if (!any(hasRelation)) {
          edL[[i]] <- list(edges = NULL)
        }
        else {
          entry2 <- unname(unique(edge.entryIDs[hasRelation,
                                                "Entry2ID"]))
          edL[[i]] <- list(edges = entry2)
        }
      }
    }
    gR <- new("graphNEL", nodes = V, edgeL = edL, edgemode = "directed")

    if(check.reaction & length(rdata)>0){
      r2e.res=reaction2edge(pathway, gR)
      gR=r2e.res[[1]]
      kedges=r2e.res[[2]]
      knodes=r2e.res[[3]]
    }

    names(kedges) <- sapply(kedges, function(x) paste(getEntryID(x),
                                                      collapse = "~"))
    env.node <- new.env()
    env.edge <- new.env()
    assign("nodes", knodes, envir = env.node)
    assign("edges", kedges, envir = env.edge)
    nodeDataDefaults(gR, "KEGGNode") <- env.node
    edgeDataDefaults(gR, "KEGGEdge") <- env.edge
    if (genesOnly) {
      gR <- subGraphByNodeType(gR, "gene")
    }
    return(gR)
  }

reaction2edge <-
function(path, gR){
  
#  gR=kegg2graph(path,genes=F,expand=F, split.group=F)
  ndata=nodes(path)
  rdata=(path@reactions)
  edata=edges(path)
  if(length(ndata)<2 | length(rdata)<1){
  message("Note: ", "Pathway has no reaction, no conversion needed!")
  return(list(gR, edata, ndata))
  }
  
  snames=slotNames(rdata[[1]])
  rdata.tab=sapply(rdata, function(x){
    sapply(snames, function(sn) slot(x, sn))#x@sn not work
  })
  rdata.tab=t(rdata.tab)

  snames2=slotNames(ndata[[1]])[c(1,2,3,5)]
  ndata.tab=sapply(ndata, function(x){
    sapply(snames2, function(sn) paste(slot(x, sn), collapse=","))#x@sn not work
  })
  ndata.tab=t(ndata.tab)
  rn.tab=ndata.tab[,"reaction"]!="NA"
  rn.tab=ndata.tab[rn.tab,]

  len.ed=length(edata)
  if(len.ed>0){
    snames3=slotNames(edata[[1]])
    edata.tab=sapply(edata, function(x){
      idtype=sapply(snames3[1:3], function(sn) slot(x, sn))#x@sn not work
      stnames=paste(sapply(x@subtype, function(y) y@name), collapse=",")
      stvals=paste(sapply(x@subtype, function(y) y@value), collapse=",")
      return(c(idtype, subtype.name=stnames,subtype.value=stvals))
    })
    edata.tab=t(edata.tab)
  }

#new node.data
  rn.grps=split(rn.tab[,"entryID"], rn.tab[,"reaction"])
  rn.len=sapply(rn.grps, length)
  grp.idx=which(rn.len>1)
#  grp nodes only for reactions with >1 genes
  eid=max(as.numeric(names(ndata)))+1:length(grp.idx)
  eid=as.character(eid)
  rnames=names(rn.grps)
  ndata.new=list()
  if(length(grp.idx)>0){
  for(i in 1:length(grp.idx)){
    j=grp.idx[i]
    ndata1=ndata[[rn.grps[[j]][1]]]
    gdata1=ndata1@graphics
    h1=gdata1@height*rn.len[j]
    gdata.new=new("KEGGGraphics",name=NA_character_, fgcolor="#000000", bgcolor="#FFFFFF", type="rectangle",
      x=gdata1@x, y=gdata1@y, width=gdata1@width, height=h1)
    ndata.new[[i]]<- new("KEGGGroup", component = rn.grps[[j]], entryID = eid[i],
                         name="undefined", type="group", link = "NA", reaction = rnames[j],
                         map = "NA", graphics = gdata.new)
  }
  names(ndata.new)=eid
#  gR=graph::addNode(eid, gR)
  gR=addNode(eid, gR)
}
  ndata.new=c(ndata,ndata.new)
  
#create edges for ECrel and maplink types of relationships
  slen=sapply(rdata.tab[,3], length)
  plen=sapply(rdata.tab[,5], length)
  glen=rn.len[match(rdata.tab[,"name"], rnames)]

  from1=to1=from2=to2=NULL
  for(i in 1:nrow(rdata.tab)){
    subs=rdata.tab[[i,"substrateName"]]
    prods=rdata.tab[[i,"productName"]]
    enzs=rn.grps[[rdata.tab[[i,"name"]]]]
    rtype=rdata.tab[[i,"type"]]
    from1=c(from1, rep(subs, each=glen[i]))
    to1=c(to1, rep(enzs, slen[i]))
    to2=c(to2, rep(prods, each=glen[i]))
    from2=c(from2, rep(enzs, plen[i]))
    if(rtype=="reversible"){
      to2=c(to2, rep(subs, each=glen[i]))
      from2=c(from2, rep(enzs, slen[i]))
      from1=c(from1, rep(prods, each=glen[i]))
      to1=c(to1, rep(enzs, plen[i]))
    }
  }
  names(from1)=names(from2)=names(to1)=names(to2)=NULL
  di1=duplicated(paste(from1, to1))
  di2=duplicated(paste(from2, to2))
  from1=from1[!di1]
  to1=to1[!di1]
  from2=from2[!di2]
  to2=to2[!di2]


#revise edges on the graph  
  if(len.ed>0){
    eci1=edata.tab[,"type"]=="ECrel"
    eci2=edata.tab[,"type"]=="maplink"
    eci=eci1|eci2
    gene1=edata.tab[eci1,"entry1ID"]
    gene2=edata.tab[eci1,"entry2ID"]
    mln1=edata.tab[eci2,"entry1ID"]
    mln2=edata.tab[eci2,"entry2ID"]
    cpd=edata.tab[eci2,"subtype.value"]
    if(sum(eci)>0) gR=removeEdge(from=c(gene1,mln1), to=c(gene2,mln2), graph=gR)
    if(sum(eci2)>0) gR=addEdge(from=cpd, to=mln2, graph=gR)
  }
  gR=addEdge(from=from1, to=to1, graph=gR)
                                        #may overlap with maplink edges
  gR=addEdge(from=from2, to=to2, graph=gR)

#revise edge.data
  st1=new("KEGGEdgeSubType", name="compound", value="NA")
  other.args=list(Class="KEGGEdge",type = "CErel", subtype = list(subtype=st1))
  edata1=mapply(new, entry1ID = from1, entry2ID = to1, MoreArgs=other.args)
  other.args=list(Class="KEGGEdge",type = "ECrel", subtype = list(subtype=st1))
  edata2=mapply(new, entry1ID = from2, entry2ID = to2, MoreArgs=other.args)
  other.args=list(Class="KEGGEdge",type = "ECrel", subtype = list(subtype=st1))#"maplink"
  edata3=NULL
  if(len.ed>0){
    if(sum(eci2)>0){
      other.args=list(Class="KEGGEdge",type = "maplink", subtype = list(subtype=st1))
      edata3=mapply(new, entry1ID = cpd, entry2ID = mln2, MoreArgs=other.args)
    }
  }
  edata.new=c(edata1, edata2, edata3)
  if(len.ed>0) edata.new=c(edata[!eci],edata.new)
  return(list(gR, edata.new, ndata.new))
}

node.color <-
  function(plot.data=NULL, discrete=FALSE, limit=1, bins=10, both.dirs=TRUE, low="green", mid="gray", high="red", na.col="transparent", trans.fun=NULL){
    if(is.null(plot.data)) return(NULL)
  node.summary=plot.data[,-c(1:8)]#c(1:7)
  if(length(dim(node.summary))==2) {
    node.summary=as.matrix(node.summary)
    } else names(node.summary)=rownames(plot.data)
  if(!is.null(trans.fun)) node.summary=trans.fun(node.summary)
if(both.dirs & length(limit)==1){
    limit=c(-abs(limit), abs(limit))
  } else if(length(limit)==1){
    limit=c(0,limit)
  }
    disc.cond1=all(as.integer(limit)==limit)
    disc.cond2=(limit[2]-limit[1]) %% bins==0
    if(discrete & disc.cond1 & disc.cond2){
      node.summary[]=as.integer(node.summary)
      limit[2]=limit[2]+1
      bins=bins+1
    } else if(discrete){
      message("Note: ", "limit or bins not proper, data not treated as discrete!")
    }

  
  node.summary[node.summary > limit[2]] = limit[2]
  node.summary[node.summary < limit[1]] = limit[1]
  if(both.dirs){
    cols = colorpanel2(bins, low=low, mid=mid, high=high)
  } else cols = colorpanel2(bins, low=mid, high=high)
  na.col=colorpanel2(1, low=na.col, high=na.col)
  data.cuts = seq(from = limit[1], to =limit[2], length=bins+1)
  index.ts = cols.ts = node.summary
  index.ts[] = cut(node.summary, data.cuts, include.lowest = TRUE, right=F)
  cols.ts[]=cols[index.ts]
  cols.ts[is.na(cols.ts)]=na.col
  return(cols.ts)
}

sim.mol.data=function(mol.type=c("gene","gene.ko","cpd")[1], id.type=NULL, species="hsa", discrete=FALSE, nmol=1000, nexp=1, rand.seed=100)
{
  msg.fmt="\"%s\" is not a good \"%s\" \"%s\" ID type for simulation!"
  msg.fmt2="\"%s\" has only %i unique IDs!"
  set.seed(rand.seed)

  species=species[1]
  if(species!="ko"){
    species.data=kegg.species.code(species, na.rm=T, code.only=FALSE)
    species=species.data["kegg.code"]
  } else if(mol.type=="gene") mol.type="gene.ko"
  
  
  if(mol.type=="gene"){
    if(is.null(id.type)) id.type="KEGG"
    id.type=toupper(id.type)
    if(id.type=="ENTREZ") id.type="ENTREZID"
      kid.map=names(species.data)[-c(1:2)]
      kid.types=names(kid.map)=c("KEGG", "ENTREZID", "NCBIPROT", "UNIPROT")
      kid.map2=gsub("[.]", "-", kid.map)
      kid.map2["UNIPROT"]="up"

    data(bods)
    data(gene.idtype.bods)
    org19=bods[,"kegg code"]

    kegg.mapping=F
    if(species %in% org19){
      if(!id.type %in% gene.idtype.bods[[species]]) kegg.mapping=T
     } else if (species != "ko") kegg.mapping=T
#      print(kegg.mapping)

    if(kegg.mapping){#!species %in% c(org19, "ko") | id.type %in% kid.types[c(1,3)]){
      if(!id.type %in% kid.types){
        msg=sprintf(msg.fmt, id.type, species, mol.type)
        stop(msg)
      }
      if(is.na(species.data[kid.map[id.type]])){
        if(!is.na(species.data["kegg.geneid"])){
          msg.fmt3="Only native KEGG gene ID is supported for species \"%s\"! Use KEGG ID instead."
          msg=sprintf(msg.fmt3, species)
          message("Note: ", msg)
          id.type="KEGG"
        } else{
          msg.fmt3="Simulation is not supported for species \"%s\"!"
          msg=sprintf(msg.fmt3, species)
          stop(msg)
        }
      }

         if(id.type=="KEGG") {
           gid.map=keggList(species)
           all.mn=gsub(paste(species, ":", sep=""), "", names(gid.map))
         } else {
           gid.map=keggConv(kid.map2[id.type],species)
           all.mn=gsub(paste0(kid.map2[id.type],":"), "", gid.map)
         }
    } else if(species %in% org19){
#      if(id.type=="ENTREZ") id.type="ENTREZID"
      if(id.type=="KEGG") {
        gid.map=keggConv("ncbi-geneid",species)        
        all.mn=gsub(paste(species, ":", sep=""), "", names(gid.map))
      } else if(id.type %in% gene.idtype.bods[[species]]){
        idx=which(bods[,3]==species)
        pkg.name=bods[idx,1]
        pkg.on=requireNamespace(pkg.name)
        if(!pkg.on) {
#          source("http://bioconductor.org/biocLite.R")
#          biocLite(pkg.name, suppressUpdates =TRUE)
          BiocManager::install(pkg.name, update=FALSE)
          pkg.on=requireNamespace(pkg.name)
          if(!pkg.on) stop(paste("Fail to install/load gene annotation package ", pkg.name, "!",  sep=""))
        }
        
        db.obj <- eval(parse(text=paste0(pkg.name, "::", pkg.name)))
        all.mn <-keys(db.obj, keytype=id.type)
      } else stop("Wrong gene ID type!")
    }            
  } else if(mol.type=="cpd"){
    data(cpd.accs)
    data(cpd.simtypes)
    data(rn.list)
    accn=cpd.accs$ACCESSION_NUMBER
    if(is.null(id.type)) id.type="KEGG COMPOUND accession"
    if(!id.type %in% cpd.simtypes){
      msg=sprintf(msg.fmt, id.type, mol.type)
      stop(msg)
    }
    all.mn=unique(as.character(accn[rn.list[[id.type]]]))
  } else if(mol.type=="gene.ko"){
    data(ko.ids)
    all.mn=ko.ids
  } else stop("Invalid mol.type!") 

  nuids=length(all.mn)
  if(nmol>nuids){
    msg=sprintf(msg.fmt2, id.type, nuids)
    message("Note: ", msg)
    nmol=nuids
  }
  sel.mn=sample(all.mn, nmol)
  if(discrete) return(sel.mn)
  sel.mn.data=matrix(rnorm(nmol*nexp), ncol=nexp)
  rownames(sel.mn.data)=sel.mn
  colnames(sel.mn.data)=paste("exp", 1:nexp, sep="")
  return(sel.mn.data[, 1:nexp])
}

keggview.native <-
function(
                         plot.data.gene=NULL,
                          plot.data.cpd=NULL,
                         cols.ts.gene=NULL,
                          cols.ts.cpd=NULL,
                         node.data,
                         pathway.name,
                           out.suffix="pathview",
                         kegg.dir=".",

                          multi.state=TRUE,
                          match.data=TRUE,
                           same.layer=TRUE, #
                         res=300, #
                         cex = 0.25,#

         discrete=list(gene=FALSE, cpd=FALSE),
         limit=list(gene=1, cpd=1),
                         bins=list(gene=10, cpd=10),
                         both.dirs=list(gene=T, cpd=T),
         low = list(gene = "green", cpd = "blue"),
                         mid = list(gene = "gray", cpd = "gray"),
                         high = list(gene = "red", cpd = "yellow"),
         na.col="transparent",
#         na.col="white",
         
                         new.signature=TRUE,
                         plot.col.key=TRUE,
                         key.align="x",
                         key.pos="topright",
#                         sign.pos="bottomright",#g
                         ...){

#read image  
  img <- readPNG(paste(kegg.dir, "/", pathway.name, ".png", 
                       sep = ""))
  width <- ncol(img)
  height <- nrow(img)

  cols.ts.gene=cbind(cols.ts.gene)
  cols.ts.cpd=cbind(cols.ts.cpd)
  nc.gene=max(ncol(cols.ts.gene),0)
  nc.cpd=max(ncol(cols.ts.cpd),0)#@
  nplots=max(nc.gene,nc.cpd)
  pn.suffix=colnames(cols.ts.gene)
  if(length(pn.suffix)<nc.cpd)  pn.suffix=colnames(cols.ts.cpd)
  if(length(pn.suffix)<nplots)  pn.suffix=1:nplots #no column names for both datasets
  if(length(pn.suffix)==1) {
    pn.suffix=out.suffix
  } else pn.suffix=paste(out.suffix, pn.suffix, sep=".")

     na.col=colorpanel2(1, low=na.col, high=na.col)
  if((match.data | !multi.state) & nc.gene!=nc.cpd){
#  if(nc.gene>nc.cpd) cols.ts.cpd= cols.ts.cpd[, rep(1:nc.cpd, nplots)[1:nplots]]
#  if(nc.gene<nc.cpd) cols.ts.gene= cols.ts.gene[, rep(1:nc.gene, nplots)[1:nplots]]

     if(nc.gene>nc.cpd & !is.null(cols.ts.cpd)){
      na.mat=matrix(na.col, ncol=nplots-nc.cpd, nrow=nrow(cols.ts.cpd))
      cols.ts.cpd= cbind(cols.ts.cpd, na.mat)
    }
    if(nc.gene<nc.cpd & !is.null(cols.ts.gene)){
      na.mat=matrix(na.col, ncol=nplots-nc.gene, nrow=nrow(cols.ts.gene))
      cols.ts.gene= cbind(cols.ts.gene, na.mat)
    }
    nc.gene=nc.cpd=nplots
  }
  
  out.fmt="Working in directory %s"
  wdir=getwd()
  out.msg=sprintf(out.fmt, wdir)
  message("Info: ", out.msg)
  out.fmt="Writing image file %s"

    multi.state=multi.state & nplots>1
if(multi.state) {
  nplots=1
  pn.suffix=paste(out.suffix, "multi", sep=".")
  if(nc.gene>0) cols.gene.plot=cols.ts.gene
  if(nc.cpd>0) cols.cpd.plot=cols.ts.cpd
}

for(np in 1:nplots){
#plot setup
 img.file =paste(pathway.name,pn.suffix[np],"png", sep=".")
 out.msg=sprintf(out.fmt, img.file)
 message("Info: ", out.msg)
  png(img.file, width = width, height = height, res=res)

  op=par(mar = c(0, 0, 0, 0))
  plot(c(0, width), c(0, height), type = "n", xlab = "", ylab = "",xaxs = "i",yaxs = "i")
  if(new.signature) img[height-4:25, 17:137, 1:3]=1
  if(same.layer!=T)  rasterImage(img, 0, 0, width, height, interpolate = F)
  
if(!is.null(cols.ts.gene) & nc.gene>=np){
    if(!multi.state) cols.gene.plot=cols.ts.gene[,np]
    if(same.layer!=T){
      render.kegg.node(plot.data.gene, cols.gene.plot, img, same.layer=same.layer, type="gene", cex=cex)
  } else{
  img=render.kegg.node(plot.data.gene, cols.gene.plot, img, same.layer=same.layer, type="gene")
  }
} 

if(!is.null(cols.ts.cpd) & nc.cpd>=np){
    if(!multi.state) cols.cpd.plot=cols.ts.cpd[,np]
  if(same.layer!=T){
      render.kegg.node(plot.data.cpd, cols.cpd.plot, img, same.layer=same.layer, type="compound", cex=cex)
  } else{
  img=render.kegg.node(plot.data.cpd, cols.cpd.plot, img, same.layer=same.layer, type="compound")
  }
}
  
  if(same.layer==T)  rasterImage(img, 0, 0, width, height, interpolate = F)

  pv.pars=list()
  pv.pars$gsizes=c(width=width, height=height)
  pv.pars$nsizes=c(46,17)
  pv.pars$op=op
  pv.pars$key.cex=2.*72/res
  pv.pars$key.lwd=1.2*72/res
  pv.pars$sign.cex=cex
  off.sets=c(x=0,y=0)
  align="n"

# na.col=colorpanel2(1, low=na.col, high=na.col)
 ucol.gene=unique(as.vector(cols.ts.gene))
 na.col.gene=ucol.gene %in% c(na.col, NA)

  if(plot.col.key & !is.null(cols.ts.gene) & !all(na.col.gene))  {
    off.sets=col.key(limit=limit$gene, bins=bins$gene, both.dirs=both.dirs$gene, discrete=discrete$gene, graph.size=pv.pars$gsizes,
      node.size=pv.pars$nsizes, key.pos=key.pos, cex=pv.pars$key.cex, lwd=pv.pars$key.lwd, low=low$gene, mid=mid$gene, high=high$gene, align="n")
    align=key.align
    
  }
  
 ucol.cpd=unique(as.vector(cols.ts.cpd))
 na.col.cpd=ucol.cpd %in% c(na.col, NA)
  if(plot.col.key & !is.null(cols.ts.cpd) & !all(na.col.cpd)) {
    off.sets=col.key(limit=limit$cpd, bins=bins$cpd, both.dirs=both.dirs$cpd, discrete=discrete$cpd, graph.size=pv.pars$gsizes, node.size=pv.pars$nsizes, key.pos=key.pos, off.sets=off.sets, cex=pv.pars$key.cex, lwd=pv.pars$key.lwd, low=low$cpd, mid=mid$cpd, high=high$cpd, align=align)
  }
  
  if(new.signature) pathview.stamp(x=17, y=20, on.kegg=T, cex = pv.pars$sign.cex)
  par(pv.pars$op)
  dev.off()
}
  
  return(invisible(pv.pars))
}

geneannot.map <- function(in.ids, in.type, out.type, org="Hs", pkg.name=NULL, unique.map=TRUE, na.rm=TRUE, keep.order=TRUE){
  if(is.null(pkg.name)) {#pkg.name=paste("org", org, "eg.db", sep=".")
    data(bods)
    ridx=grep(tolower(paste0(org, "[.]")), tolower(bods[,1]))
    if(length(ridx)==0) {
      ridx=grep(tolower(org), tolower(bods[,2:3])) %% nrow(bods)
      if(length(ridx)==0) stop("Wrong org value!")
      if(any(ridx==0)) ridx[ridx==0]=nrow(bods)
    }
    pkg.name=bods[ridx,1]
  }

  pkg.on=try(requireNamespace(pkg.name),silent = TRUE)
  if(!pkg.on) {
#    source("http://bioconductor.org/biocLite.R")
#    biocLite(pkg.name, suppressUpdates =TRUE)
    BiocManager::install(pkg.name, update=FALSE)
    pkg.on=try(requireNamespace(pkg.name),silent = TRUE)
    if(!pkg.on) stop(paste("Fail to install/load gene annotation package ", pkg.name, "!",  sep=""))
  }
  
  
  db.obj <- eval(parse(text=paste0(pkg.name, "::", pkg.name)))
  id.types <- columns(db.obj) #columns(eval(as.name(pkg.name)))

  in.type=toupper(in.type)
  out.type=toupper(out.type)
  eii=in.type==toupper("entrez") | in.type==toupper("eg")
  if(any(eii)) in.type[eii]="ENTREZID"
  eio=out.type==toupper("entrez") | out.type==toupper("eg")
  if(any(eio)) out.type[eio]="ENTREZID"
  if(all(in.type==out.type)) stop("in.type and out.type are the same, no need to map!")
  
  nin=length(in.type)
  if(nin!=1) stop("in.type must be of length 1!")
  out.type=out.type[!out.type %in% in.type]
  nout=length(out.type)
  
  msg <-  paste0("must from: ", paste(id.types, collapse=", "), "!")
  if (! in.type %in% id.types) stop("'in.type' ", msg)
  if (! all(out.type %in% id.types)) stop("'out.type' ", msg)

  in.ids0=in.ids
  in.ids <- unique(as.character(in.ids))#unique necessary for select()# if(unique.map)
  out.ids=character(length(in.ids))
  res <- try(suppressWarnings(AnnotationDbi::select(db.obj,
                                     keys = in.ids,
                                     keytype = in.type,
                                     columns=c(in.type, out.type))))
  if(class(res)[1]=="data.frame"){
    
    res <- res[, c(in.type, out.type)]

    if(nout==1) na.idx <- is.na(res[,2])
    else na.idx <- apply(res[,-1],1,function(x) all(is.na(x)))
    if (sum(na.idx)>0) {
      n.na <- length(unique(res[na.idx, 1]))
      if (n.na>0) {
        print(paste("Note:", n.na, "of", length(in.ids), "unique input IDs unmapped."))
      }
      if (na.rm) res <- res[!na.idx, ]
    }

    cns=colnames(res)
    if(unique.map){
      if(length(out.type)==1)  umaps=tapply(res[,out.type], res[,in.type], paste, sep="", collapse="; ")
      else umaps=apply(res[,out.type], 2, function(x) tapply(x, res[,in.type], function(y) paste(unique(y), sep="", collapse="; ")))
      umaps=cbind(umaps)
      res.uniq=cbind(rownames(umaps), umaps)
      res=res.uniq
      colnames(res)=cns
    }

    res=as.matrix(res)
    if(!keep.order){
      rownames(res)=NULL
      return(res)
    } else {
      res1=matrix(NA, ncol=length(cns), nrow=length(in.ids0))
      res1[,1]=in.ids0
      rns=match(in.ids0, res[,1])
      res1[,-1]=res[rns,-1]
      colnames(res1)=cns
      return(res1)
    }
  } else{
    res=cbind(in.ids,out.ids)
    colnames(res)=c(in.type,out.type)
    return(res)
  }
}

combineKEGGnodes <-
function (nodes, graph, combo.node)
  {
    nodes=unique(nodes)
    nodes.all=nodes(graph)
    ii=nodes %in% nodes.all
    if(!all(ii)){
      anodes=nodes[!ii]
      stop(paste("Nodes ", paste(anodes, collapse=", "), " are not part of the graph!", sep=""))
    }



    knode=getKEGGnodeData(graph)
    kedge=getKEGGedgeData(graph)
    nodeNames=sapply(knode, getName)
    nodeType=sapply(knode, getType)
    nodeComp=sapply(knode, getComponent)
    node.size=sapply(nodeComp, length)
    grp.nodes=nodes.all[node.size>1]

    nt=unique(nodeType[nodes])
    len.nt=length(nt)
    if("ortholog" %in% nt) len.nt=len.nt-1
    if(len.nt>1) {
      warn.msg="Nodes are not the same type, hence unable to combine!"
      message("Warning: ", warn.msg)
      return(graph)
    }
    
    if(combo.node %in% grp.nodes){

      combgraph <- combineNodes(c(nodes,combo.node), graph, combo.node)
      gnode=knode[[combo.node]]
      ng.component= unique(c(gnode@component, nodes))
      ng.graphics= gnode@graphics
      ng.graphics@height=as.integer(length(ng.component)/length(gnode@component)*ng.graphics@height)
    } else{
      if(combo.node %in% nodes.all){
        combgraph <- combineNodes(c(nodes,combo.node), graph, combo.node)
        ng.component= unique(c(combo.node, nodes))
        ng.graphics= knode[[combo.node]]@graphics
        ng.graphics@height=as.integer(length(ng.component)*ng.graphics@height)
      } else{
        combgraph <- combineNodes(nodes, graph, combo.node)
        ng.component= unique(nodes)
        ng.graphics= knode[[nodes[1]]]@graphics
        ng.graphics@height=as.integer(length(ng.component)*ng.graphics@height)
      }
    }

    ng.entryID= combo.node
    ng.name= "undefined"
    ng.type= "group"
    ng.link= as.character(NA)
    ng.reaction= as.character(NA)
    ng.map= as.character(NA)
    new.gnode <- new("KEGGGroup", component = ng.component, entryID = ng.entryID,
                     name = ng.name, type = ng.type, link = ng.link, reaction = ng.reaction,
                     map = ng.map, graphics = ng.graphics)

    knode.new=knode[!nodes.all %in% nodes]
    knode.new[[combo.node]]=new.gnode

    kedge.new=kedge
    enames=names(kedge)
    eeids=sapply(strsplit(enames, "~"), function(x) x)
    idx1=eeids[1,] %in% nodes
    idx2=eeids[2,] %in% nodes
    idx=idx1 | idx2
    for(i in which(idx1)) kedge.new[[i]]@entry1ID=eeids[1,i]=combo.node
    for(i in which(idx2)) kedge.new[[i]]@entry2ID=eeids[2,i]=combo.node
    names(kedge.new)[idx]=paste(eeids[1,idx],eeids[2,idx], sep = "~")
    
    env.node <- new.env()
    env.edge <- new.env()
    assign("nodes", knode.new, envir = env.node)
    assign("edges", kedge.new, envir = env.edge)
    nodeDataDefaults(combgraph, "KEGGNode") <- env.node
    edgeDataDefaults(combgraph, "KEGGEdge") <- env.edge
    return(combgraph)
  }

pathview.stamp <-
function(x=NULL, y=NULL, position="bottomright", graph.sizes, on.kegg=TRUE, cex=1){
if(on.kegg)    labels ="Data on KEGG graph\nRendered by Pathview"
else labels="-Data with KEGG pathway-\n-Rendered  by  Pathview-"
if(is.null(x)| is.null(y)){
  x=graph.sizes[1]*.80
  y=graph.sizes[2]/40
  if(length(grep('left',position))==1)  x=graph.sizes[1]/40
  if(length(grep('top', position))==1)  y=graph.sizes[2]-y
}
text(x=x, y=y, labels=labels, adj=0, cex = cex, font=2)
}

ellipses <-
function(x0,y0,w,h,n=100, cols, ...){
  np=length(x0)
  x=seq(-w,w,length=n/2)
  y=(1-(x/w)^2)^.5*h
  x=c(x,rev(x))
  y=c(y,-rev(y))
  x0=matrix(rep(x0, each=n), nrow=n)
  y0=matrix(rep(y0, each=n), nrow=n)
  x <- x0 + x
  y <- y0 + y
  for(i in 1:np) polygon(x[,i], y[,i], col=cols[i], ...)
  return(list(x,y))
}

### R code from vignette source 'pathview.Rnw'

###################################################
### code chunk number 1: synopsis1 (eval = FALSE)
###################################################
## library(pathview)
## data(gse16873.d)
## pv.out <- pathview(gene.data = gse16873.d[, 1], pathway.id = "04110",
##                    species = "hsa", out.suffix = "gse16873")


###################################################
### code chunk number 2: install (eval = FALSE)
###################################################
## source("http://bioconductor.org/biocLite.R")
## biocLite("pathview")


###################################################
### code chunk number 3: <install (eval = FALSE)
###################################################
## source("http://bioconductor.org/biocLite.R")
## biocLite(c("Rgraphviz", "png", "KEGGgraph", "org.Hs.eg.db"))


###################################################
### code chunk number 4: install (eval = FALSE)
###################################################
## install.packages("pathview",repos="http://R-Forge.R-project.org")


###################################################
### code chunk number 5: install (eval = FALSE)
###################################################
## install.packages("/your/local/directory/pathview_1.0.0.tar.gz", 
##     repos = NULL, type = "source")


###################################################
### code chunk number 6: <install (eval = FALSE)
###################################################
## install.packages("/your/local/directory/XML_3.95-0.2.zip", repos = NULL)


###################################################
### code chunk number 7: start
###################################################
options(width=80)


###################################################
### code chunk number 8: start
###################################################
library(pathview)


###################################################
### code chunk number 9: start (eval = FALSE)
###################################################
## library(help=pathview)


###################################################
### code chunk number 10: start (eval = FALSE)
###################################################
## help(pathview)
## ?pathview


###################################################
### code chunk number 11: dataPrep.gse16873
###################################################
data(gse16873.d)


###################################################
### code chunk number 12: readin (eval = FALSE)
###################################################
## filename=system.file("extdata/gse16873.demo", package = "pathview")
## gse16873=read.delim(filename, row.names=1)
## gse16873.d=gse16873[,2*(1:6)]-gse16873[,2*(1:6)-1]


###################################################
### code chunk number 13: dataPrep.demo.paths
###################################################
data(demo.paths)


###################################################
### code chunk number 14: dataPreppaths.hsa
###################################################
data(paths.hsa)
head(paths.hsa,3)


###################################################
### code chunk number 15: kegg.native
###################################################
i <- 1
pv.out <- pathview(gene.data = gse16873.d[, 1], pathway.id = demo.paths$sel.paths[i],
                   species = "hsa", out.suffix = "gse16873", kegg.native = T)
list.files(pattern="hsa04110", full.names=T)
str(pv.out)
head(pv.out$plot.data.gene)


###################################################
### code chunk number 16: kegg.native_2layer
###################################################
pv.out <- pathview(gene.data = gse16873.d[, 1], pathway.id = demo.paths$sel.paths[i],
                   species = "hsa", out.suffix = "gse16873.2layer", kegg.native = T,
                   same.layer = F)


###################################################
### code chunk number 17: graphviz
###################################################
pv.out <- pathview(gene.data = gse16873.d[, 1], pathway.id = demo.paths$sel.paths[i],
                   species = "hsa", out.suffix = "gse16873", kegg.native = F,
                   sign.pos = demo.paths$spos[i])
#pv.out remains the same
dim(pv.out$plot.data.gene)
head(pv.out$plot.data.gene)


###################################################
### code chunk number 18: graphviz.2layer
###################################################
pv.out <- pathview(gene.data = gse16873.d[, 1], pathway.id = demo.paths$sel.paths[i], 
    species = "hsa", out.suffix = "gse16873.2layer", kegg.native = F, 
    sign.pos = demo.paths$spos[i], same.layer = F)


###################################################
### code chunk number 19: split
###################################################
pv.out <- pathview(gene.data = gse16873.d[, 1], pathway.id = demo.paths$sel.paths[i], 
    species = "hsa", out.suffix = "gse16873.split", kegg.native = F, 
    sign.pos = demo.paths$spos[i], split.group = T)
dim(pv.out$plot.data.gene)
head(pv.out$plot.data.gene)


###################################################
### code chunk number 20: expanded
###################################################
pv.out <- pathview(gene.data = gse16873.d[, 1], pathway.id = demo.paths$sel.paths[i], 
    species = "hsa", out.suffix = "gse16873.split.expanded", kegg.native = F,
    sign.pos = demo.paths$spos[i], split.group = T, expand.node = T)
dim(pv.out$plot.data.gene)
head(pv.out$plot.data.gene)


###################################################
### code chunk number 21: dataPrep.sim.cpd
###################################################
sim.cpd.data=sim.mol.data(mol.type="cpd", nmol=3000)
data(cpd.simtypes)


###################################################
### code chunk number 22: gene_cpd.data
###################################################
i <- 3
print(demo.paths$sel.paths[i])
pv.out <- pathview(gene.data = gse16873.d[, 1], cpd.data = sim.cpd.data, 
    pathway.id = demo.paths$sel.paths[i], species = "hsa", out.suffix = "gse16873.cpd", 
    keys.align = "y", kegg.native = T, key.pos = demo.paths$kpos1[i])
str(pv.out)
head(pv.out$plot.data.cpd)


###################################################
### code chunk number 23: graphviz.gene_cpd.data
###################################################
pv.out <- pathview(gene.data = gse16873.d[, 1], cpd.data = sim.cpd.data, 
    pathway.id = demo.paths$sel.paths[i], species = "hsa", out.suffix = "gse16873.cpd", 
    keys.align = "y", kegg.native = F, key.pos = demo.paths$kpos2[i], 
    sign.pos = demo.paths$spos[i], cpd.lab.offset = demo.paths$offs[i])


###################################################
### code chunk number 24: sim.cpd.data2
###################################################
set.seed(10)
sim.cpd.data2 = matrix(sample(sim.cpd.data, 18000, 
    replace = T), ncol = 6)
rownames(sim.cpd.data2) = names(sim.cpd.data)
colnames(sim.cpd.data2) = paste("exp", 1:6, sep = "")
head(sim.cpd.data2, 3)


###################################################
### code chunk number 25: multisample.gene_cpd.data
###################################################
#KEGG view
pv.out <- pathview(gene.data = gse16873.d[, 1:3], 
    cpd.data = sim.cpd.data2[, 1:2], pathway.id = demo.paths$sel.paths[i], 
    species = "hsa", out.suffix = "gse16873.cpd.3-2s", keys.align = "y", 
    kegg.native = T, match.data = F, multi.state = T, same.layer = T)
head(pv.out$plot.data.cpd)
#KEGG view with data match
pv.out <- pathview(gene.data = gse16873.d[, 1:3], 
    cpd.data = sim.cpd.data2[, 1:2], pathway.id = demo.paths$sel.paths[i], 
    species = "hsa", out.suffix = "gse16873.cpd.3-2s.match", 
    keys.align = "y", kegg.native = T, match.data = T, multi.state = T, 
    same.layer = T)
#graphviz view
pv.out <- pathview(gene.data = gse16873.d[, 1:3], 
    cpd.data = sim.cpd.data2[, 1:2], pathway.id = demo.paths$sel.paths[i], 
    species = "hsa", out.suffix = "gse16873.cpd.3-2s", keys.align = "y", 
    kegg.native = F, match.data = F, multi.state = T, same.layer = T,
    key.pos = demo.paths$kpos2[i], sign.pos = demo.paths$spos[i])


###################################################
### code chunk number 26: multisample.gene_cpd.data.seperate (eval = FALSE)
###################################################
## #plot samples/states separately
## pv.out <- pathview(gene.data = gse16873.d[, 1:3], 
##     cpd.data = sim.cpd.data2[, 1:2], pathway.id = demo.paths$sel.paths[i], 
##     species = "hsa", out.suffix = "gse16873.cpd.3-2s", keys.align = "y", 
##     kegg.native = T, match.data = F, multi.state = F, same.layer = T)


###################################################
### code chunk number 27: multisample.gene_cpd.data.2layer (eval = FALSE)
###################################################
## pv.out <- pathview(gene.data = gse16873.d[, 1:3], 
##     cpd.data = sim.cpd.data2[, 1:2], pathway.id = demo.paths$sel.paths[i], 
##     species = "hsa", out.suffix = "gse16873.cpd.3-2s.2layer", 
##     keys.align = "y", kegg.native = T, match.data = F, multi.state = T, 
##     same.layer = F)


###################################################
### code chunk number 28: discrete.gene_cpd.data
###################################################
require(org.Hs.eg.db)
gse16873.t <- apply(gse16873.d, 1, function(x) t.test(x, 
    alternative = "two.sided")$p.value)
sel.genes <- names(gse16873.t)[gse16873.t < 0.1]
sel.cpds <- names(sim.cpd.data)[abs(sim.cpd.data) > 0.5]
pv.out <- pathview(gene.data = sel.genes, cpd.data = sel.cpds, 
    pathway.id = demo.paths$sel.paths[i], species = "hsa", out.suffix = "sel.genes.sel.cpd", 
    keys.align = "y", kegg.native = T, key.pos = demo.paths$kpos1[i], 
    limit = list(gene = 5, cpd = 2), bins = list(gene = 5, cpd = 2), 
    na.col = "gray", discrete = list(gene = T, cpd = T))
pv.out <- pathview(gene.data = sel.genes, cpd.data = sim.cpd.data, 
    pathway.id = demo.paths$sel.paths[i], species = "hsa", out.suffix = "sel.genes.cpd",
    keys.align = "y", kegg.native = T, key.pos = demo.paths$kpos1[i], 
    limit = list(gene = 5, cpd = 1), bins = list(gene = 5, cpd = 10), 
    na.col = "gray", discrete = list(gene = T, cpd = F))


###################################################
### code chunk number 29: gene.ensprot_cpd.cas
###################################################
cpd.cas <- sim.mol.data(mol.type = "cpd", id.type = cpd.simtypes[2], 
    nmol = 10000)
gene.ensprot <- sim.mol.data(mol.type = "gene", id.type = gene.idtype.list[4], 
    nmol = 50000)
pv.out <- pathview(gene.data = gene.ensprot, cpd.data = cpd.cas, 
    gene.idtype = gene.idtype.list[4], cpd.idtype = cpd.simtypes[2], 
    pathway.id = demo.paths$sel.paths[i], species = "hsa", same.layer = T, 
    out.suffix = "gene.ensprot.cpd.cas", keys.align = "y", kegg.native = T, 
    key.pos = demo.paths$kpos2[i], sign.pos = demo.paths$spos[i], 
    limit = list(gene = 3, cpd = 3), bins = list(gene = 6, cpd = 6))


###################################################
### code chunk number 30: gene.ensprot_cpd.cas.manual.map
###################################################
id.map.cas <- cpdidmap(in.ids = names(cpd.cas), in.type = cpd.simtypes[2], 
    out.type = "KEGG COMPOUND accession")
cpd.kc <- mol.sum(mol.data = cpd.cas, id.map = id.map.cas)
id.map.ensprot <- id2eg(ids = names(gene.ensprot), 
    category = gene.idtype.list[4], org = "Hs")
gene.entrez <- mol.sum(mol.data = gene.ensprot, id.map = id.map.ensprot)
pv.out <- pathview(gene.data = gene.entrez, cpd.data = cpd.kc, 
    pathway.id = demo.paths$sel.paths[i], species = "hsa", same.layer = T, 
    out.suffix = "gene.entrez.cpd.kc", keys.align = "y", kegg.native = T, 
    key.pos = demo.paths$kpos2[i], sign.pos = demo.paths$spos[i], 
    limit = list(gene = 3, cpd = 3), bins = list(gene = 6, cpd = 6))


###################################################
### code chunk number 31: korg
###################################################
data(korg)
head(korg)
#number of species which use Entrez Gene as the default ID
sum(korg[,"entrez.gnodes"]=="1",na.rm=T)
#number of species which use other ID types or none as the default ID
sum(korg[,"entrez.gnodes"]=="0",na.rm=T)
#new from 2017: most species which do not have Entrez Gene annotation any more
na.idx=is.na(korg[,"ncbi.geneid"])
sum(na.idx)


###################################################
### code chunk number 32: bods_gene.idtype.list
###################################################
data(bods)
bods
data(gene.idtype.list)
gene.idtype.list


###################################################
### code chunk number 33: eco.dat.kegg
###################################################
eco.dat.kegg <- sim.mol.data(mol.type="gene",id.type="kegg",species="eco",nmol=3000)
head(eco.dat.kegg)
pv.out <- pathview(gene.data = eco.dat.kegg, gene.idtype="kegg",
    pathway.id = "00640", species = "eco", out.suffix = "eco.kegg",
    kegg.native = T, same.layer=T)


###################################################
### code chunk number 34: eco.dat.entrez (eval = FALSE)
###################################################
## eco.dat.entrez <- sim.mol.data(mol.type="gene",id.type="entrez",species="eco",nmol=3000)
## head(eco.dat.entrez)
## pv.out <- pathview(gene.data = eco.dat.entrez, gene.idtype="entrez",
##     pathway.id = "00640", species = "eco", out.suffix = "eco.entrez",
##     kegg.native = T, same.layer=T)


###################################################
### code chunk number 35: eco.dat.symbol (eval = FALSE)
###################################################
## egid.eco=eg2id(names(eco.dat.entrez), category="symbol", pkg="org.EcK12.eg.db")
## eco.dat.symbol <- eco.dat.entrez
## names(eco.dat.symbol) <- egid.eco[,2]
## head(eco.dat.kegg)
## pv.out <- pathview(gene.data = eco.dat.symbol, gene.idtype="symbol",
##     pathway.id = "00640", species = "eco", out.suffix = "eco.symbol.2layer",
##     kegg.native = T, same.layer=F)


###################################################
### code chunk number 36: gene.ensprot_cpd.cas.manual.map
###################################################
ko.data=sim.mol.data(mol.type="gene.ko", nmol=5000)
pv.out <- pathview(gene.data = ko.data, pathway.id = "04112",
                   species = "ko", out.suffix = "ko.data", kegg.native = T)


###################################################
### code chunk number 37: GAGE.Pathview.pipeline (eval = FALSE)
###################################################
## library(gage)
## data(gse16873)
## cn <- colnames(gse16873)
## hn <- grep('HN',cn, ignore.case =TRUE)
## dcis <- grep('DCIS',cn, ignore.case =TRUE)
## data(kegg.gs)
## #pathway analysis using gage
## gse16873.kegg.p <- gage(gse16873, gsets = kegg.gs,
##     ref = hn, samp = dcis)
## #prepare the differential expression data
## gse16873.d <- gagePrep(gse16873, ref = hn, samp = dcis)
## #equivalently, you can do simple subtraction for paired samples
## gse16873.d <- gse16873[,dcis]-gse16873[,hn]
## #select significant pathways and extract their IDs
## sel <- gse16873.kegg.p$greater[, "q.val"] < 0.1 & !is.na(gse16873.kegg.p$greater[, 
##     "q.val"])
## path.ids <- rownames(gse16873.kegg.p$greater)[sel]
## path.ids2 <- substr(path.ids[c(1, 2, 7)], 1, 8)
## #pathview visualization
## pv.out.list <- pathview(gene.data = gse16873.d[, 1:3], pathway.id = path.ids2, 
##     species = "hsa")

circles <-
function(x0,y0,r,n=100, cols, ...){
  theta <- seq(0, 2 * pi, length=n)
  x=matrix(rep(x0, each=n), nrow=n)
  y=matrix(rep(y0, each=n), nrow=n)
  x <- x + r * cos(theta)
  y <- y + r * sin(theta)
  for(i in 1:length(x0)) polygon(x[,i], y[,i], col=cols[i], ...)
  return(list(x,y))
}

mol.sum <-
function(mol.data, id.map, gene.annotpkg="org.Hs.eg.db", sum.method=c("sum","mean", "median", "max", "max.abs", "random")[1]){
  if(is.character(mol.data)){
    gd.names=mol.data
    mol.data=rep(1, length(mol.data))
    names(mol.data)=gd.names
    ng=length(mol.data)
  } else if(!is.null(mol.data)){
    if(length(dim(mol.data))==2){
    gd.names=rownames(mol.data)
    ng=nrow(mol.data)
    } else if(is.numeric(mol.data) & is.null(dim(mol.data))){
    gd.names=names(mol.data)
    ng=length(mol.data)
    } else stop("wrong mol.data format!")
  } else stop("NULL mol.data!")

  if(is.character(id.map) & length(id.map)==1){
    id.map=id2eg(gd.names, category=id.map, pkg.name=gene.annotpkg)
  }
  
  sel.idx=id.map[,2]>"" & !is.na(id.map[,2])
  id.map=id.map[sel.idx,,drop=F]
  eff.idx=gd.names %in% id.map[,1]
  mapped.ids=id.map[match(gd.names[eff.idx], id.map[,1]),2]
  if(sum(eff.idx)<1) stop("no ID can be mapped!")
  else if(sum(eff.idx)==1){
    mapped.data=cbind(mol.data)[eff.idx,,drop=F] #rbind(cbind(mol.data)[eff.idx,])
    rownames(mapped.data)=mapped.ids[1]
  }
  else{
    if(sum.method %in% c("sum","mean")){
      sum.method=eval(as.name(sum.method))
#      mapped.data=apply(cbind(cbind(mol.data)[eff.idx,]),2,function(x){      
      mapped.data=apply(cbind(mol.data)[eff.idx,,drop=F],2,function(x){
        sum.res=tapply(x, mapped.ids, sum.method, na.rm=T)
        return(sum.res)
      })
      if(length(unique(mapped.ids))==1){
        if(length(mapped.data)>1){
          mapped.data=rbind(mapped.data)
          rownames(mapped.data)=mapped.ids[1]
        }
        else names(mapped.data)=mapped.ids[1]
      }
    } else{
    sum.method=eval(as.name(sum.method))
    mol.data=cbind(mol.data)[eff.idx,,drop=F] #cbind(cbind(mol.data)[eff.idx,])
    if(all(mol.data>=0) | all(mol.data<=0)){
      vars=apply(cbind(mol.data), 1, IQR)
    } else vars=apply(cbind(mol.data), 1, sum, na.rm=T)
    
    sel.rn=tapply(1:sum(eff.idx), mapped.ids, function(x){
#    sel.rn=tapply(which(eff.idx), mapped.ids, function(x){
      if(length(x)==1) return(x)
      else return(x[which.min(abs(vars[x]-sum.method(vars[x], na.rm=T)))])
    })
#    if(length(sel.rn)>1) mapped.data=cbind(mol.data[sel.rn,])
#    else mapped.data=mol.data[sel.rn,,drop=F] #rbind(mol.data[sel.rn,])
    mapped.data=mol.data[sel.rn,,drop=F]
    rownames(mapped.data)=names(sel.rn)
  }
}
  return(mapped.data)

}

download.kegg <-
  function (pathway.id = "00010", species = "hsa", kegg.dir = ".", file.type=c("xml", "png"))
  {
    npath=length(pathway.id)
    if(species!="ko") species=kegg.species.code(species, na.rm=T)
    nspec=length(species)

    if(npath!=1 | nspec!=1) {
      species=rep(species, npath)
      pathway.id=rep(pathway.id, each=nspec)
    }
    pathway.id <- paste(species, pathway.id, sep = "")
    uidx=!duplicated(pathway.id)
    pathway.id=pathway.id[uidx]
    species=species[uidx]
    npath=length(pathway.id)
    
    xml.fnames=paste(pathway.id, ".xml", sep="")
    png.fnames=paste(pathway.id, ".png", sep="")
    ##      xml.fmt="http://www.genome.jp/kegg-bin/download?entry=%s&format=kgml"
    ##      png.fmt="http://www.genome.jp/kegg/pathway/%s/%s"
    xml.fmt="https://rest.kegg.jp/get/%s/kgml"
    png.fmt="https://rest.kegg.jp/get/%s/image"
    all.status=rep("succeed", npath)
    names(all.status)=pathway.id
    warn.fmt.xml="Download of %s xml file failed!\nThis pathway may not exist!"
    warn.fmt.png="Download of %s png file failed!\nThis pathway may not exist!"
    

    if("xml" %in% file.type){
      for (i in 1:npath) {
        msg=sprintf("Downloading xml files for %s, %d/%d pathways..", pathway.id[i], i, length(pathway.id))
        message("Info: ", msg)
        xml.url=sprintf(xml.fmt,  pathway.id[i])
        xml.target=sprintf("%s/%s", kegg.dir, xml.fnames[i])
        xml.status=try(download.file(xml.url, xml.target, quiet=T), silent=T)

        if(xml.status!=0) all.status[i]="failed"
        if(class(xml.status)[1]=="try-error"){
          warn.msg=sprintf(warn.fmt.xml, pathway.id[i])
          message("Warning: ", warn.msg)
          unlink(xml.target)
        }
      }
    }
    
    if("png" %in% file.type){
      for (i in 1:npath) {
        msg=sprintf("Downloading png files for %s, %d/%d pathways..", pathway.id[i], i, length(pathway.id))
        message("Info: ", msg)
        png.url=sprintf(png.fmt,  pathway.id[i])#species[i], png.fnames[i])
        png.target=sprintf("%s/%s", kegg.dir, png.fnames[i])
        png.status=suppressWarnings(try(download.file(png.url, png.target, quiet=T, mode="wb"), silent=T))

        if(png.status!=0) all.status[i]="failed"
        if(class(png.status)[1]=="try-error"){
          warn.msg=sprintf(warn.fmt.png, pathway.id[i])
          message("Warning: ", warn.msg)
          unlink(png.target)
        }
      }
    }

    return(all.status)
  }

utils::globalVariables("KEGGEdgeSubtype")

kegg.legend=function (type=c("both", "edge", "node")[1]) {
  if(!type %in% c("both", "edge", "node")){
    msg.fmt="type has to be one of: \"%s\", \"%s\" and \"%s\"!"
    msg=sprintf(msg.fmt, "both", "edge", "node")
    message("Note: ", msg)
    return(invisible(0))
  }
#  if (!exists("KEGGEdgeSubtype")) {
#    data(KEGGEdgeSubtype)
#  }
  KEGGEdgeSubtype <- KEGGEdgeSubtype[-7,]
  levels(KEGGEdgeSubtype$name)[13]="others/unknown"
  subtypes <- KEGGEdgeSubtype$name
  cols <- as.character(KEGGEdgeSubtype$color)
  labels <- as.character(KEGGEdgeSubtype$label)
  fontcolors <- as.character(KEGGEdgeSubtype$fontcolor)
  arrowheads <- as.character(KEGGEdgeSubtype$arrowhead)
  styles <- as.character(KEGGEdgeSubtype$style)
  ltytrans <- c(solid = 1, dashed = 2, dotted = 3)
  ltys <- ltytrans[styles]
  nst=nrow(KEGGEdgeSubtype)

  if(type=="edge"){
    top.margin=2
    xlim=c(-0.4,1.8)
    ylim=c(0, nst)
    main.txt="Edge types"
  } else if(type=="node"){
    top.margin=2
    xlim=c(-0.4,1.8)
    ylim=c(0, nst)
    main.txt="Node types"
    ntext.x=0.7
    xs=1.3
  } else {
    top.margin=3
    xlim=c(-0.2,4.0)
    ylim=c(0, nst+1)
    main.txt="KEGG diagram legend"
    ntext.x=3.1
    xs=3.7  
  }
  opar <- par(mar = c(0, 0, top.margin, 0), mgp = c(0, 0, 0), lwd=2)
  on.exit(par(opar))

  plot(1, 1, type = "n", xlim = xlim, ylim = ylim,
       axes = FALSE, xlab = "", ylab = "", main = main.txt)
  if(type %in% c("both","edge")){
    for (i in 1:nst) {
      j=nst-i
      text(0.8, j, subtypes[i], pos = 2, cex = 1.2)
      segments(1, j, 1.8, j, col = cols[i], lty = ltys[i])
      text(1.4, j+.2, labels[i], col = fontcolors[i])
      if (arrowheads[i] == "normal") {
        x=c(1.77,1.77,1.8)
        h=0.15/cos(pi/6)
        y=c(h,-h,0)+j
        polygon(x,y,col=cols[i], border=cols[i], lwd=1)
      }
      if (arrowheads[i] == "open") {
        x=c(1.78,1.78,1.8)
        h=0.1/cos(pi/6)
        y=c(h,-h,0)+j
        polygon(x,y,col=cols[i], border=cols[i], lwd=1)
      }
      if (arrowheads[i] == "tee") {
        lines(c(1.8,1.8), c(.3,-.3)+j, col = cols[i], lwd=3)
      }
    }
  }
  
  if(type %in% c("both","node")){
    ntypes=c("gene", "group", "compound", "map")
    naliases=c("protein/enzyme", "complex", "metabolite/glycan", "pathway")
    nshapes=c("rectangle", "rectangle group", "ellipse", "plaintext")
    node.types=data.frame(types=ntypes, aliases=naliases, shapes=nshapes, width=rep(46,4), height=rep(17,4))

                                        #  if(!edges.only){
    ntypes=as.character(node.types$types)
    naliases=as.character(node.types$aliases)
    nlabels=paste(ntypes, "\n(", naliases, ")", sep="")
    nshapes=as.character(node.types$shapes)
    nwidth=node.types$width/250
    nheight=node.types$height/250
    nnt=nrow(node.types)
    for (i in 1:nnt) {
      j=nst-i*3
      text(ntext.x, j, nlabels[i], pos = 2, cex = 1.2)
    }
    ys=nst-(1:4)*3
    rect.x1=rep(xs-nwidth,4)
    rect.x2=rep(xs+nwidth,4)
    rect.y1=rep(ys[1:2],c(1,3))-nheight[1:2]*c(1,2,0,2)*6
    rect.y2=rep(ys[1:2],c(1,3))+nheight[1:2]*c(1,2,2,0)*6
    rect(rect.x1,rect.y1,rect.x2,rect.y2)
    ellipses(xs,ys[3],nwidth[3],nheight[3]*6, cols=NULL)
    text(xs, ys[4], "Pathway name", cex = 1.2)
    if(type!="node"){
      text(1.0, nst+1, "Edge Types", cex = 1.2, font=2)
      text(3.0, nst+1, "Node Types", cex = 1.2, font=2)
    }
  }
}

cpd2kegg <-
function(in.ids, in.type){
#cpd id or name to kegg accession, use cpdname2kegg or cpdidmap
#  cpd.type=c(names(rn.list),"name")
data(rn.list)
cpd.type=tolower(c(names(rn.list),"name"))
  if(!tolower(in.type) %in% cpd.type) stop("Incorrect type!")
  kg.idx=grep("kegg", cpd.type)
  if(in.type %in% cpd.type[kg.idx]) stop("A native KEGG compound ID type, no need to map!")
  if(in.type=="name") {
    kg.accs=cpdname2kegg(in.ids)
  } else kg.accs=cpdidmap(in.ids, in.type=in.type, out.type="KEGG")
  
  return(kg.accs)
}

%\VignetteIndexEntry{Pathview: pathway based data integration and visualization}
%\VignetteDepends{org.Hs.eg.db}
%\VignetteSuggests{gage, org.Mm.eg.db}
%\VignettePackage{pathview}

\documentclass[11pt]{article}

\usepackage{times}
\usepackage{hyperref}
\usepackage{geometry}
\usepackage{longtable}
\usepackage[cm]{fullpage}
\usepackage[pdftex]{graphicx}
\usepackage[authoryear,round]{natbib}
\usepackage{caption}
\usepackage{subcaption}
\SweaveOpts{keep.source=TRUE,eps=FALSE,pdf=TRUE,prefix=TRUE} 

% R part
\newcommand{\R}[1]{{\texttt{#1}}}
\newcommand{\Rfunction}[1]{{\texttt{#1}}}
\newcommand{\Robject}[1]{{\texttt{#1}}}
\newcommand{\Rpackage}[1]{{\textit{#1}}}
\newcommand{\Rclass}[1]{{\textit{#1}}}
\newcommand{\Metas}[1]{{\texttt{#1}}}

%float placement
\renewcommand{\textfraction}{0.05}
\renewcommand{\topfraction}{0.95}
\renewcommand{\bottomfraction}{0.95}

\begin{document}
\setkeys{Gin}{width=0.8\textwidth}
\title{Pathview: pathway based data integration and visualization }
\author{Weijun Luo {\small(\href{mailto:luo\_weijun@yahoo.com}{luo\_weijun AT yahoo.com})}}


\maketitle

\begin{abstract}
  In this vignette, we demonstrate the \Rpackage{pathview} package as a tool set for pathway based data integration and visualization. It maps and renders user data on relevant pathway graphs. All users need is to supply their gene or compound data and specify the target pathway. \Rpackage{Pathview} automatically downloads the pathway graph data, parses the data file, maps user data to the pathway, and renders pathway graph with the mapped data. Although built as a stand-alone program, \Rpackage{pathview} may seamlessly integrate with pathway and gene set (enrichment) analysis tools for a large-scale and fully automated analysis pipeline. In this vignette, we introduce common and advanced uses of \Rpackage{pathview}. We also cover package installation, data preparation, other useful features and common application errors. In \Rpackage{gage} package, vignette \href{http://bioconductor.org/packages/release/bioc/vignettes/gage/inst/doc/RNA-seqWorkflow.pdf}{"RNA-Seq Data Pathway and Gene-set Analysis Workflows"} demonstrates GAGE/Pathview workflows on RNA-seq (and microarray) pathway analysis.
\end{abstract}


\section{Cite our work}
\label{sec:cite}
Please cite the Pathview paper formally if you use this package. This will help to support the maintenance and growth of the open source project. 
\\
\\
Weijun Luo and Cory Brouwer. Pathview: an R/Bioconductor package for pathway-based data integration and visualization. Bioinformatics, 29(14):1830-1831, 2013. \href{http://bioinformatics.oxfordjournals.org/content/29/14/1830.full}{doi: 10.1093/bioinformatics/btt285}.

\section{Quick start with demo data}
\label{sec:quick}
This is the most basic use of \Rpackage{pathview}, please check the full description below for details. Here we assume that the input data are already read in R as in the demo examples. If you are not sure how to do so, you may check \hyperref[code:readin]{Section Common uses for data visualization} or \Rpackage{gage} secondary vignette, \href{http://bioconductor.org/packages/release/bioc/vignettes/gage/inst/doc/dataPrep.pdf}{"Gene set and data preparation"}. 

<<synopsis1, echo=TRUE,  eval=FALSE>>=
library(pathview)
data(gse16873.d)
pv.out <- pathview(gene.data = gse16873.d[, 1], pathway.id = "04110",
                   species = "hsa", out.suffix = "gse16873")
@ 

If you have difficulties in using R in general, you may use the \href{https://pathview.uncc.edu/}{Pathview Web server: pathview.uncc.edu}. In addition to a user friendly access, \href{https://pathview.uncc.edu/}{the server} provides many extra useful features including \href{https://pathview.uncc.edu/example4}{a comprehensive pathway analysis workflow}. Please check \href{https://pathview.uncc.edu/overview}{the overview page} and \href{https://academic.oup.com/nar/article-lookup/doi/10.1093/nar/gkx372}{the NAR web server paper} \citep{luo:etal:2017} for details. 

Note \Rpackage{pathview} focuses on KEGG pathways, which is good for most regular analyses. If you are interested in working with other major pathway databases, including Reactome, MetaCyc, SMPDB, PANTHER, METACROP etc, you can use \href{https://github.com/datapplab/SBGNview}{\Rpackage{SBGNview}}. \href{https://github.com/datapplab/SBGNview}{\Rpackage{SBGNview}} covers 5,200 reference pathways and over 3,000 species by default, and offer more extensive graphics control, sub-pathway highlight, and more. Please check \href{https://github.com/datapplab/SBGNview}{the quick start  page} and \href{https://bioconductor.org/packages/devel/bioc/vignettes/SBGNview/inst/doc/SBGNview.Vignette.html}{the main tutorial} for details. 

\section{New features}
\label{sec:news}
\Rpackage{Pathview} ($\ge1.5.4$) provides paths.hsa data, the full list of human pathway ID/names from KEGG, as to help user specify target pathway IDs when calling \R{pathview}. Please check \hyperref[subsec:paths]{Section Common uses} for details.
\\
\\
\Rpackage{Pathview} ($\ge1.5.4$) adjust the definitions of 7 arguments for \R{pathview} function: \R{discrete, limit, bins, both.dirs, trans.fun, low, mid, high}. These arguments now accept 1- or 2-element vectors beside of 2-element lists. For example, limit=1 is equivalent to limit=list(gene=1, cpd=1), and bins=c(3, 10) is equivalent to bins=list(gene=3, cpd=10) etc. This would makes pathview easier to use.
\\
\\
\Rpackage{Pathview} ($\ge1.1.6$) can plot/integrate/compare multiple states or samples in the same graph (\hyperref[subsec:multiple]{Subsection \ref*{subsec:multiple}}).
\\
\\
\Rpackage{Pathview} ($\ge1.2.4$) work with all KEGG species (about 4800) plus KEGG Orthology (with \R{species="ko"}) (\hyperref[subsec:species]{Subsection \ref*{subsec:species}}).
\\

\section{Overview}
\label{sec:overview}
\Rpackage{Pathview} \citep{luo:etal:2013} is a stand-alone software package for pathway based data integration and visualization. This package can be divided into four functional modules: the Downloader, Parser, Mapper and Viewer. Mostly importantly, \Rpackage{pathview} maps and renders user data on relevant pathway graphs. 

\Rpackage{Pathview} generates both native KEGG view (like \hyperref[fig:1]{Figure \ref*{fig:1}} in PNG format) and Graphviz view (like \hyperref[fig:2]{Figure \ref*{fig:2}} in PDF format) for pathways (\hyperref[sec:visualization]{Section \ref*{sec:visualization}}). KEGG view keeps all the meta-data on pathways, spacial and temporal information, tissue/cell types, inputs, outputs and connections. This is important for human reading and interpretation of pathway biology. Graphviz view provides better control of node and edge attributes, better view of pathway topology, better understanding of the pathway analysis statistics. Currently only KEGG pathways are implemented. Hopefully, pathways from Reactome, NCI and other databases will be supported in the future. Notice that KEGG requires subscription for FTP access since May 2011. However, \Rpackage{Pathview} downloads individual pathway graphs and data files through API or HTTP access, which is freely available (for academic and non-commerical uses). \Rpackage{Pathview} uses \Rpackage{KEGGgraph} \citep{zhang:etal:2009} when parsing KEGG xml data files.

\Rpackage{Pathview} provides strong support for data integration (\hyperref[sec:integration]{Section \ref*{sec:integration}}). It works with: 1) essentially all types of biological data mappable to pathways, 2) over 10 types of gene or protein IDs, and 20 types of compound or metabolite IDs, 3) pathways for about 4800 species as well as KEGG orthology, 4) varoius data attributes and formats, i.e. continuous/discrete data, matrices/vectors, single/multiple samples etc.

\Rpackage{Pathview} is open source, fully automated and error-resistant. Therefore, it seamlessly integrates with pathway or gene set (enrichment) analysis tools. In \hyperref[sec:workflow]{Section \ref*{sec:workflow}}, we will show an integrated analysis using \Rpackage{pathview} with anothr the Bioconductor \Rpackage{gage} package \citep{luo:etal:2009}, available from the \href{www.bioconductor.org/packages/release/bioc/html/gage.html}{Bioconductor website}.

Note that although we use microarray data as example gene data in this vignette, Pathview is equally applicable to RNA-Seq data and other types of gene/protein centered high throughput data. The secondary vignette in \Rpackage{gage} package, \href{http://bioconductor.org/packages/release/bioc/vignettes/gage/inst/doc/RNA-seqWorkflow.pdf}{"RNA-Seq Data Pathway and Gene-set Analysis Workflows"}, demonstrates such applications.

This vignette is written by assuming the user has minimal R/Bioconductor knowledge. Some descriptions and code chunks cover very basic usage of R. The more experienced users may simply omit these parts. 


\section{Installation}
\label{sec:installation}
Assume R and Bioconductor have been correctly installed and accessible under current directory. Otherwise, please contact your system admin or follow the instructions on \href{http://www.r-project.org/}{R website} and \href{http://www.bioconductor.org/install/}{Bioconductor website}. Here I would strongly recommend users to install or upgrade to the latest verison of R (3.0.2+)/Bioconductor (2.14+) for simpler installation and better use of \Rpackage{Pathview}. You may need to update your \Rfunction{biocLite} too if you upgrade R/Biocondutor under Windows.

Start R: from Linux/Unix command line, type \verb@R@ (Enter); for Mac or Windows GUI, double click the R application icon to enter R console.

End R: type in \R{q()} when you are finished with the analysis using R, but not now.

Two options:

Simple way: install with Bioconductor installation script \R{biocLite} directly (this included all dependencies automatically too):
<<install, echo=TRUE, eval=FALSE>>=
source("http://bioconductor.org/biocLite.R")
biocLite("pathview")
@ 

Or a bit more complexer: install through R-forge or manually, but require dependence packages to be installed using Bioconductor first:
<<<install, echo=TRUE, eval=FALSE>>=
source("http://bioconductor.org/biocLite.R")
biocLite(c("Rgraphviz", "png", "KEGGgraph", "org.Hs.eg.db"))
@ 

Then install \Rpackage{pathview} through R-forge.
<<install, echo=TRUE, eval=FALSE>>=
install.packages("pathview",repos="http://R-Forge.R-project.org")
@ 

Or install manually: download \Rpackage{pathview} package (from R-forge or Bioconductor, make sure with proper version number and zip format) and save to \verb@/your/local/directory/@.
<<install, echo=TRUE, eval=FALSE>>=
install.packages("/your/local/directory/pathview_1.0.0.tar.gz", 
    repos = NULL, type = "source")
@ 

Note that there might be problems when installing \Rpackage{Rgraphviz} or \Rpackage{XML} (\Rpackage{KEGGgraph} dependency) package with outdated R/Biocondutor. \Rpackage{Rgraphviz} installation is a bit complicate with R 2.5 (Biocondutor 2.10) or earlier versions. Please check this  \href{http://pathview.r-forge.r-project.org/Rgraphviz.README}{Readme file} on \Rpackage{Rgraphviz}. On Windows systems,\Rpackage{XML} frequently needs to be installed manually. Its windows binary can be downloaded from \href{http://cran.r-project.org/web/packages/XML/index.html}{CRAN} and then:
<<<install, echo=TRUE, eval=FALSE>>=
install.packages("/your/local/directory/XML_3.95-0.2.zip", repos = NULL)
@ 

\section{Get Started}
\label{sec:start}
Under R, first we load the \Rpackage{pathview} package:

<<start, echo=FALSE>>=
options(width=80)
@ 

<<start, echo=TRUE, results=hide>>=
library(pathview)
@ 

To see a brief overview of the package:
<<start, echo=TRUE, eval=FALSE>>=
library(help=pathview)
@ 

To get help on any function (say the main function, \R{pathview}), use the \R{help} command in either one of the following two forms:
<<start, echo=TRUE, eval=FALSE>>=
help(pathview)
?pathview
@ 

\section{Common uses for data visualization}
\label{sec:visualization}
\Rpackage{Pathview} is primarily used for visualizing data on pathway graphs. \Rpackage{pathview} generates both native KEGG view (like \hyperref[fig:1]{Figure \ref*{fig:1}}) and Graphviz view (like \hyperref[fig:2]{Figure \ref*{fig:2}}). The former render user data on native KEGG pathway graphs, hence is natural and more readable for human. The latter layouts pathway graph using Graphviz engine, hence provides better control of node or edge attributes and pathway topology.

We load and look at the demo microarray data first. This is a breast cancer dataset. Here we would like to view the pair-wise gene expression changes between DCIS (disease) and HN (control) samples. Note that the microarray data are log2 transformed. Hence expression changes are log2 ratios.
<<dataPrep.gse16873, echo=TRUE>>=
data(gse16873.d)
@ 

Here we assume that the input data are already read in R. If not, you may use R functions \Rfunction{read.delim}, \Rfunction{read.table} etc to read in your data. For example, you may read in a truncated version of gse16873 and process it as below.
\label{code:readin}
<<readin, echo=TRUE, eval=FALSE>>=
filename=system.file("extdata/gse16873.demo", package = "pathview")
gse16873=read.delim(filename, row.names=1)
gse16873.d=gse16873[,2*(1:6)]-gse16873[,2*(1:6)-1]
@ 

We also load the demo pathway related data, which includes 3 pathway ids and related plotting parameters.
<<dataPrep.demo.paths, echo=TRUE>>=
data(demo.paths)
@

\label{subsec:paths}
We may also check the full list of KEGG pathway ID/names if needed. We provide human pathway IDs (in the form of hsa+5 digits) mapping to pathway names. It is almost the same for other species, excpet for the 3 or 4 letter species code. Please check \hyperref[subsec:species]{Subsection \ref*{subsec:species}} for KEGG species code.
<<dataPreppaths.hsa, echo=TRUE>>=
data(paths.hsa)
head(paths.hsa,3)
@

First, we view the exprssion changes of a single sample (pair) on a typical signaling pathway, "Cell Cycle", by specifying the \R{gene.data} and \R{pathway.id} (\hyperref[fig:1a]{Figure \ref*{fig:1a}}). The microarray was done on human tissue, hence \R{species = "hsa"}. Note that such native KEGG view was outupt as a raster image in a PNG file in your working directory.
<<kegg.native, echo=TRUE>>=
i <- 1
pv.out <- pathview(gene.data = gse16873.d[, 1], pathway.id = demo.paths$sel.paths[i],
                   species = "hsa", out.suffix = "gse16873", kegg.native = T)
list.files(pattern="hsa04110", full.names=T)
str(pv.out)
head(pv.out$plot.data.gene)
@ 


Graph from the first example above has a single layer. Node colors were modified on the original graph layer, and original KEGG node labels (node names) were kept intact. This way the output file size is as small as the original KEGG PNG file, but the computing time is relative long. If we want a fast view and do not mind doubling the output file size, we may do a two-layer graph with \R{same.layer = F}  (\hyperref[fig:1b]{Figure \ref*{fig:1b}}). This way node colors and labels are added on an extra layer above the original KEGG graph. Notice that the original KEGG gene labels (or EC numbers) were replaced by official gene symbols.
<<kegg.native_2layer, echo=TRUE>>=
pv.out <- pathview(gene.data = gse16873.d[, 1], pathway.id = demo.paths$sel.paths[i],
                   species = "hsa", out.suffix = "gse16873.2layer", kegg.native = T,
                   same.layer = F)
@ 



\setlength\fboxsep{0pt}
\setlength\fboxrule{0.5pt}
\begin{figure}
        \centering
        \begin{subfigure}[b]{1.0\textwidth}
                \centering
                \fbox{\includegraphics[width=14cm,type=png,ext=.png,read=.png]{hsa04110.gse16873}}
                \caption{}
                \label{fig:1a}
        \end{subfigure}
        \\
        \vspace{20pt}
        \begin{subfigure}[b]{1.0\textwidth}
                \centering
                \fbox{\includegraphics[width=14cm,type=png,ext=.png,read=.png]{hsa04110.gse16873.2layer}}
                \caption{}
                \label{fig:1b}
        \end{subfigure}
        \\
        \caption{Example native KEGG view on gene data with the (a) default settings; or (b) \R{same.layer=F}.}
        \label{fig:1}
\end{figure}


\begin{figure} 
\begin{center} 
\includegraphics[width=16.5cm,type=pdf,ext=.pdf,read=.pdf]{hsa04110.gse16873}
\caption{Example Graphviz view on gene data with default settings. Note that legend is put on the same page as main graph.}
\label{fig:2}
\end{center}  
\end{figure} 

In the above two examples, we view the data on native KEGG pathway graph. This view we get all notes and meta-data on the KEGG graphs, hence the data is more readable and interpretable. However, the output graph is a raster image in PNG format. We may also view the data with a \textit{de novo} pathway graph layout using Graphviz engine  (\hyperref[fig:2]{Figure \ref*{fig:2}}). The graph has the same set of nodes and edges, but with a different layout. We get more controls over the nodes and edge attributes and look. Importantly, the graph is a vector image in PDF format in your working directory. 

<<graphviz, echo=TRUE>>=
pv.out <- pathview(gene.data = gse16873.d[, 1], pathway.id = demo.paths$sel.paths[i],
                   species = "hsa", out.suffix = "gse16873", kegg.native = F,
                   sign.pos = demo.paths$spos[i])
#pv.out remains the same
dim(pv.out$plot.data.gene)
head(pv.out$plot.data.gene)
@ 

In the example above, both main graph and legend were put in one layer (or page). We just list KEGG edge types and ignore node types in legend as to save space. If we want the complete legend, we can do a Graphviz view with two layers  (\hyperref[fig:3]{Figure \ref*{fig:3}}): page 1 is the main graph, page 2 is the legend. Note that for Graphviz view (PDF file), the concept of ``layer" is slightly different from native KEGG view (PNG file). In both cases, we set argument \R{same.layer=F} for two-layer graph.
<<graphviz.2layer, echo=TRUE>>=
pv.out <- pathview(gene.data = gse16873.d[, 1], pathway.id = demo.paths$sel.paths[i], 
    species = "hsa", out.suffix = "gse16873.2layer", kegg.native = F, 
    sign.pos = demo.paths$spos[i], same.layer = F)
@ 

\begin{figure}
        \centering
        \begin{subfigure}[b]{1.0\textwidth}
                \centering
                \includegraphics[width=11cm,type=pdf,ext=.pdf,read=.pdf,page=1]{hsa04110.gse16873.2layer}
                \caption{page 1}
                \label{fig:3a}
        \end{subfigure}
        \\
        \vspace{20pt}
        \begin{subfigure}[b]{1.0\textwidth}
                \centering
                \includegraphics[width=11cm,type=pdf,ext=.pdf,read=.pdf,page=2]{hsa04110.gse16873.2layer}
                \caption{page 2}
                \label{fig:3b}
        \end{subfigure}
        \\

        \caption{Example Graphviz view on gene data with \R{same.layer=F}. Note that legend is put on a different page than main graph.}
        \label{fig:3}
\end{figure}

\begin{figure}
        \centering
        \begin{subfigure}[b]{1.0\textwidth}
                \centering
                \includegraphics[width=16.5cm,type=pdf,ext=.pdf,read=.pdf,page=1]{hsa04110.gse16873.split}
                \caption{}
                \label{fig:4a}
        \end{subfigure}
        \\
        \vspace{20pt}
        \begin{subfigure}[b]{1.0\textwidth}
                \centering
                \includegraphics[width=16.5cm,type=pdf,ext=.pdf,read=.pdf,page=1]{hsa04110.gse16873.split.expanded}
                \caption{}
                \label{fig:4b}
        \end{subfigure}
        \\
        \caption{Example Graphviz view on gene data with (a) \R{split.group = T}; or (b) \R{expand.node = T}.}
        \label{fig:4}
\end{figure}

In Graphviz view, we have more control over the graph layout. We may split the node groups into individual detached nodes  (\hyperref[fig:4a]{Figure \ref*{fig:4a}}). We may even expand the multiple-gene nodes into individual genes  (\hyperref[fig:4b]{Figure \ref*{fig:4b}}). The split nodes or expanded genes may inherit the edges from the unsplit group or unexpanded nodes. This way we tend to get a gene/protein-gene/protein interaction network. And we may better view the network characteristics (modularity etc) and gene-wise (instead of node-wise) data. Note in native KEGG view, a gene node may represent multiple genes/proteins with similar or redundant functional role. The number of member genes range from 1 up to several tens. They are intentionally put together as a single node on pathway graphs for better clarity and readability. Therefore, we do not split node and mark each member genes separately by default. But rather we visualize the node-wise data by summarize gene-wise data, users may specify the summarization method using \R{node.sum} arguement. 
<<split, echo=TRUE>>=
pv.out <- pathview(gene.data = gse16873.d[, 1], pathway.id = demo.paths$sel.paths[i], 
    species = "hsa", out.suffix = "gse16873.split", kegg.native = F, 
    sign.pos = demo.paths$spos[i], split.group = T)
dim(pv.out$plot.data.gene)
head(pv.out$plot.data.gene)
@ 

<<expanded, echo=TRUE>>=
pv.out <- pathview(gene.data = gse16873.d[, 1], pathway.id = demo.paths$sel.paths[i], 
    species = "hsa", out.suffix = "gse16873.split.expanded", kegg.native = F,
    sign.pos = demo.paths$spos[i], split.group = T, expand.node = T)
dim(pv.out$plot.data.gene)
head(pv.out$plot.data.gene)
@ 


\section{Data integration}
\label{sec:integration}
\Rpackage{Pathview} provides strong support for data integration. It can be used to integrate, analyze and visualize a wide variety of biological data (\hyperref[subsec:compound]{Subsection \ref*{subsec:compound}}): gene expression, protein expression, genetic association, metabolite, genomic data, literature, and other data types mappable to pathways. Notebaly, it can be directly used for metagenomic, microbiome or unknown species data when the data are mapped to KEGG ortholog pathways (\hyperref[subsec:species]{Subsection \ref*{subsec:species}}). The integrated Mapper module maps a variety of gene/protein IDs and compound/metabolite IDs to standard KEGG gene or compound IDs (\hyperref[subsec:ids]{Subsection \ref*{subsec:ids}}). User data named with any of these different ID types get accurately mapped to target KEGG pathways. Currently, \Rpackage{pathview} covers KEGG pathways for about 4800 species (\hyperref[subsec:species]{Subsection \ref*{subsec:species}}), and species can be specified either as KEGG code, scientific name or comon name. In addition, \Rpackage{pathview}  works with different data attributes and formats, both continuous and discrete data (\hyperref[subsec:discrete]{Subsection \ref*{subsec:discrete}}), either in matrix or vector format, with single or multiple samples/experiments etc. Partcullary, \Rpackage{Pathview} can now integrate and compare multiple samples or states into one graph (\hyperref[subsec:multiple]{Subsection \ref*{subsec:multiple}}).

\subsection{Compound and gene data}
\label{subsec:compound}
In examples above, we viewed  gene data with canonical signaling pathways. We frequently want to look at metabolic pathways too. Besides gene nodes, these pathways also have compound nodes.  Therefore, we may integrate or visualize both gene data and compound data with metabolic pathways. Here gene data is a broad concept including genes, transcripts, protein , enzymes and their expression, modifications and any measurable attributes. Same is compound data, including metabolites, drugs, their measurements and attributes.
Here we still use the breast cancer microarray dataset as gene data. We then generate simulated compound or metabolomic data, and load proper compound ID types (with sufficient number of unique entries) for demonstration.
<<dataPrep.sim.cpd, echo=TRUE>>=
sim.cpd.data=sim.mol.data(mol.type="cpd", nmol=3000)
data(cpd.simtypes)
@

We generate a native KEGG view graph with both gene data and compound data  (\hyperref[fig:5a]{Figure \ref*{fig:5a}}). Such metabolic pathway graphs generated by \Rpackage{pathview} is the same as the original KEGG graphs, except that the compound nodes are magnified for better view of the colors.
<<gene_cpd.data, echo=TRUE>>=
i <- 3
print(demo.paths$sel.paths[i])
pv.out <- pathview(gene.data = gse16873.d[, 1], cpd.data = sim.cpd.data, 
    pathway.id = demo.paths$sel.paths[i], species = "hsa", out.suffix = "gse16873.cpd", 
    keys.align = "y", kegg.native = T, key.pos = demo.paths$kpos1[i])
str(pv.out)
head(pv.out$plot.data.cpd)
@ 


We also generate Graphviz view of the same pathway and data  (\hyperref[fig:5b]{Figure \ref*{fig:5b}}). Graphviz view better shows the hierarchical structure. For metabolic pathways, we need to parse the reaction entries from xml files and convert it to relationships between gene and compound nodes. We use ellipses for compound nodes. The labels are standard compound names, which are retrieved from CHEMBL database. KEGG does not provide it in the pathway database files. Chemical names are long strings, we need to do word wrap to fit them to specified width on the graph.
<<graphviz.gene_cpd.data, echo=TRUE>>=
pv.out <- pathview(gene.data = gse16873.d[, 1], cpd.data = sim.cpd.data, 
    pathway.id = demo.paths$sel.paths[i], species = "hsa", out.suffix = "gse16873.cpd", 
    keys.align = "y", kegg.native = F, key.pos = demo.paths$kpos2[i], 
    sign.pos = demo.paths$spos[i], cpd.lab.offset = demo.paths$offs[i])
@ 


\begin{figure}
        \centering
        \begin{subfigure}[b]{1.0\textwidth}
                \centering
                \fbox{\includegraphics[width=15cm,type=png,ext=.png,read=.png]{hsa00640.gse16873.cpd}}
                \caption{}
                \label{fig:5a}
        \end{subfigure}
        \\
        \vspace{20pt}
        \begin{subfigure}[b]{1.0\textwidth}
                \centering
                \includegraphics[width=15cm,type=pdf,ext=.pdf,read=.pdf]{hsa00640.gse16873.cpd}
                \caption{}
                \label{fig:5b}
        \end{subfigure}
        \\
        \caption{Example (a) KEGG view or (b) Graphviz view on both gene data and compound data simultaneously.}
        \label{fig:5}
\end{figure}

\subsection{Multiple states or samples}
\label{subsec:multiple}
In all previous examples, we looked at single sample data, which are either vector or single-column matrix. \Rpackage{Pathview} also handles multiple sample data, it used to generate graph for each sample. Since version 1.1.6, \Rpackage{Pathview} can integrate and plot multiple samples or states into one graph (Figure \hyperref[fig:multi1]{\ref*{fig:multi1}} - \hyperref[fig:multi2]{\ref*{fig:multi2}}). 

Let's simulate compound data with multiple replicate samples first.
<<sim.cpd.data2, echo=TRUE>>=
set.seed(10)
sim.cpd.data2 = matrix(sample(sim.cpd.data, 18000, 
    replace = T), ncol = 6)
rownames(sim.cpd.data2) = names(sim.cpd.data)
colnames(sim.cpd.data2) = paste("exp", 1:6, sep = "")
head(sim.cpd.data2, 3)
@ 

In the following examples, \R{gene.data} has three samples while \R{cpd.data} has two. We may include all these samples in one graph. We can do either native KEGG view (\hyperref[fig:multi1]{Figure \ref*{fig:multi1}}) or Graphviz view (\hyperref[fig:multi2]{Figure \ref*{fig:multi2}})on such multiple-sample data. In these graphs, we see that the gene nodes and compound nodes are sliced into multiple pieces corresponding to different states or samples. Since the sample sizes are different for \R{gene.data} and \R{cpd.data}, we can choose to match the data if samples in the two data types are actually paired, i.e. first columns of for \R{gene.data} and \R{cpd.data} come from the same experiment/sample, and so on.

<<multisample.gene_cpd.data, echo=TRUE>>=
#KEGG view
pv.out <- pathview(gene.data = gse16873.d[, 1:3], 
    cpd.data = sim.cpd.data2[, 1:2], pathway.id = demo.paths$sel.paths[i], 
    species = "hsa", out.suffix = "gse16873.cpd.3-2s", keys.align = "y", 
    kegg.native = T, match.data = F, multi.state = T, same.layer = T)
head(pv.out$plot.data.cpd)
#KEGG view with data match
pv.out <- pathview(gene.data = gse16873.d[, 1:3], 
    cpd.data = sim.cpd.data2[, 1:2], pathway.id = demo.paths$sel.paths[i], 
    species = "hsa", out.suffix = "gse16873.cpd.3-2s.match", 
    keys.align = "y", kegg.native = T, match.data = T, multi.state = T, 
    same.layer = T)
#graphviz view
pv.out <- pathview(gene.data = gse16873.d[, 1:3], 
    cpd.data = sim.cpd.data2[, 1:2], pathway.id = demo.paths$sel.paths[i], 
    species = "hsa", out.suffix = "gse16873.cpd.3-2s", keys.align = "y", 
    kegg.native = F, match.data = F, multi.state = T, same.layer = T,
    key.pos = demo.paths$kpos2[i], sign.pos = demo.paths$spos[i])
@

\begin{figure}
        \centering
        \begin{subfigure}[b]{1.0\textwidth}
                \centering
                \fbox{\includegraphics[width=15cm,type=png,ext=.png,read=.png]{hsa00640.gse16873.cpd.3-2s.multi}}
                \caption{}
                \label{fig:multi1a}
        \end{subfigure}
        \\
        \vspace{20pt}
        \begin{subfigure}[b]{1.0\textwidth}
                \centering
                \fbox{\includegraphics[width=15cm,type=png,ext=.png,read=.png]{hsa00640.gse16873.cpd.3-2s.match.multi}}
                \caption{}
                \label{fig:multi1b}
        \end{subfigure}
        \\
        \caption{Example KEGG view on multiple states of both gene data and compound data simultaneously (a) without or (b) with matching the samples.}
        \label{fig:multi1}
\end{figure}

\begin{figure} 
\begin{center} 
\includegraphics[width=16.5cm,type=pdf,ext=.pdf,read=.pdf]{hsa00640.gse16873.cpd.3-2s.multi}
\caption{Example Graphviz view on multiple states of both gene data and compound data simultaneously.}
\label{fig:multi2}
\end{center}  
\end{figure} 

Again, we may choose to plot the samples separately, i.e. one sample per graph. Note that in this case, the samples in \R{gene.data} and \R{cpd.data} has to be matched as to be assigned to the same graph. Hence, argument \R{match.data} isn't really effective here.

<<multisample.gene_cpd.data.seperate, echo=TRUE, eval=FALSE>>=
#plot samples/states separately
pv.out <- pathview(gene.data = gse16873.d[, 1:3], 
    cpd.data = sim.cpd.data2[, 1:2], pathway.id = demo.paths$sel.paths[i], 
    species = "hsa", out.suffix = "gse16873.cpd.3-2s", keys.align = "y", 
    kegg.native = T, match.data = F, multi.state = F, same.layer = T)
@ 

As described above, KEGG views on the same layer may takes some time. Again, we can choose to do KEGG view with two layers as to speed up the process if we don't mind losing the original KEGG gene labels (or EC numbers).
<<multisample.gene_cpd.data.2layer, echo=TRUE, eval=FALSE>>=
pv.out <- pathview(gene.data = gse16873.d[, 1:3], 
    cpd.data = sim.cpd.data2[, 1:2], pathway.id = demo.paths$sel.paths[i], 
    species = "hsa", out.suffix = "gse16873.cpd.3-2s.2layer", 
    keys.align = "y", kegg.native = T, match.data = F, multi.state = T, 
    same.layer = F)
@

\subsection{Discrete data}
\label{subsec:discrete}
So far, we have been dealing with continuous data. But we often work with discrete data too. For instance, we select list of signficant genes or compound based on some statistics (p-value, fold change etc). The input data can be named vector of two levels, either 1 or 0 (signficant or not), or it can be a shorter list of signficant gene/compound names. In the next two examples, we made both \R{gene.data} and \R{cpd.data} or \R{gene.data} only  (\hyperref[fig:6]{Figure \ref*{fig:6}}) discrete.
<<discrete.gene_cpd.data, echo=TRUE>>=
require(org.Hs.eg.db)
gse16873.t <- apply(gse16873.d, 1, function(x) t.test(x, 
    alternative = "two.sided")$p.value)
sel.genes <- names(gse16873.t)[gse16873.t < 0.1]
sel.cpds <- names(sim.cpd.data)[abs(sim.cpd.data) > 0.5]
pv.out <- pathview(gene.data = sel.genes, cpd.data = sel.cpds, 
    pathway.id = demo.paths$sel.paths[i], species = "hsa", out.suffix = "sel.genes.sel.cpd", 
    keys.align = "y", kegg.native = T, key.pos = demo.paths$kpos1[i], 
    limit = list(gene = 5, cpd = 2), bins = list(gene = 5, cpd = 2), 
    na.col = "gray", discrete = list(gene = T, cpd = T))
pv.out <- pathview(gene.data = sel.genes, cpd.data = sim.cpd.data, 
    pathway.id = demo.paths$sel.paths[i], species = "hsa", out.suffix = "sel.genes.cpd",
    keys.align = "y", kegg.native = T, key.pos = demo.paths$kpos1[i], 
    limit = list(gene = 5, cpd = 1), bins = list(gene = 5, cpd = 10), 
    na.col = "gray", discrete = list(gene = T, cpd = F))
@

\begin{figure} 
\begin{center} 
\fbox{\includegraphics[width=15cm,type=png,ext=.png,read=.png]{hsa00640.sel.genes.cpd}}
\caption{Example native KEGG view on discrete gene data and continuous compound data simultaneously.}
\label{fig:6}
\end{center}
\end{figure} 


\subsection{ID mapping}
\label{subsec:ids}
A distinguished feature of \Rpackage{pathview} is its strong ID mapping capability. The integrated Mapper module maps over 10 types of gene or protein IDs, and 20 types of compound or metabolite IDs to standard KEGG gene or compound IDs, and also maps between these external IDs. In other words, user data named with any of these different ID types get accurately mapped to target KEGG pathways. \Rpackage{Pathview} applies to pathways for about 4800 species, and species can be specified in multiple formats: KEGG code, scientific name or comon name.

    The following example makes use of the integrated mapper to map external ID types to standard KEGG IDs automatically  (\hyperref[fig:7]{Figure \ref*{fig:7}}). We only need to specify the external ID types using \R{gene.idtype} and \R{cpd.idtype} arguments. Note that automatic mapping is limited to certain ID types. For details check: \R{gene.idtype.list} and \R{data(rn.list); names(rn.list)}.
<<gene.ensprot_cpd.cas, echo=TRUE>>=
cpd.cas <- sim.mol.data(mol.type = "cpd", id.type = cpd.simtypes[2], 
    nmol = 10000)
gene.ensprot <- sim.mol.data(mol.type = "gene", id.type = gene.idtype.list[4], 
    nmol = 50000)
pv.out <- pathview(gene.data = gene.ensprot, cpd.data = cpd.cas, 
    gene.idtype = gene.idtype.list[4], cpd.idtype = cpd.simtypes[2], 
    pathway.id = demo.paths$sel.paths[i], species = "hsa", same.layer = T, 
    out.suffix = "gene.ensprot.cpd.cas", keys.align = "y", kegg.native = T, 
    key.pos = demo.paths$kpos2[i], sign.pos = demo.paths$spos[i], 
    limit = list(gene = 3, cpd = 3), bins = list(gene = 6, cpd = 6))
@

\begin{figure} 
\begin{center} 
\fbox{\includegraphics[width=15cm,type=png,ext=.png,read=.png]{hsa00640.gene.ensprot.cpd.cas}}
\caption{Example native KEGG view on gene data and compound data with other ID types.}
\label{fig:7}
\end{center}
\end{figure} 

For external IDs not in the auto-mapping lists, we may make use of the \R{mol.sum} function (also part of the Mapper module) to do the ID and data mapping explicitly. Here we need to provide \R{id.map}, the mapping matrix between external ID and KEGG standard ID. We use ID mapping functions including \R{id2eg} and \R{cpdidmap} etc to get \R{id.map} matrix.  Note that these ID mapping functions can be used independent of \R{pathview} main function. The following example use this route with the simulated \R{gene.ensprot} and \R{cpd.kc} data above, and we get the same results (Figure not shown).
<<gene.ensprot_cpd.cas.manual.map, echo=TRUE>>=
id.map.cas <- cpdidmap(in.ids = names(cpd.cas), in.type = cpd.simtypes[2], 
    out.type = "KEGG COMPOUND accession")
cpd.kc <- mol.sum(mol.data = cpd.cas, id.map = id.map.cas)
id.map.ensprot <- id2eg(ids = names(gene.ensprot), 
    category = gene.idtype.list[4], org = "Hs")
gene.entrez <- mol.sum(mol.data = gene.ensprot, id.map = id.map.ensprot)
pv.out <- pathview(gene.data = gene.entrez, cpd.data = cpd.kc, 
    pathway.id = demo.paths$sel.paths[i], species = "hsa", same.layer = T, 
    out.suffix = "gene.entrez.cpd.kc", keys.align = "y", kegg.native = T, 
    key.pos = demo.paths$kpos2[i], sign.pos = demo.paths$spos[i], 
    limit = list(gene = 3, cpd = 3), bins = list(gene = 6, cpd = 6))
@ 

\subsection{Working with species}
\label{subsec:species}
Species is a tricky yet important issue when working with KEGG. KEGG has its own dedicated nomenclature and database for species, which includes about 4800 organisms with complete genomes. In other words, gene data for any of these organisms can be mapped, visualized and analyzed using \Rpackage{pathview}. This comprehensive species coverage is a prominent feature of \Rpackage{pathview}'s data integration capacity. However, KEGG does not treat all of these organisms/genomes the same way. KEGG use NCBI GeneID (or Entrez Gene) as the default ID for the most common model animals, including human, mouse, rat etc and a few crops, e.g. soybean, wine grape and maize. On the other hand, KEGG uses Locus tag and other IDs for all others organisms, including animals, plants, fungi, protists, as well as bacteria and archaea.

\Rpackage{Pathview} carries a data matrix \R{korg}, which includes a complete list of supported KEGG species and default gene IDs. Let's explore \R{korg} data matrix as to have some idea on KEGG species and its Gene ID usage.
<<korg, echo=TRUE>>=
data(korg)
head(korg)
#number of species which use Entrez Gene as the default ID
sum(korg[,"entrez.gnodes"]=="1",na.rm=T)
#number of species which use other ID types or none as the default ID
sum(korg[,"entrez.gnodes"]=="0",na.rm=T)
#new from 2017: most species which do not have Entrez Gene annotation any more
na.idx=is.na(korg[,"ncbi.geneid"])
sum(na.idx)
@ 

From the exploration of \R{korg} above, we see that out of the ~4800 KEGG species, only a few don't have NCBI (Entrez) Gene ID or any gene ID (annotation) at all. Almost all of them have both default KEGG gene ID (often Locus tag) and Entrez Gene ID annotation. Therefore, \R{pathview} accepts \R{gene.idtype="kegg"} or \R{"Entrez"} (case insensitive) for all these species. The users need to make sure the right \R{gene.idtype} is specified because KEGG and Entrez Gene IDs are not the same except for 35 common species. For 19 species, Bioconductor provides gene annotation packages. The users have the freedom to input \R{gene.data} with other \R{gene.idtype}'s. For a list of these Bioconductor annotated species and extra Gene ID types allowed:
<<bods_gene.idtype.list, echo=TRUE>>=
data(bods)
bods
data(gene.idtype.list)
gene.idtype.list
@ 

All previous examples show human data, where Entrez Gene is KEGG's default gene ID. \Rpackage{Pathview} now (since version 1.1.5) explicitly handles species which use Locus tag or other gene IDs as the KEGG default ID. Below are an couple of examples with E coli strain K12 data. First, we work on gene data with the default KEGG ID (Locus tag) for E coli K12.
<<eco.dat.kegg, echo=TRUE>>=
eco.dat.kegg <- sim.mol.data(mol.type="gene",id.type="kegg",species="eco",nmol=3000)
head(eco.dat.kegg)
pv.out <- pathview(gene.data = eco.dat.kegg, gene.idtype="kegg",
    pathway.id = "00640", species = "eco", out.suffix = "eco.kegg",
    kegg.native = T, same.layer=T)
@ 

We may also work on gene data with Entrez Gene ID for E coli K12 the same way as for human. Note this part may not work as org.EcK12.eg.db had some problem from BioC 3.21.
<<eco.dat.entrez, echo=TRUE, eval=FALSE>>=
eco.dat.entrez <- sim.mol.data(mol.type="gene",id.type="entrez",species="eco",nmol=3000)
head(eco.dat.entrez)
pv.out <- pathview(gene.data = eco.dat.entrez, gene.idtype="entrez",
    pathway.id = "00640", species = "eco", out.suffix = "eco.entrez",
    kegg.native = T, same.layer=T)
@ 

Based on the \R{bods} data described above, E coli K12 is an Bioconductor annotated species. Hence we may further work on its gene data with other ID types, for example, official gene symbols. When calling \R{pathview}, such data need to be mapped to Entrez Gene ID first (if not yet), then to default KEGG ID (Locus tag). Therefore, it takes longer time than the last example.
<<eco.dat.symbol, echo=TRUE, eval=FALSE>>=
egid.eco=eg2id(names(eco.dat.entrez), category="symbol", pkg="org.EcK12.eg.db")
eco.dat.symbol <- eco.dat.entrez
names(eco.dat.symbol) <- egid.eco[,2]
head(eco.dat.kegg)
pv.out <- pathview(gene.data = eco.dat.symbol, gene.idtype="symbol",
    pathway.id = "00640", species = "eco", out.suffix = "eco.symbol.2layer",
    kegg.native = T, same.layer=F)
@

Importantly, \Rpackage{pathview} can be directly used for metagenomic or microbiome data when the data are mapped to KEGG ortholog pathways. And data from any new species that has not been annotated and included in KEGG (non-KEGG species) can also been analyzed and visualized using \Rpackage{pathview} by mapping to KEGG ortholog pathways the same way. In the next example, we simulate the mapped KEGG ortholog gene data first. Then the data is input as \R{gene.data} with \R{species="ko"}. Check \R{pathview} function for details.
<<gene.ensprot_cpd.cas.manual.map, echo=TRUE>>=
ko.data=sim.mol.data(mol.type="gene.ko", nmol=5000)
pv.out <- pathview(gene.data = ko.data, pathway.id = "04112",
                   species = "ko", out.suffix = "ko.data", kegg.native = T)
@ 


\section{Integrated workflow with pathway analysis}
\label{sec:workflow}
Although built as a stand alone program, \Rpackage{Pathview} may seamlessly integrate with pathway and functional analysis tools for large-scale and fully automated analysis pipeline. The next example shows how to connect common pathway analysis to results rendering with \Rpackage{pathview}. The pathway analysis was done using another Bioconductor package \Rpackage{gage} \citep{luo:etal:2009}, and the selected signficant pathways plus the expression data were then piped to \Rpackage{pathview} for auomated results visualization (Figure not shown).  In \Rpackage{gage} package, vignette \href{http://bioconductor.org/packages/release/bioc/vignettes/gage/inst/doc/RNA-seqWorkflow.pdf}{"RNA-Seq Data Pathway and Gene-set Analysis Workflows"} demonstrates GAGE/Pathview workflows on RNA-seq (and microarray) pathway analysis. Note \href{https://pathview.uncc.edu/}{the Pathview Web server} provides a comprehensive workflow for both regular and integrated pathway analysis of multiple omics data \citep{luo:etal:2017}, as shown in \href{https://pathview.uncc.edu/example4}{Example 4 online}. 

Note \Rpackage{pathview} focuses on KEGG pathways, which is good for most regular analyses. If you are interested in working with other major pathway databases, including Reactome, MetaCyc, SMPDB, PANTHER, METACROP etc, you can use \href{https://github.com/datapplab/SBGNview}{\Rpackage{SBGNview}}. Please check \href{https://github.com/datapplab/SBGNview}{the quick start  page} and \href{https://bioconductor.org/packages/devel/bioc/vignettes/SBGNview/inst/doc/SBGNview.Vignette.html}{the main tutorial} for details. Also see \href{https://bioconductor.org/packages/devel/bioc/vignettes/SBGNview/inst/doc/pathway.enrichment.analysis.html}{SBGNview + GAGE based pathway analysis workflow}.

<<GAGE.Pathview.pipeline, echo=TRUE, eval=FALSE>>=
library(gage)
data(gse16873)
cn <- colnames(gse16873)
hn <- grep('HN',cn, ignore.case =TRUE)
dcis <- grep('DCIS',cn, ignore.case =TRUE)
data(kegg.gs)
#pathway analysis using gage
gse16873.kegg.p <- gage(gse16873, gsets = kegg.gs,
    ref = hn, samp = dcis)
#prepare the differential expression data
gse16873.d <- gagePrep(gse16873, ref = hn, samp = dcis)
#equivalently, you can do simple subtraction for paired samples
gse16873.d <- gse16873[,dcis]-gse16873[,hn]
#select significant pathways and extract their IDs
sel <- gse16873.kegg.p$greater[, "q.val"] < 0.1 & !is.na(gse16873.kegg.p$greater[, 
    "q.val"])
path.ids <- rownames(gse16873.kegg.p$greater)[sel]
path.ids2 <- substr(path.ids[c(1, 2, 7)], 1, 8)
#pathview visualization
pv.out.list <- pathview(gene.data = gse16873.d[, 1:3], pathway.id = path.ids2, 
    species = "hsa")
@ 


\section{Common Errors}
\label{sec:errors}
\begin{itemize}
\item mismatch between the IDs for \R{gene.data} (or \R{cpd.data}) and \R{gene.idtype} (or \R{cpd.idtype}). For example, \R{gene.data} or \R{cpd.data} uses some extern ID types, while \R{gene.idtype = "entrez"} and \R{cpd.idtype = "kegg"} (default).
\item mismatch between \R{gene.data} (or \R{cpd.data}) and \R{species}. For example, \R{gene.data} come from "mouse", while \R{species="hsa"}.
\item \R{pathway.id} wrong or wrong format, right format should be a five digit number, like 04110, 00620 etc.
\item any of \R{limit, bins, both.dir, trans.fun, discrete, low, mid, high} arguments is specified as a vector of length 1 or 2, instead of a list of 2 elements. Correct format should be like \R{limit = list(gene = 1, cpd = 1)}.
\item \R{key.pos} or \R{sign.pos} not good, hence the color key or signature overlaps with pathway main graph. 
\item \textbf{Special Note}: some KEGG xml data files are incomplete, inconsistent with corresponding png image or inaccurate/incorrect on some parts. These issues may cause inaccuracy, incosistency, or error messages although \Rpackage{pathview} tries the best to accommodate them. For instance, we may see inconistence between KEGG view and Graphviz view. As in the latter case, the pathway layout is generated based on data from xml file. 
\end{itemize}


\bibliographystyle{plainnat}
\bibliography{pathview}

\end{document}

\name{node.info}
\alias{node.info}

\title{
  Extract node information from KEGG pathway
}
\description{
  The parser function, parser KGML file and/or extract node information
  from KEGG pathway. 
}
\usage{
node.info(object, short.name = TRUE)
}

\arguments{
  \item{object}{
either a character specifying the full KGML file name (with directory),
  or a object of "KEGGPathway" class, or a object of "graphNEL"
  class. The latter two are parsed results of KGML file.
}
  \item{short.name}{
logical, if TRUE, the short labels, i.e. the first iterm separated by
  "," in the long labels are parsed out as node labels. Default short.name=TRUE.
}
}
\details{
Parser function node.info extract node data from parsed KEGG
  pathways. KGML files are parsed using \code{parseKGML2} and
  \code{KEGGpathway2Graph2}. These functions from KEGGgraph package have
  been heavily modified for reaction parsing and conversion to
  edges.
}
\value{
a named list of 10 elements: "kegg.names", "type", "component", "size",
"labels", "shape", "x", "y", "width" and "height". Each elements record
  the corresponding attribute for all nodes in the parsed KEGG pathway.
}
\references{
  Luo, W. and Brouwer, C., Pathview: an R/Bioconductor package for
  pathway based data integration and visualization. Bioinformatics,
 2013, 29(14): 1830-1831, doi: 10.1093/bioinformatics/btt285
}
\author{
  Weijun Luo <luo_weijun@yahoo.com>
}
\seealso{
  \code{\link{pathview}} the main function,
  \code{\link{combineKEGGnodes}} and \code{\link{reaction2edge}} for
  special treatment of nodes or edges.
}
\examples{
xml.file=system.file("extdata", "hsa04110.xml", package = "pathview")
node.data=node.info(xml.file)
names(node.data)
#or parse into a graph object, then extract node info
gR1=pathview:::parseKGML2Graph2(xml.file, genesOnly=FALSE, expand=FALSE, split.group=FALSE)
node.data=node.info(gR1)
}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
\keyword{ ~kwd1 }
\keyword{ ~kwd2 }% __ONLY ONE__ keyword per line

\name{node.map}
\alias{node.map}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{
Map molecular data onto KEGG pathway nodes
}
\description{
  The mapper function, mapping molecular data(gene expression, metabolite
  abundance etc)to nodes in KEGG pathway. 
}
\usage{
node.map(mol.data = NULL, node.data, node.types = c("gene", "ortholog",
"compound")[1], node.sum = c("sum", "mean", "median", "max", "max.abs",
"random")[1], entrez.gnodes=TRUE)
}
\arguments{
  \item{mol.data}{
  Either vector (single sample) or a matrix-like data (multiple
  sample). Vector should be numeric with molecule IDs as names or it
  may also be character of molecule IDs. Character vector is treated as
  discrete or count data. Matrix-like data structure has molecules as
  rows and samples as columns. Row names should be molecule IDs. Default
  mol.data=NULL. 
  This argument is equivalent to gene.data or cpd.data in the pathview
  function. Check pahtview function for more information.
}
\item{node.data}{
a named list of 10 elements, the results returned by \code{node.info},
check the function for details.
}
  \item{node.types}{
character, sepcify the node type to map the mol.data to, either "gene",
"compound", or "compound". Default node.types="gene".
}
  \item{node.sum}{
  character, the method name to calculate node summary given that
  multiple genes or compounds are mapped to it. Poential options include
  "sum","mean", "median", "max", "max.abs" and "random". Default node.sum="sum".
}
  \item{entrez.gnodes}{
  logical, whether EntrezGene (NCBI GeneID) is used as the default gene
  ID in the KEGG data files. This is needed because KEGG uses different
  types default gene ID for different species. Some most common model
  species use EntrezGene, but majority of others use Locus tag. Default
  entrez.gnodes=TRUE.
}
}
\details{
Mapper function node.map maps user supplied molecular data to KEGG
pathways. This function takes standard KEGG molecular IDs (Entrez Gene
ID or KEGG Compound Accession) and map them to pathway nodes. None KEGG
molecular gene IDs or Compound IDs are pre-mapped to standard KEGG IDs
by calling another function \code{mol.sum}. When
multiple molecules map to one node, the corresponding molecular data are
summarized into a single node summary by calling function specified by
\code{node.sum}. This mapped node summary data together with the parsed
KGML data are then returned for further processing.
Proper input data include: gene expression, protein
expression, genetic association, metabolite abundance, genomic data,
literature, and other data types mappable to pathways.
The input mol.data may be NULL, then no molecular data are actually
mapped, but all nodes of the specified node.type are considered
"mappable" and their parsed KGML data returned.
}
\value{
  A data.frame composed of parsed KGML data and summary molecular data
  for each mapped node. Each row is a mapped node, and columns are:
  \item{kegg.names}{standard KEGG IDs/Names for mapped nodes. It's
    Entrez Gene ID or KEGG Compound Accessions.}
  \item{labels}{Node labels to be used when needed
  }
  \item{type}{node type, currently 4 types are supported:
    "gene","enzyme", "compound" and "ortholog".
  }
  \item{x}{x coordinate in the original KEGG pathway graph.
  }
  \item{y}{y coordinate in the original KEGG pathway graph.
  }
  \item{width}{node width in the original KEGG pathway graph.
  }
  \item{height}{node height in the original KEGG pathway graph.
  }
  \item{other columns }{columns of the mapped gene/compound data 
  }      
}
\references{
  Luo, W. and Brouwer, C., Pathview: an R/Bioconductor package for
  pathway based data integration and visualization. Bioinformatics,
 2013, 29(14): 1830-1831, doi: 10.1093/bioinformatics/btt285
}
\author{
  Weijun Luo <luo_weijun@yahoo.com>
}

\seealso{
  \code{\link{mol.sum}} the auxillary molecular data mapper,
  \code{\link{id2eg}}, \code{\link{cpd2kegg}} etc the auxillary molecular ID mappers,
  \code{\link{node.color}} the node color coder,
  \code{\link{pathview}} the main function,
  \code{\link{node.info}} the parser.
}
\examples{
xml.file=system.file("extdata", "hsa04110.xml", package = "pathview")
node.data=node.info(xml.file)
names(node.data)
data(gse16873.d)
plot.data.gene=node.map(mol.data=gse16873.d[,1], node.data,
  node.types="gene")
head(plot.data.gene)
}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
\keyword{ ~kwd1 }
\keyword{ ~kwd2 }% __ONLY ONE__ keyword per line

\name{cpdidmap}
\alias{cpdidmap}
\alias{cpd2kegg}
\alias{cpdkegg2name}
\alias{cpdname2kegg}

\title{
Mapping between compound IDs and KEGG accessions
}
\description{
These auxillary compound ID mappers connect KEGG compound/glycan/drug
accessions to compound names/synonyms and other commonly used
compound-related IDs. 
}
\usage{
cpdidmap(in.ids, in.type, out.type)
cpd2kegg(in.ids, in.type)
cpdkegg2name(in.ids, in.type = c("KEGG", "KEGG COMPOUND accession")[1])
cpdname2kegg(in.ids)
}

\arguments{
  \item{in.ids}{
    character, input IDs to be mapped.
  }
  \item{in.type}{
    character, the input ID type, needs to be either "KEGG" (including
    compound, glycan and durg) or one of the
    compound-related ID types used in CHEMBL database. For a full list of
    the CHEMBL IDs, do \code{data(rn.list); names(rn.list)}. For
    \code{cpdkegg2name)}, default in.type = "KEGG".
  }
  \item{out.type}{
    character, the output ID type, needs to be either "KEGG" (including
    compound/glycan/durg) or one of the
    compound-related ID types used in CHEMBL database. For a full list of
    the CHEMBL IDs, do \code{data(rn.list); names(rn.list)}.
  }
}
\details{
character, the output ID type, needs to be either "KEGG" or one of the
  compound-related ID types used in CHEMBL database. For a full list of
  the CHEMBL IDs, do \code{data(rn.list); names(rn.list)}.

  KEGG has its own compound ID system, including compound (glycan/durg)
  accessions.  Therefore, all compound
data need to be mapped to KEGG accessions when working with KEGG
pathways. Function \code{cpd2kegg} does this mapping by calling
  \code{cpdname2kegg} or \code{cpdidmap}. On the other hand, we
frequently want to check or show compound full names or other commonly
  used IDs instead of the less informative KEGG accessions when working with KEGG compound nodes,
Functions \code{cpdkegg2name} and \code{cpdidmap} do this reverse mapping.
These functions are
written as part of the Pathview mapper module, they are equally useful
for other compound ID or data mapping tasks.
The use of these functions depends on a few data objects:
"cpd.accs", "cpd.names", "keg.met" and "rn.list", which are included in
  this package. To access them, use \code{data()} function.
}
\value{
a 2-column character matrix recording the mapping between input IDs to
the target ID type.
}
\references{
  Luo, W. and Brouwer, C., Pathview: an R/Bioconductor package for
  pathway based data integration and visualization. Bioinformatics,
 2013, 29(14): 1830-1831, doi: 10.1093/bioinformatics/btt285
}
\author{
  Weijun Luo <luo_weijun@yahoo.com>
}

\seealso{
  \code{\link{eg2id}} and \code{\link{id2eg}} the auxillary gene ID mappers,
  \code{\link{mol.sum}} the auxillary molecular data mapper,
  \code{\link{node.map}} the node data mapper function.
}
\examples{
data(cpd.simtypes)
#generate simulated compound data named with non-KEGG ("CAS Registry Number")IDs
cpd.cas <- sim.mol.data(mol.type = "cpd", id.type = cpd.simtypes[2], 
    nmol = 10000)
#construct map between non-KEGG ID and KEGG ID ("KEGG COMPOUND accession")
id.map.cas <- cpdidmap(in.ids = names(cpd.cas), in.type = cpd.simtypes[2], 
    out.type = "KEGG COMPOUND accession")
#Map molecular data onto standard KEGG IDs
cpd.kc <- mol.sum(mol.data = cpd.cas, id.map = id.map.cas)
#check the results
head(cpd.cas)
head(id.map.cas)
head(cpd.kc)

#map KEGG ID to compound name
cpd.names=cpdkegg2name(in.ids=id.map.cas[,2])
head(cpd.names)
}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
\keyword{ ~kwd1 }
\keyword{ ~kwd2 }% __ONLY ONE__ keyword per line

\name{korg}
\alias{korg}
\alias{bods}
\docType{data}
\title{
  Mapping data on KEGG species code and corresponding Bioconductor gene
  annotation package
}
\description{

  Data on KEGG species, including taxonomy IDs, KEGG code, scientific name,
  common name, corresponding gene ID types, and gene annotation package
  names in Bioconductor
}
\usage{
data(korg)
data(bods)
}
\format{
  korg is a character matrix of ~4800 rows and 10 columns. First 5
  columns are KEGG and NCBI taxonomy IDs, KEGG species code, scientific
  name and common name, followed columns on gene ID types used for each
  species: entrez.gnodes ("1" or "0", whether EntrezGene is the default
  gene ID) and representative KEGG gene ID, NCBI or Entrez Gene ID, NCBI
  protein and Uniprot ID. Note korg includes 4800 KEGG species (as of
  06/2017), in the meantime, an updated version of korg is now checked
  out from Pathview Web server each time pathview package is loaded.

  
  bods is a character matrix of 19 rows and 3 columns on the mapping
  between gene annotation package names in Bioconductor, common name and
  KEGG code of most common research species.
}
\source{
  http://www.genome.jp/kegg-bin/get_htext?br08601.keg

  http://bioconductor.org/packages/release/BiocViews.html#___OrgDb
}
\examples{
data(korg)
data(bods)
head(korg)
head(bods)
}
\keyword{datasets}

\name{pathview}
\alias{pathview}
\alias{keggview.native}
\alias{keggview.graph}

%- Also NEED an '\alias' for EACH other topic documented here.
\title{
Pathway based data integration and visualization
}
\description{
Pathview is a tool set for pathway based data integration and
visualization. It maps and renders user data on relevant pathway
graphs. All users need is to supply their gene or compound data and
specify the target pathway. Pathview automatically downloads the pathway
graph data, parses the data file, maps user data to the pathway, and
render pathway graph with the mapped data.
Pathview generates both native KEGG view and Graphviz views for
pathways. keggview.native and keggview.graph are the two viewer
functions, and pathview is the main function providing a unified
interface to downloader, parser, mapper and viewer functions.
}
\usage{
pathview(gene.data = NULL, cpd.data = NULL, pathway.id,
species = "hsa", kegg.dir = ".", cpd.idtype = "kegg", gene.idtype =
"entrez", gene.annotpkg = NULL, min.nnodes = 3, kegg.native = TRUE,
map.null = TRUE, expand.node = FALSE, split.group = FALSE, map.symbol =
TRUE, map.cpdname = TRUE, node.sum = "sum", discrete=list(gene=FALSE,
cpd=FALSE), limit = list(gene = 1, cpd = 1), bins = list(gene = 10, cpd
= 10), both.dirs = list(gene = T, cpd = T), trans.fun = list(gene =
NULL, cpd = NULL), low = list(gene = "green", cpd = "blue"), mid =
list(gene = "gray", cpd = "gray"), high = list(gene = "red", cpd =
"yellow"), na.col = "transparent", ...)

keggview.native(plot.data.gene = NULL, plot.data.cpd = NULL,
cols.ts.gene = NULL, cols.ts.cpd = NULL, node.data, pathway.name,
out.suffix = "pathview", kegg.dir = ".", multi.state=TRUE, match.data =
TRUE, same.layer = TRUE, res = 300, cex = 0.25, discrete =
list(gene=FALSE, cpd=FALSE), limit= list(gene = 1, cpd = 1), bins =
list(gene = 10, cpd = 10), both.dirs =list(gene = T, cpd = T), low =
list(gene = "green", cpd = "blue"), mid = list(gene = "gray", cpd =
"gray"), high = list(gene = "red", cpd = "yellow"), na.col =
"transparent", new.signature = TRUE, plot.col.key = TRUE, key.align =
"x", key.pos = "topright", ...)

keggview.graph(plot.data.gene = NULL, plot.data.cpd = NULL, cols.ts.gene
= NULL, cols.ts.cpd = NULL, node.data, path.graph, pathway.name,
out.suffix = "pathview", pdf.size = c(7, 7), multi.state=TRUE,
same.layer = TRUE, match.data = TRUE, rankdir = c("LR", "TB")[1],
is.signal = TRUE, split.group = F, afactor = 1, text.width = 15, cex =
0.5, map.cpdname = FALSE, cpd.lab.offset = 1.0,
discrete=list(gene=FALSE, cpd=FALSE), limit = list(gene = 1, cpd = 1),
bins = list(gene = 10, cpd = 10), both.dirs = list(gene = T, cpd = T),
low = list(gene = "green", cpd = "blue"), mid = list(gene = "gray", cpd
= "gray"), high = list(gene = "red", cpd = "yellow"), na.col =
"transparent", new.signature = TRUE, plot.col.key = TRUE, key.align =
"x", key.pos = "topright", sign.pos = "bottomright", ...)  }

\arguments{
  \item{gene.data}{
  either vector (single sample) or a matrix-like data (multiple
  sample). Vector should be numeric with gene IDs as names or it may
  also be character of gene IDs. Character vector is treated as discrete
  or count data. Matrix-like data structure has genes as rows and
  samples as columns. Row names should be gene IDs. Here gene ID is a
  generic concepts, including multiple types of gene, transcript and
  protein uniquely mappable to KEGG gene IDs. KEGG ortholog IDs are also
  treated as gene IDs as to handle metagenomic data. Check details for
  mappable ID types. Default gene.data=NULL.

  numeric, character, continuous
}
  \item{cpd.data}{
 the same as gene.data, excpet named with IDs mappable to KEGG
 compound IDs. Over 20 types of IDs included in CHEMBL database can be
 used here. Check details for mappable ID types. Default
 cpd.data=NULL. Note that gene.data and cpd.data can't be NULL
 simultaneously.
}
  \item{pathway.id}{
 character vector, the KEGG pathway ID(s), usually 5 digit, may also include the 3
 letter KEGG species code.
}
  \item{species}{
character, either the kegg code, scientific name or the common name of
the target species. This applies to both pathway and gene.data or
cpd.data. When KEGG ortholog pathway is considered,
species="ko". Default species="hsa", it is equivalent to use either
"Homo sapiens" (scientific name) or "human" (common name). 
}
  \item{kegg.dir}{
character, the directory of KEGG pathway data file (.xml) and image file
(.png). Users may supply their own data files in the same format and
naming convention of KEGG's (species code + pathway id,
e.g. hsa04110.xml, hsa04110.png etc) in this directory. Default
kegg.dir="." (current working directory). 
}
  \item{cpd.idtype}{
character, ID type used for the cpd.data. Default cpd.idtype="kegg" (include
compound, glycan and drug accessions). 
}
  \item{gene.idtype}{
character, ID type used for the gene.data, case insensitive. Default
gene.idtype="entrez", i.e. Entrez Gene, which are the primary KEGG gene ID
for many common model organisms. For other species, gene.idtype should
be set to "KEGG" as KEGG use other types of gene IDs. For the common
model organisms (to check the list, do: \code{data(bods); bods}), you
may also specify other types of valid IDs. To check the ID list, do:
\code{data(gene.idtype.list); gene.idtype.list}. 
}
  \item{gene.annotpkg}{
character, the name of the annotation package to use for mapping between
other gene ID types including symbols and Entrez gene ID. Default gene.annotpkg=NULL.
}
  \item{min.nnodes}{
integer, minimal number of nodes of type "gene","enzyme", "compound" or
"ortholog" for a pathway to be considered. Default min.nnodes=3.
}
  \item{kegg.native}{
logical, whether to render pathway graph as native KEGG graph (.png) or
using graphviz layout engine (.pdf). Default kegg.native=TRUE.
}
  \item{map.null}{
logical, whether to map the NULL gene.data or cpd.data to pathway. When
NULL data are mapped, the gene or compound nodes in the pathway will be
rendered as actually mapped nodes, except with NA-valued color. When
NULL data are not mapped, the nodes are rendered as unmapped nodes. This
argument mainly affects native KEGG graph view, i.e. when
kegg.native=TRUE. Default map.null=TRUE.
}
  \item{expand.node}{
logical, whether the multiple-gene nodes are expanded into single-gene
nodes. Each expanded single-gene nodes inherits all edges from the
original multiple-gene node. This option only affects graphviz graph view, i.e. when
kegg.native=FALSE. This option is not effective for most metabolic
pathways where it conflits with  converting reactions to edges. Default expand.node=FLASE.
}
  \item{split.group}{
logical, whether split node groups are split to individual nodes. Each
split member nodes inherits all edges from the node group. This
option only affects graphviz graph view, i.e. when
kegg.native=FALSE. This option also effects most metabolic
pathways even without group nodes defined orginally. For these pathways,
genes involved in the same reaction are grouped automatically when
converting reactions to edges unless split.group=TRUE. d
split.group=FLASE.
}
  \item{map.symbol}{
logical, whether map gene IDs to symbols for gene node labels or use the
graphic name from the KGML file. This option is only effective for
kegg.native=FALSE or same.layer=FALSE when kegg.native=TRUE. For
same.layer=TRUE when kegg.native=TRUE, the native KEGG labels will be
kept. Default map.symbol=TRUE.
}
  \item{map.cpdname}{
logical, whether map compound IDs to formal names for compound node labels or use the
graphic name from the KGML file (KEGG compound accessions). This option is only effective for
kegg.native=FALSE. When kegg.native=TRUE, the native KEGG labels will be
kept. Default map.cpdname=TRUE.
}
  \item{node.sum}{
  character, the method name to calculate node summary given that
  multiple genes or compounds are mapped to it. Poential options include
  "sum","mean", "median", "max", "max.abs" and "random". Default node.sum="sum".
}
  \item{discrete}{
a list of two logical elements with "gene" and "cpd" as the names. This
argument tells whether gene.data or cpd.data should be treated as
discrete. Default dsicrete=list(gene=FALSE, cpd=FALSE), i.e. both data should
be treated as continuous.
}
  \item{limit}{
a list of two numeric elements with "gene" and "cpd" as the names. This
argument specifies the limit values for gene.data and cpd.data when converting
them to pseudo colors. Each element of the list could be of length 1 or
2. Length 1 suggests discrete data or 1 directional (positive-valued)
data, or the absolute limit for 2 directional data. Length 2 suggests 2
directional data. Default limit=list(gene=1, cpd=1).
}
  \item{bins}{
a list of two integer elements with "gene" and "cpd" as the names. This
argument specifies the number of levels or bins for gene.data and cpd.data when converting
them to pseudo colors. Default limit=list(gene=10, cpd=10).
}
  \item{both.dirs}{
a list of two logical  elements with "gene" and "cpd" as the names. This
argument specifies whether gene.data and cpd.data are 1 directional or 2
directional data when converting them to pseudo colors. Default limit=list(gene=TRUE, cpd=TRUE).
}
  \item{trans.fun}{
a list of two function (not character) elements with "gene" and "cpd" as the names. This
argument specifies whether and how gene.data and cpd.data are
transformed. Examples are \code{log}, \code{abs} or users' own
functions. Default limit=list(gene=NULL, cpd=NULL).
}
  \item{low, mid, high}{
each is a list of two colors with "gene" and "cpd" as the names. This
argument specifies the color spectra to code gene.data and
cpd.data. When data are 1 directional (TRUE value in both.dirs), only
mid and high are used to specify the color spectra. Default spectra (low-mid-high)
"green"-"gray"-"red" and "blue"-"gray"-"yellow" are used for gene.data
and cpd.data respectively. 
The values for 'low, mid, high' can be given as color names
('red'), plot color index (2=red), and HTML-style RGB,
("\#FF0000"=red).
}
  \item{na.col}{
color used for NA's or missing values in gene.data and cpd.data. d
na.col="transparent". 
}
  \item{\dots}{
extra arguments passed to keggview.native or keggview.graph function.
}

special arguments for keggview.native or keggview.graph function.
  \item{plot.data.gene}{
data.frame returned by node.map function for rendering mapped gene
  nodes, including node name, type, positions (x, y), sizes (width,
  height), and mapped gene.data. This data is also used as input for
  pseduo-color coding through node.color function. Default plot.data.gene=NULL.
}
  \item{plot.data.cpd}{
same as plot.data.gene function, except for mapped compound node data. d
  plot.data.cpd=NULL. Default plot.data.cpd=NULL. Note that plot.data.gene and
  plot.data.cpd can't be NULL simultaneously.
}
  \item{cols.ts.gene}{
vector or matrix of colors returned by node.color function for rendering
  gene.data. Dimensionality is the same as the latter. Default cols.ts.gene=NULL.
}
  \item{cols.ts.cpd}{
same as cols.ts.gene, except corresponding to cpd.data. d
  cols.ts.cpd=NULL. Note that cols.ts.gene and cols.ts.cpd
  plot.data.gene can't be NULL simultaneously. 
}
  \item{node.data}{
list returned by node.info function, which parse KGML file directly or
  indirectly, and extract the node data.
}
  \item{pathway.name}{
character, the full KEGG pathway name in the format of 3-letter species
  code with 5-digit pathway id, eg "hsa04612". 
}
  \item{out.suffix}{
character, the suffix to be added after the pathway name as part of the
  output graph file. Sample names or column names of the gene.data or
  cpd.data are also added when there are multiple samples. Default out.suffix="pathview".
}
\item{multi.state}{
  logical, whether multiple states (samples or columns) gene.data or
  cpd.data should be integrated and plotted in the same graph. Default
  match.data=TRUE. In other words, gene or compound nodes will be sliced
  into multiple pieces corresponding to the number of states in the
  data.
}
  \item{match.data}{
  logical, whether the samples of gene.data and cpd.data are
  paired. Default match.data=TRUE. When let sample sizes of gene.data
  and cpd.data be m and n, when m>n, extra columns of NA's (mapped to no
  color) will be added to cpd.data as to make the sample size the
  same. This will result in the smae number of slice in gene nodes and
  compound when multi.state=TRUE.
}
  \item{same.layer}{
    logical, control plotting layers:
    1) if node colors be plotted in the same layer as the pathway
  graph when kegg.native=TRUE, 2) if  edge/node type legend be plotted in
  the same page when kegg.native=FALSE.
}
  \item{res}{
    The nominal resolution in ppi which will be recorded in the
    bitmap file, if a positive integer.  Also used for 'units'
    other than the default, and to convert points to pixels. This
  argument is only effective when kegg.native=TRUE. Default res=300.
}
  \item{cex}{
    A numerical value giving the amount by which plotting text
    and symbols should be scaled relative to the default 1. Default cex=0.25
  when kegg.native=TRUE, cex=0.5 when kegg.native=FALSE.
}
  \item{new.signature}{
    logical, whether pathview signature is added to the pathway
  graphs. Default new.signature=TRUE.
}
  \item{plot.col.key}{
    logical, whether color key is added to the pathway
  graphs. Default plot.col.key= TRUE.

}
  \item{key.align}{
character, controlling how the color keys are aligned when both
  gene.data and cpd.data are not NULL. Potential values are "x", aligned
  by x coordinates, and "y", aligned by y coordinates. Default key.align="x".
}
  \item{key.pos}{
character, controlling the position of color key(s). Potentail values
  are "bottomleft", "bottomright", "topleft" and "topright". d
  key.pos="topright". 
}
  \item{sign.pos}{
character, controlling the position of pathview signature. Only
  effective when kegg.native=FALSE, Signature position is fixed in place
  of the original KEGG signature when kegg.native=TRUE. Potentail values
  are "bottomleft", "bottomright", "topleft" and "topright". d
  sign.pos="bottomright". 
}

  \item{path.graph}{
a graph object parsed from KGML file, only effective when
kegg.native=FALSE.
}
  \item{pdf.size}{
a numeric vector of length 2, giving the width and height of the pathway
graph pdf file. Note that pdf width increase by half when
same.layer=TRUE to accommodate legends. Only effective when
kegg.native=FALSE. Default pdf.size=c(7,7).
}
  \item{rankdir}{
character, either "LR" (left to right) or "TB" (top to bottom),
specifying the pathway graph layout direction. Only effective when
kegg.native=FALSE. Default rank.dir="LR".
}
  \item{is.signal}{
logical, if the pathway is treated as a signaling pathway, where all the
unconnected nodes are dropped. This argument also affect the graph
layout type, i.e. "dot" for signals or "neato" otherwise. Only effective when
kegg.native=FALSE. Default is.signal=TRUE.
}
  \item{afactor}{
numeric, node amplifying factor. This argument is for node size
fine-tuning, its effect is subtler than expected. Only effective when
kegg.native=FALSE. Default afctor=1.
}
  \item{text.width}{
numeric, specifying the line width for text wrap. Only effective when
kegg.native= FALSE. Default text.width=15 (characters).
}
  \item{cpd.lab.offset}{
numeric, specifying how much compound labels should be put above the
default position or node center. This argument is useful when
map.cpdname=TRUE, i.e. compounds are labelled by full name, which
affects the look of compound nodes and color. Only effective when
kegg.native=FALSE. Default cpd.lab.offset=1.0.
}
}
\details{
Pathview maps and renders user data on relevant pathway graphs. Pathview
is a stand alone program for pathway based data integration and
visualization. It also seamlessly integrates with pathway and functional
analysis tools for large-scale and fully automated analysis.
Pathview provides strong support for data Integration. It works with: 1)
essentially all types of biological data mappable to pathways, 2) over
10 types of gene or protein IDs, and 20 types of compound or metabolite
IDs, 3) pathways for over 2000 species as well as KEGG orthology, 4)
varoius data attributes and formats, i.e. continuous/discrete data,
matrices/vectors, single/multiple samples etc.
To see mappable external gene/protein IDs do:
\code{data(gene.idtype.list)}, to see mappable external compound related
IDs do: \code{data(rn.list)}; names(rn.list).
Pathview generates both native KEGG view and Graphviz views for
pathways. Currently only KEGG pathways are implemented. Hopefully, pathways from
Reactome, NCI and other databases will be supported in the future.
}
\value{
  From viersion 1.9.3, pathview can accept either a single pathway or
  multiple pathway ids. The result returned by pathview function is a
  named list corresponding to the input pathway ids. Each element (for
  each pathway itself is a named list, with 2
  elements ("plot.data.gene", "plot.data.cpd"). Both elements
  are data.frame or NULL depends on the corresponding input
  data gene.data and cpd.data. These data.frames record the plot data
  for mapped gene or compound nodes: rows are mapped genes/compounds,
  columns are:
  \item{kegg.names}{standard KEGG IDs/Names for mapped nodes. It's
    Entrez Gene ID or KEGG Compound Accessions.}
  \item{labels}{Node labels to be used when needed.
  }
  \item{all.mapped}{All molecule (gene or compound) IDs mapped to this
    node.
  }
  \item{type}{node type, currently 4 types are supported:
    "gene","enzyme", "compound" and "ortholog".
  }
  \item{x}{x coordinate in the original KEGG pathway graph.
  }
  \item{y}{y coordinate in the original KEGG pathway graph.
  }
  \item{width}{node width in the original KEGG pathway graph.
  }
  \item{height}{node height in the original KEGG pathway graph.
  }
  \item{other columns }{columns of the mapped gene/compound data and
    corresponding pseudo-color codes for individual samples
  }      

    The results returned by \code{keggview.native} and
  code{keggview.graph} are both a list of graph plotting
  parameters. These are not intended to be used externally.

 }
\references{
  Luo, W. and Brouwer, C., Pathview: an R/Bioconductor package for
  pathway based data integration and visualization. Bioinformatics,
 2013, 29(14): 1830-1831, doi: 10.1093/bioinformatics/btt285
}
\author{
  Weijun Luo <luo_weijun@yahoo.com>
}
\seealso{
  \code{\link{download.kegg}} the downloader,
  \code{\link{node.info}} the parser,
  \code{\link{node.map}} and \code{\link{node.color}} the mapper. 
}

\examples{
#load data
data(gse16873.d)
data(demo.paths)

#KEGG view: gene data only
i <- 1
pv.out <- pathview(gene.data = gse16873.d[, 1], pathway.id =
demo.paths$sel.paths[i], species = "hsa", out.suffix = "gse16873",
kegg.native = TRUE)
str(pv.out)
head(pv.out$plot.data.gene)
#result PNG file in current directory

#Graphviz view: gene data only
pv.out <- pathview(gene.data = gse16873.d[, 1], pathway.id =
demo.paths$sel.paths[i], species = "hsa", out.suffix = "gse16873",
kegg.native = FALSE, sign.pos = demo.paths$spos[i])
#result PDF file in current directory

#KEGG view: both gene and compound data
sim.cpd.data=sim.mol.data(mol.type="cpd", nmol=3000)
i <- 3
print(demo.paths$sel.paths[i])
pv.out <- pathview(gene.data = gse16873.d[, 1], cpd.data = sim.cpd.data,
pathway.id = demo.paths$sel.paths[i], species = "hsa", out.suffix =
"gse16873.cpd", keys.align = "y", kegg.native = TRUE, key.pos = demo.paths$kpos1[i])
str(pv.out)
head(pv.out$plot.data.cpd)

#multiple states in one graph
set.seed(10)
sim.cpd.data2 = matrix(sample(sim.cpd.data, 18000, 
    replace = TRUE), ncol = 6)
pv.out <- pathview(gene.data = gse16873.d[, 1:3], 
    cpd.data = sim.cpd.data2[, 1:2], pathway.id = demo.paths$sel.paths[i], 
    species = "hsa", out.suffix = "gse16873.cpd.3-2s", keys.align = "y", 
    kegg.native = TRUE, match.data = FALSE, multi.state = TRUE, same.layer = TRUE)
str(pv.out)
head(pv.out$plot.data.cpd)

#result PNG file in current directory

##more examples of pathview usages are shown in the vignette.
}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
\keyword{ ~kwd1 }
\keyword{ ~kwd2 }% __ONLY ONE__ keyword per line

\name{cpd.accs}
\alias{cpd.accs}
\alias{cpd.names}
\alias{kegg.met}
\alias{ko.ids}
\alias{rn.list}
\alias{gene.idtype.list}
\alias{gene.idtype.bods}
\alias{cpd.simtypes}
\docType{data}
\title{
  Mapping data between compound or gene IDs and KEGG accessions
}
\description{
  Mapping data between compound or gene IDs and KEGG accessions
}
\usage{
data(cpd.accs)
data(cpd.names)
data(kegg.met)
data(ko.ids)
data(rn.list)
data(gene.idtype.list)
data(gene.idtype.bods)
data(cpd.simtypes)
}
\format{
  cpd.accs is a data frame with 30054 observations on the following 4 variables.
  cpd.names is a data frame with 12314 observations on the following 5
  variables.
  kegg.met is a character matrix of 694 rows and 3 columns.
  ko.ids is a character vector 8511 KEGG ortholog gene IDs, as used in
  KEGG ortholog pathways.
  rn.list is a namedlist of 21 vectors. Each vector records the row numbers
  for one of 21 dfferent compound ID types in cpd.accs data.frame.
  gene.idtype.list is a character vector of 13 common gene, transcript
  or protein ID types. Note some ID types are species specific, for
  example TAIR or ORF.
  gene.idtype.bods is a list of character vectors ofcommon gene, transcript
  or protein ID types for the 19 major research species in bods. Each
  element corresponds to a species.
  cpd.simtypes is a character vector of 7 common compound related ID
  types, each of them has over 1000 unique entries. Hence these ID types
  are good for generating simulation compound data.
}
\source{
  ftp://ftp.ebi.ac.uk/pub/databases/chebi/Flat_file_tab_delimited/

  http://www.genome.jp/kegg-bin/get_htext?br08001.keg
}
\examples{
data(cpd.accs)
data(rn.list)
names(rn.list)
cpd.accs[rn.list[[1]][1:4],]
lapply(rn.list[1:4], function(rn) cpd.accs[rn[1:4],])

data(kegg.met)
head(kegg.met)
}
\keyword{datasets}

\name{node.color}
\alias{node.color}
\alias{col.key}

\title{
Code molecular data as pseudo colors on the pathway graph
}
\description{


\code{node.color} converts the mapped molecular (gene, protein
or metabolite etc) data as pseudo colors on pathway nodes.
\code{col.key} draws color key(s) for mapped molecular data on the
pathway graph.
}
\usage{
node.color(plot.data = NULL, discrete=FALSE, limit, bins, both.dirs =
TRUE, low = "green", mid = "gray", high = "red", na.col = "transparent",
trans.fun = NULL)
col.key(discrete=FALSE, limit = 1, bins = 10, cols = NULL, both.dirs =
TRUE, low = "green", mid = "gray", high = "red", graph.size, node.size,
size.by.graph = TRUE, key.pos = "topright", off.sets = c(x = 0, y = 0),
align = "n", cex = 1, lwd = 1)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{plot.data}{
  the result returned by \code{node.map} function. It is a data.frame composed
  of parsed KGML data and summary molecular data
  for each mapped node. Rows are mapped nodes, and columns are
  parsed or mapped node data. Check \code{node.map} for details.
}
  \item{discrete}{
logical, whether to treat the molecular data or node summary data as discrete. d
  discrete=FALSE, otherwise, mol.data will be a charactor vector of
  molecular IDs.
}
  \item{limit}{
a list of two numeric elements with "gene" and "cpd" as the names. This
argument specifies the limit values for gene.data and cpd.data when converting
them to pseudo colors. Each element of the list could be of length 1 or
2. Length 1 suggests discrete data or 1 directional (positive-valued)
data, or the absolute limit for 2 directional data. Length 2 suggests 2
directional data. Default limit=list(gene=0.5, cpd=1).
}
  \item{bins}{
a list of two integer elements with "gene" and "cpd" as the names. This
argument specifies the number of levels or bins for gene.data and cpd.data when converting
them to pseudo colors. Default limit=list(gene=10, cpd=10).
}
  \item{both.dirs}{
a list of two logical  elements with "gene" and "cpd" as the names. This
argument specifies whether gene.data and cpd.data are 1 directional or 2
directional data when converting them to pseudo colors. Default limit=list(gene=TRUE, cpd=TRUE).
}
  \item{trans.fun}{
a list of two function (not character) elements with "gene" and "cpd" as the names. This
argument specifies whether and how gene.data and cpd.data are
transformed. Examples are \code{log}, \code{abs} or users' own
functions. Default limit=list( gene=NULL, cpd=NULL).
}
  \item{low, mid, high}{
each is a list of two colors with "gene" and "cpd" as the names. This
argument specifies the color spectra to code gene.data and
cpd.data. When data are 1 directional (TRUE value in both.dirs), only
mid and high are used to specify the color spectra. Default spectra (low-mid-high)
"green"-"gray"-"red" and "blue"-"gray"-"yellow" are used for gene.data
and cpd.data respectively. 
The values for 'low, mid, high' can be given as color names
('red'), plot color index (2=red), and HTML-style RGB,
("\#FF0000"=red).
}
  \item{na.col}{
color used for NA's or missing values in gene.data and cpd.data. d
na.col="transparent". 
}

  \item{cols}{
character, specifying a discrete spectrum of colors to be plotted as
color key. Note this argument is usually NULL (default), otherwise, the
number of discrete colors has to match \code{bins}.
}
  \item{graph.size}{
numeric vector of length 2, i.e. the sizes (width, height) of the
pathway graph panel. This is needed to determine the sizes and exact
location of the color key. 
}
  \item{node.size}{
numeric vector of length 2, i.e. the sizes (width, height) of the
standard gene nodes (rectangles). This is needed to determine the sizes
and exact location of the color key when size.by.graph=FALSE. 
}
  \item{size.by.graph}{
logical, whether to determine the sizes and exact location of the color
key with respect to the size of the whole graph panel or that of a
single node. Default size.by.graph=TRUE. 
}
  \item{key.pos}{
character, controlling the position of color key(s). Potentail values
  are "bottomleft", "bottomright", "topleft" and "topright". d
  key.pos="topright". 
}
  \item{off.sets}{
numeric vector of length 2, with "x" and "y" as the names. This argument
specifies the offset values in x and y axes when plotting a new color
key, as to avoid overlap with existing color keys or boundaries. Note
that the \code{off.sets} value is reset and returned each time
\code{col.key} function is called, as for the reference of plotting the
next color key. Default off.sets=c(0,0).
}
  \item{align}{
character, controlling how the color keys are aligned when
needed. Potential values are "x", aligned by x coordinates, and "y",
aligned by y coordinates. Default align="x". 
}
  \item{cex}{
    A numerical value giving the amount by which legend text
    and symbols should be scaled relative to the default 1. 
}
  \item{lwd}{
 numeric, the line width, a _positive_ number, defaulting to '1'. 
}
}
\details{
\code{node.color} converts the mapped molecular data (gene.data or cpd.data) by
node.map function into pseudo colors, which then can be plotted on the
pathway graph.
\code{col.key} is used in combination with node.color in pathview, although
this function can be used independently for similar tasks.
}
\value{
\code{node.color} returns a vector or matrix of colors. Its
dimensionality is the same as the corresponding gene.data or cpd.data.
\code{col.key} plots a color key on existing pathway graph, then returns
a updated version of off.sets for the reference of next color key.
}
\references{
  Luo, W. and Brouwer, C., Pathview: an R/Bioconductor package for
  pathway based data integration and visualization. Bioinformatics,
 2013, 29(14): 1830-1831, doi: 10.1093/bioinformatics/btt285
}
\author{
  Weijun Luo <luo_weijun@yahoo.com>
}

\seealso{
  \code{\link{keggview.native}} and \code{\link{keggview.graph}} the
  viwer functions,
  \code{\link{node.map}} the node data mapper function.
}
\examples{
xml.file=system.file("extdata", "hsa04110.xml", package = "pathview")
node.data=node.info(xml.file)
names(node.data)
data(gse16873.d)
plot.data.gene=node.map(mol.data=gse16873.d[,1], node.data,
  node.types="gene")
head(plot.data.gene)
cols.ts.gene=node.color(plot.data.gene, limit=1, bins=10)
head(cols.ts.gene)
}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
\keyword{ ~kwd1 }
\keyword{ ~kwd2 }% __ONLY ONE__ keyword per line

\name{kegg.species.code}
\alias{kegg.species.code}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{
  Mapping species name to KEGG code
}
\description{
  This function maps species name to KEGG code.
}
\usage{
kegg.species.code(species = "hsa", na.rm = FALSE, code.only = TRUE)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{species}{
character, either the KEGG code, scientific name or the common name of
the target species. Default species="hsa", it is equivalent to use either
"Homo sapiens" (scientific name) or "human" (common name). 
}
  \item{na.rm}{
logical, should unmapped entris be removed. Default na.rm = FALSE.
}
  \item{code.only}{
logical, whether to extract KEGG species code only or with gene ID usage
info too. Default , code.only = TRUE.
}
}
\value{
  a character vector of mapped KEGG code of species.
}
\references{
  Luo, W. and Brouwer, C., Pathview: an R/Bioconductor package for
  pathway based data integration and visualization. Bioinformatics,
 2013, 29(14): 1830-1831, doi: 10.1093/bioinformatics/btt285
}
\author{
  Weijun Luo <luo_weijun@yahoo.com>
}
\seealso{
  \code{\link{korg}} the species and KEGG code mapping data,
  \code{\link{cpd2kegg}} etc the auxillary compound ID mappers,
  \code{\link{download.kegg}} the downloader function.
}
\examples{
species=c("ptr", "Mus musculus", "dog", "happ")
kcode=kegg.species.code(species = species, na.rm = FALSE)
print(kcode)
}
\keyword{ ~kwd1 }
\keyword{ ~kwd2 }% __ONLY ONE__ keyword per line

\name{sim.mol.data}
\alias{sim.mol.data}

\title{
Simulate molecular data for pathview experiment
}
\description{
The molecular data simulator generates either gene.data or cpd.data of
different ID types, molecule numbers, sample sizes, either
continuous or discrete.
}
\usage{
sim.mol.data(mol.type = c("gene", "gene.ko", "cpd")[1], id.type = NULL,
species="hsa", discrete = FALSE, nmol = 1000, nexp = 1, rand.seed=100)
}

\arguments{
  \item{mol.type}{
character of length 1, specifing the molecular type, either "gene" (including
transcripts, proteins), or "gene.ko" (KEGG ortholog genes, as defined in
  KEGG ortholog pathways), or "cpd" (including metabolites, glycans,
  drugs). Note that KEGG ortholog gene are considered "gene" in function
  \code{pathview}. Default mol.type="gene".
}
  \item{id.type}{
    character of length 1, the molecular ID type. When mol.type="gene",
  proper ID types include "KEGG" and "ENTREZ" (Entrez Gene). Multiple other ID
  types are also valid When species is among 19 major species fully
  annotated in Bioconductor, e.g. "hsa" (human), "mmu" (mouse) etc, check:

    \code{data(gene.idtype.bods); gene.idtype.bods} for
  other valid ID types. When mol.type="cpd", check \code{data(cpd.simtypes);
  cpd.simtypes} for valid ID types. Default id.type=NULL, then "Entrez" and
  "KEGG COMPOUND accession" will be assumed for mol.type = "gene" or
  "cpd".
}
  \item{species}{
character, either the kegg code, scientific name or the common name of
the target species. This is only effective when mol.type =
  "gene". Setting species="ko" is equilvalent to
  mol.type="gene.ko". Default species="hsa", equivalent to either "Homo
  sapiens" (scientific name) or "human" (common name). Gene data id.type
  has multiple other choices for 19 major research species, for details
  do: \code{data(gene.idtype.bods); gene.idtype.bods}. When other
  species are specified, gene id.type is limited to "KEGG" and "ENTREZ".
  }
  \item{discrete}{
logical, whether to generate discrete or continuous data. d
  discrete=FALSE, otherwise, mol.data will be a charactor vector of
  molecular IDs.
}
  \item{nmol}{
integer, the target number of different molecules. Note that the
  specified id.type may not have as many different IDs as nmol. In this
  case, all IDs of id.type are used.
}
  \item{nexp}{
integer, the sample size or the number of columns in the result
  simulated data.
}
  \item{rand.seed}{
numeric of length 1, the seed number to start the random sampling
  process. This argumemnt makes the simulation reproducible as long as
  its value keeps the same. Default rand.seed=100. 
}
}
\details{
This function is written mainly for simulation or experiment with
  pathview package. With the simulated molecular data, you may check
  whether and how pathview works for molecular data of different types,
  IDs, format or sample sizes etc. You may also generate both gene.data
  and cpd.data and check data pathway based integration with pathview.
}
\value{
    either vector (single sample) or a matrix-like data (multiple
  sample), depends on the value of \code{nexp}. Vector should be numeric
  with molecular IDs as names or it may also be character of molecular
  IDs depending on the value of \code{discrete}. Matrix-like data structure has molecules as
  rows and samples as columns. Row names should be molecular IDs.

  This returned data can be used directly as gene.data or cpd.data
  input of \code{pathview} main function.
}
\references{
  Luo, W. and Brouwer, C., Pathview: an R/Bioconductor package for
  pathway based data integration and visualization. Bioinformatics,
 2013, 29(14): 1830-1831, doi: 10.1093/bioinformatics/btt285
}
\author{
  Weijun Luo <luo_weijun@yahoo.com>
}

\seealso{
  \code{\link{node.map}} the node data mapper function.
  \code{\link{mol.sum}} the auxillary molecular data mapper,
  \code{\link{id2eg}}, \code{\link{cpd2kegg}} etc the auxillary molecular ID mappers,
  \code{\link{pathview}} the main function,
}
\examples{
#continuous compound data
cpd.data.c=sim.mol.data(mol.type="cpd", nmol=3000)
#discrete compound data
cpd.data.d=sim.mol.data(mol.type="cpd", nmol=3000, discrete=TRUE)
head(cpd.data.c)
head(cpd.data.d)
#continuous compound data named with "CAS Registry Number"
cpd.cas <- sim.mol.data(mol.type = "cpd", id.type = "CAS Registry Number", nmol = 10000)

#gene data with two samples
gene.data.2=sim.mol.data(mol.type="gene", nmol=1000, nexp=2)
head(gene.data.2)

#KEGG ortholog gene data
ko.data=sim.mol.data(mol.type="gene.ko", nmol=5000)
}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
\keyword{ ~kwd1 }
\keyword{ ~kwd2 }% __ONLY ONE__ keyword per line

\name{pathview-package}
\alias{pathview-package}
\docType{package}
\title{
  Pathway based data integration and visualization
}
\description{
  Pathway based data integration and visualization
}
\details{
\tabular{ll}{
Package: \tab pathview\cr
Type: \tab Package\cr
Version: \tab 1.0\cr
Date: \tab 2012-12-26\cr
License: \tab What license is it under?\cr
LazyLoad: \tab yes\cr
}
~~ An overview of how to use the package, including the most important ~~
~~ functions ~~
}
\author{
  Weijun Luo <luo_weijun@yahoo.com>

Maintainer: Weijun Luo <luo_weijun@yahoo.com>
}
\references{
  Luo, W. and Brouwer, C., Pathview: an R/Bioconductor package for
  pathway based data integration and visualization. Bioinformatics,
 2013, 29(14): 1830-1831, doi: 10.1093/bioinformatics/btt285
}
\keyword{package}

\name{download.kegg}
\alias{download.kegg}

\title{
Download KEGG pathway graphs and associated KGML data
}
\description{
This is the downloader function for KEGG pathways, automatically
download graph images and associated KGML data.
}
\usage{
download.kegg(pathway.id = "00010", species = "hsa", kegg.dir = ".",
file.type=c("xml", "png"))
}

\arguments{
  \item{pathway.id}{
character, 5-digit KEGG pathway IDs. Default pathway.id="00010".
}
  \item{species}{
character, either the KEGG code, scientific name or the common name of
the target species. When KEGG ortholog pathway is considered,
species="ko". Default species="hsa", it is equivalent to use either
"Homo sapiens" (scientific name) or "human" (common name). 
}
  \item{kegg.dir}{
character, the directory of KEGG pathway data file (.xml) and image file
(.png). Default kegg.dir="." (current working directory).
}
  \item{file.type}{
character, the file type(s) to be downloaded, either KEGG pathway data
file (xml) or image file (png). Default include both types. 
}
}
\details{
  Species can be specified as either kegg code, scientific name or the
  common name. Scientific name and the common name are always mapped to
  kegg code first.
  Length of species should be either 1 or the same as pathway.id, if
  not, the same set of pathway.id will be applied to all species.
}
\value{
a named character vector, either "succeed" or "failed", indicating the
download  status of corresponding pathways.
}
\references{
  Luo, W. and Brouwer, C., Pathview: an R/Bioconductor package for
  pathway based data integration and visualization. Bioinformatics,
 2013, 29(14): 1830-1831, doi: 10.1093/bioinformatics/btt285
}
\author{
  Weijun Luo <luo_weijun@yahoo.com>
}


\seealso{
  \code{\link{pathview}} the main function,
  \code{\link{node.info}} the parser,
}
\examples{
data(demo.paths)
sel.2paths=demo.paths$sel.paths[1:2]
download.kegg(pathway.id = sel.2paths, species = "hsa")
#pathway files should be downloaded into current working directory
}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
\keyword{ ~kwd1 }
\keyword{ ~kwd2 }% __ONLY ONE__ keyword per line

\name{wordwrap}
\alias{wordwrap}
\alias{strfit}

\title{
  Wrap or break strings into lines of specified width
}
\description{
  strfit does hard wrapping, i.e. break within long words, wordwrap is a
  wrapper of strfit but also provides soft wrapping option, i.e. break
  only between words, and keep long words intact.
}
\usage{
wordwrap(s, width = 20, break.word = FALSE)
strfit(s, width = 20)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{s}{
    characcter, strings to be wrapped or broken down.
  }
  \item{width}{
    integer, target line width in terms of number of characters. d
    width=20.
  }
  \item{break.word}{
    logical, whether to break within words or only between words as to fit
    the line width. Default break.word=FALSE, i.e. keep words intact and only
    break between words. Therefore, some line may exceed the \code{width}
    limit.
  }
}
\details{
These functions are called as to wrap long node labels into shorter
  lines on pathway graphs in \code{keggview.graph} function (when
  keggview.native=FALSE). They are equally useful for wrapping long
  labels in other types of graphs or output formats.
}
\value{
character of the same length of \code{s} except that each element has
been wrapped softly or hardly.
}
\references{
  Luo, W. and Brouwer, C., Pathview: an R/Bioconductor package for
  pathway based data integration and visualization. Bioinformatics,
 2013, 29(14): 1830-1831, doi: 10.1093/bioinformatics/btt285
}
\author{
  Weijun Luo <luo_weijun@yahoo.com>
}

\seealso{
\code{strwrap} in R base. 
}
\examples{
long.str="(S)-Methylmalonate semialdehyde"
wr1=wordwrap(long.str, width=15)
#long word intact
cat(wr1, sep="\n")
wr2=strfit(long.str, width=15)
#long word split
cat(wr2, sep="\n")
}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
\keyword{ ~kwd1 }
\keyword{ ~kwd2 }% __ONLY ONE__ keyword per line

\name{demo.data}
\alias{demo.paths}
\alias{gse16873.d}
\alias{paths.hsa}
\docType{data}
\title{
  Data for demo purpose
}
\description{
demo.paths includes pathway ids and optimal plotting parameters when
calling pathview.

GSE16873 is a breast cancer study (Emery et al, 2009) downloaded from
Gene Expression Omnibus (GEO). Dataset gse16873 is pre-processed using FARMS
method and includes 6 patient cases,
each with HN (histologically normal) and DCIS (ductal carcinoma in situ)
 RMA samples. The same dataset is also used in \code{gage}
 package. Dataset gse16873.d includes the gene expression changes of two
 pairs of DCIS vs HN samples.

paths.hsa includes the full list of human pathway ID/names from KEGG.
}
\usage{
data(demo.paths)
data(gse16873.d)
data(paths.hsa)
}
\format{
  demo.paths is a named list with ids and plotting parameters for 3
  pathways. For details do:

  \code{data(demo.paths); demo.paths}
  
  gse16873.d is a numeric matrix with over 10000 rows (genes) and 2
  columns (samples). For details do:
  \code{data(gse16873.d); str(gse16873.d)}.

  paths.hsa is a named vector mapping KEGG pathway ID to human pathway names.
}
\source{
  http://www.ncbi.nlm.nih.gov/geo/query/acc.cgi?acc=GSE16873
}
\keyword{datasets}

\name{eg2id}
\alias{eg2id}
\alias{id2eg}
\alias{geneannot.map}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{
Mapping between different gene ID and annotation types
}
\description{
These auxillary gene ID mappers connect different gene ID or annotation
types, especially they are used to map Entrez Gene ID to external gene,
transcript or protein IDs or vise versa. 
}
\usage{
eg2id(eg, category = gene.idtype.list[1:2], org = "Hs", pkg.name = NULL,
...)
id2eg(ids, category = gene.idtype.list[1], org = "Hs", pkg.name = NULL, ...)
geneannot.map(in.ids, in.type, out.type, org="Hs", pkg.name=NULL,
unique.map=TRUE, na.rm=TRUE, keep.order=TRUE)
}
\arguments{
  \item{eg}{
character, input Entrez Gene IDs.
}
  \item{ids}{
character, input gene/transcript/protein IDs to be converted to Entrez
Gene IDs.
}
  \item{in.ids}{
character, input gene/transcript/protein IDs to be converted or mapped
to other Gene IDs or annotation types.
}
  \item{category}{
character, for \code{eg2id} the output ID types to map from Entrez Gene,
d to be c("SYMBOL", "GENENAME"); for \code{id2eg}, the input ID type  to
be mapped to Entrez Gene, default to be "SYMBOL".
}
  \item{in.type}{
character, the input gene/transcript/protein ID type  to be mapped or
converted to other ID/annotation types. 
}
  \item{out.type}{
character, the output gene/transcript/protein ID type  to be mapped or
converted to other ID/annotation types. 
}
  \item{org}{
character, the two-letter abbreviation of organism name, or KEGG species
code, or the common species name, used to determine the gene annotation
package. For all potential values check:  \code{data(bods);
  bods}. Default org="Hs", and can also be "hsa" or "human" (case
insensitive). Only effective when pkg.name is not NULL.
}
  \item{pkg.name}{
character, name of the gene annotation package. This package should be
one of the standard annotation packages from Bioconductor, such as
"org.Hs.eg.db". Check \code{data(bods); bods} for a full list of standard
annotation packages. You may also use your custom annotation package
built with AnnotationDbi, the Bioconductor Annotation Database
Interface. Default pkg.name=NULL, hence argument \code{org} should be
specified.
}
  \item{unique.map}{
logical, whether to combine multiple entries mapped to the same input ID
as a single entry (separted by "; "). Default unique.map=TRUE.
}
  \item{na.rm}{
logical, whether to remove the lines where input ID is not mapped (NA
for mapped entries). Default na.rm=TRUE.
}
  \item{keep.order}{
logical, whether to keep the original input order even with all unmapped
input IDs. Default keep.order=TRUE.
}
  \item{\dots}{
    other arguments to be passed to geneannot.map function.
  }
}
\details{
KEGG uses Entrez Gene ID as its standard gene ID. Therefore, all gene
data need to be mapped to Entrez Genes when working with KEGG
pathways. Function \code{id2eg} does this mapping. On the other hand, we
frequently want to check or show gene symbols or full names instead of
the less informative Entrez Gene ID when working with KEGG gene nodes,
Function \code{eg2id} does this reverse mapping. Both \code{id2eg} and
\code{eg2id} are wrapper functions of \code{geneannot.map} function. The
latter can be used to map between a range of major
gene/transcript/protein IDs or annotation types, not just Entrez Gene ID.
These functions are written as part of the Pathview mapper module, they
are equally useful for other gene ID or data mapping tasks.
The use of these functions depends on gene annotation packages like
"org.Hs.eg.db", which are Bioconductor standard. IFf no such packages not available for
your interesting organisms, you may build one with Bioconductor
AnnotationDbi package.
}
\value{
a 2- or multi-column character matrix recording the mapping between input IDs to
the target ID type(s).
}
\references{
  Luo, W. and Brouwer, C., Pathview: an R/Bioconductor package for
  pathway based data integration and visualization. Bioinformatics,
 2013, 29(14): 1830-1831, doi: 10.1093/bioinformatics/btt285
}
\author{
  Weijun Luo <luo_weijun@yahoo.com>
}
\seealso{
  \code{\link{cpd2kegg}} etc the auxillary compound ID mappers,
  \code{\link{mol.sum}} the auxillary molecular data mapper,
  \code{\link{node.map}} the node data mapper function.
}
\examples{
data(gene.idtype.list)
#generate simulated gene data named with non-KEGG/Entrez gene IDs
gene.ensprot <- sim.mol.data(mol.type = "gene", id.type = gene.idtype.list[4], 
    nmol = 50000)
#construct map between non-KEGG ID and KEGG ID (Entrez gene)
id.map.ensprot <- id2eg(ids = names(gene.ensprot), 
    category = gene.idtype.list[4], org = "Hs")
#Map molecular data onto Entrez Gene IDs
gene.entrez <- mol.sum(mol.data = gene.ensprot, id.map = id.map.ensprot)
#check the results
head(gene.ensprot)
head(id.map.ensprot)
head(gene.entrez)

#map Entrez Gene to Gene Symbol and Name
eg.symbname=eg2id(eg=id.map.ensprot[,2])
#entries with more than 1 Entrez Genes are not mapped
head(eg.symbname)

#not run: map between other ID types for other species
#ath.tair=sim.mol.data(id.type="tair", species="ath", nmol=1000)
#data(gene.idtype.bods)
#gid.map <-geneannot.map(in.ids=names(ath.tair)[rep(1:100,each=2)],
#in.type="tair", out.type=gene.idtype.bods$ath[-1], org="At")
#gid.map1 <-geneannot.map(in.ids=names(ath.tair)[rep(1:100,each=2)],
#in.type="tair", out.type=gene.idtype.bods$ath[-1], org="At",
#unique.map=F, keep.order=F)
#str(gid.map)
#str(gid.map1)
}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
\keyword{ ~kwd1 }
\keyword{ ~kwd2 }% __ONLY ONE__ keyword per line

\name{combineKEGGnodes}
\alias{combineKEGGnodes}
\alias{reaction2edge}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{
  Special treatment of nodes or edges for KEGG pathway rendering
}
\description{
  \code{combineKEGGnodes} combines nodes into a group in a KEGG pathway
  graph. 
  \code{reaction2edge} converts reactions into edges in KEGG pathway
  graph.
}
\usage{
combineKEGGnodes(nodes, graph, combo.node)
reaction2edge(path, gR)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{nodes}{
character, names of the names to be combined.
}
  \item{graph, gR}{
a object of "graphNEL" class, the graph parsed and converted from KEGG
pathway.
}
  \item{path}{
a object of "KEGGPathway" class, the parsed  KEGG pathway.
}
  \item{combo.node}{
character, the name of result combined node.
}
}
\details{
  \code{combineKEGGnodes} not only combines nodes in the graph object,
  but also corresponding node data in the KEGG pathway object. This
  function is needed for KEGG-defined group nodes and parsed enzyme
  groups involved in the same reaction.
  \code{reaction2edge} converts a reaction into 2 consecutive edges
  between substrate and enzyme and enzyme and product. This function is
  needed as to faithfully show the compound-enzyme nodes and their
  interactions in Graphviz-style view of KEGG pathway.
}
\value{
    The results returned by \code{combineKEGGnodes} is a combined graph
    of "graphNEL" class.
  The results returned by \code{reaction2edge} is a list of 3
  elements: \code{gR}, the converted graph ("graphNEL"); edata.new, the
  new edge data ("KEGGEdge"); ndata.new, the new node data ("KEGGNode").
}
\references{
  Luo, W. and Brouwer, C., Pathview: an R/Bioconductor package for
  pathway based data integration and visualization. Bioinformatics,
 2013, 29(14): 1830-1831, doi: 10.1093/bioinformatics/btt285
}
\author{
  Weijun Luo <luo_weijun@yahoo.com>
}

\seealso{
  \code{\link{node.info}} the main parser function
}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
\keyword{ ~kwd1 }
\keyword{ ~kwd2 }% __ONLY ONE__ keyword per line

\name{mol.sum}
\alias{mol.sum}

\title{
Mapping and summation of molecular data onto standard IDs
}
\description{
Molecular data like gene or metabolite data are frequently annotated by
various types of IDs. This function maps and summarize molecular data
onto standard gene or compound IDs. It would be straightforward to
integrate, analyze or visualize the "standardized" data with pathways or
functional categories.
}
\usage{
mol.sum(mol.data, id.map, gene.annotpkg = "org.Hs.eg.db", sum.method =
c("sum", "mean", "median", "max", "max.abs", "random")[1])
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{mol.data}{
  Either vector (single sample) or a matrix-like data (multiple
  sample). Vector should be numeric with molecule IDs as names or it
  may also be character of molecule IDs. Character vector is treated as
  discrete or count data. Matrix-like data structure has molecules as
  rows and samples as columns. Row names should be molecule IDs. Default
  mol.data=NULL. 
  This argument is equivalent to gene.data or cpd.data in the pathview
  function. Check pahtview function for more information.
}
  \item{id.map}{
a two-column character matrix, giving the mapping between molecular IDs
used in mol.data and taget/standard molecular IDs. Then mol.data are
gene data, \code{id.map} may also be a character specifying the type of
IDs used in mol.data. The two-column mapping matrix will be generated
automatically.
}
  \item{gene.annotpkg}{
character, name of the gene annotation package. This package should be
one of the standard annotation packages from Bioconductor, such as
"org.Hs.eg.db" (default). Check \code{data(bods); bods} for a full list of standard
annotation packages. You may also use your custom annotation package
built with AnnotationDbi, the Bioconductor Annotation Database
Interface. Only effective when mol.data are gene.data and id.map gives
the ID type being used.
}
  \item{sum.method}{
  character, the method name to calculate node summary given that
  multiple genes or compounds are mapped to it. Poential options include
  "sum","mean", "median", "max", "max.abs" and "random". Default sum.method="sum".
}
}
\details{
This function is called in pathview main function when gene.idtype or
cpd.idtype is not the standard type, so that the molecular data can be
mapped and summarized onto standard IDs. This is needed for further
mapping to KEGG pathways. The same standard ID mapping is needed when
carry out pathway or functional analysis on molecular data, which are
labeled by non-standard (or alien) IDs or probe names, like in most of
the microarray or metabolomics datasets. In other words, function
\code{mol.sum} can be useful in all these situations.
}
\value{
a numeric vector or matrix. Its dimensionality is the same as the input
mol.data except row names are standard molecular IDs.
}
\references{
  Luo, W. and Brouwer, C., Pathview: an R/Bioconductor package for
  pathway based data integration and visualization. Bioinformatics,
 2013, 29(14): 1830-1831, doi: 10.1093/bioinformatics/btt285
}
\author{
  Weijun Luo <luo_weijun@yahoo.com>
}


\seealso{
  \code{\link{node.map}} the node data mapper function.
  \code{\link{id2eg}}, \code{\link{cpd2kegg}} etc the auxillary molecular ID mappers,
  \code{\link{pathview}} the main function,
}
\examples{
data(gene.idtype.list)
#generate simulated gene data named with non-KEGG/Entrez gene IDs
gene.ensprot <- sim.mol.data(mol.type = "gene", id.type = gene.idtype.list[4], 
    nmol = 50000)
#construct map between non-KEGG ID and KEGG ID (Entrez gene)
id.map.ensprot <- id2eg(ids = names(gene.ensprot), 
    category = gene.idtype.list[4], org = "Hs")
#Map molecular data onto Entrez Gene IDs
gene.entrez <- mol.sum(mol.data = gene.ensprot, id.map = id.map.ensprot)
#check the results
head(gene.ensprot)
head(id.map.ensprot)
head(gene.entrez)
}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
\keyword{ ~kwd1 }
\keyword{ ~kwd2 }% __ONLY ONE__ keyword per line

\name{pathview-internal}

\alias{KEGGEdgeSubtype}
\alias{parseReaction2}
\alias{subtypeDisplay.kedge}
\alias{KEGGpathway2Graph2}
\alias{parseKGML2.R}
\alias{parseKGML2Graph2}
\alias{max.abs}
\alias{random}
\alias{kegg.legend}
\alias{pathview.stamp}
\alias{circles}
\alias{ellipses}
\alias{sliced.shapes}
\alias{colorpanel2}

\title{Internal functions}
\description{
  Not intended to be called by the users.
}
\details{
  These functions are not to be called by the user directly.

  Functions parseReaction2, parseKGML2,  KEGGpathway2Graph2 and
  parseKGML2Graph2 parse KEGG pathways
  from KGML files. Function subtypeDisplay.kedge and data
  KEGGEdgeSubtype extact and store edge subtypes and corresponding
  rendering information. All these functions/data were modified from the
  original copies in KEGGgraph package.

  Function kegg.legend generates legend for KEGG edge and node
  types. Function pathview.stamp generates pathview sisgnature on graphs.
  
  Function colorpanel2 comes from gplots package function colorpanel.
  
  Functions max.abs and random among others are method to summarize data
  at molecular level or node level when multiple items mapping to the
  same ID/node. 

  Function circles, ellipses and sliced.shapes draw KEGG nodes in colored shapes
  (circles and ellipses).
  
  Functions deComp and rownorm were written by Weijun Luo, the author of
  gage package.
}
\keyword{ internal }
